/*
 * Copyright (c) 2013 Turi Inc.
 *     All rights reserved.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
 * For more about this software visit:
 *
 *      http://turicreate.com
 *
 */

#define BOOST_TEST_MODULE
#include <boost/test/unit_test.hpp>
#include <util/test_macros.hpp>
#include <iostream>
#include <typeinfo>       // operator typeid
#include <cstddef>
#include <functional>
#include <iterator>
#include <type_traits>
#include <iterator>


#include <generics/gl_string.hpp>
#include <util/testing_utils.hpp>

using namespace turi;

struct test_string_modifiers  {
 public:

  void test_basic_append() {
    std::string s("123");
    s.append({'a', 'b', 'c'});
    TS_ASSERT(s == "123abc");
  }

  template <class It>
  void _test_string_append_iterator(gl_string s, It first, It last, gl_string expected) {
    s.append(first, last);
    TS_ASSERT(s == expected);
  }

  void test_string_append_iterator()  {
    const char* s = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
    _test_string_append_iterator(gl_string(), s, s, gl_string());
    _test_string_append_iterator(gl_string(), s, s+1, gl_string("A"));
    _test_string_append_iterator(gl_string(), s, s+10, gl_string("ABCDEFGHIJ"));
    _test_string_append_iterator(gl_string(), s, s+52, gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_append_iterator(gl_string("12345"), s, s, gl_string("12345"));
    _test_string_append_iterator(gl_string("12345"), s, s+1, gl_string("12345A"));
    _test_string_append_iterator(gl_string("12345"), s, s+10, gl_string("12345ABCDEFGHIJ"));
    _test_string_append_iterator(gl_string("12345"), s, s+52, gl_string("12345ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_append_iterator(gl_string("1234567890"), s, s, gl_string("1234567890"));
    _test_string_append_iterator(gl_string("1234567890"), s, s+1, gl_string("1234567890A"));
    _test_string_append_iterator(gl_string("1234567890"), s, s+10, gl_string("1234567890ABCDEFGHIJ"));
    _test_string_append_iterator(gl_string("1234567890"), s, s+52, gl_string("1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_append_iterator(gl_string("12345678901234567890"), s, s, gl_string("12345678901234567890"));
    _test_string_append_iterator(gl_string("12345678901234567890"), s, s+1, gl_string("12345678901234567890""A"));
    _test_string_append_iterator(gl_string("12345678901234567890"), s, s+10, gl_string("12345678901234567890""ABCDEFGHIJ"));
    _test_string_append_iterator(gl_string("12345678901234567890"), s, s+52,
                                 gl_string("12345678901234567890""ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_append_iterator(gl_string(), (const char*)(s), (const char*)(s), gl_string());
    _test_string_append_iterator(gl_string(), (const char*)(s), (const char*)(s+1), gl_string("A"));
    _test_string_append_iterator(gl_string(), (const char*)(s), (const char*)(s+10),
                                 gl_string("ABCDEFGHIJ"));
    _test_string_append_iterator(gl_string(), (const char*)(s), (const char*)(s+52),
                                 gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_append_iterator(gl_string("12345"), (const char*)(s), (const char*)(s),
                                 gl_string("12345"));
    _test_string_append_iterator(gl_string("12345"), (const char*)(s), (const char*)(s+1),
                                 gl_string("12345A"));
    _test_string_append_iterator(gl_string("12345"), (const char*)(s), (const char*)(s+10),
                                 gl_string("12345ABCDEFGHIJ"));
    _test_string_append_iterator(gl_string("12345"), (const char*)(s), (const char*)(s+52),
                                 gl_string("12345ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_append_iterator(gl_string("1234567890"), (const char*)(s), (const char*)(s),
                                 gl_string("1234567890"));
    _test_string_append_iterator(gl_string("1234567890"), (const char*)(s), (const char*)(s+1),
                                 gl_string("1234567890A"));
    _test_string_append_iterator(gl_string("1234567890"), (const char*)(s), (const char*)(s+10),
                                 gl_string("1234567890ABCDEFGHIJ"));
    _test_string_append_iterator(gl_string("1234567890"), (const char*)(s), (const char*)(s+52),
                                 gl_string("1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_append_iterator(gl_string("12345678901234567890"), (const char*)(s), (const char*)(s),
                                 gl_string("12345678901234567890"));
    _test_string_append_iterator(gl_string("12345678901234567890"), (const char*)(s), (const char*)(s+1),
                                 gl_string("12345678901234567890""A"));
    _test_string_append_iterator(gl_string("12345678901234567890"), (const char*)(s), (const char*)(s+10),
                                 gl_string("12345678901234567890""ABCDEFGHIJ"));
    _test_string_append_iterator(gl_string("12345678901234567890"), (const char*)(s), (const char*)(s+52),
                                 gl_string("12345678901234567890""ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));
  }
  
  void _test_string_append_pointer(gl_string s, const char* str, gl_string expected) {
    s.append(str);
    TS_ASSERT(s == expected);
  }

  void test_string_append_pointer() {
    
    _test_string_append_pointer(gl_string(), "", gl_string());
    _test_string_append_pointer(gl_string(), "12345", gl_string("12345"));
    _test_string_append_pointer(gl_string(), "12345678901234567890", gl_string("12345678901234567890"));

    _test_string_append_pointer(gl_string("12345"), "", gl_string("12345"));
    _test_string_append_pointer(gl_string("12345"), "12345", gl_string("1234512345"));
    _test_string_append_pointer(gl_string("12345"), "1234567890", gl_string("123451234567890"));

    _test_string_append_pointer(gl_string("12345678901234567890"), "", gl_string("12345678901234567890"));
    _test_string_append_pointer(gl_string("12345678901234567890"), "12345", gl_string("1234567890123456789012345"));
    _test_string_append_pointer(gl_string("12345678901234567890"), "12345678901234567890",
                                gl_string("1234567890123456789012345678901234567890"));
  }
  
  void _test_string_append_pointer_size(gl_string s, const char* str, size_t n, gl_string expected) {
    s.append(str, n);
    TS_ASSERT(s == expected);
  }

  void test_string_append_pointer_size() {
    _test_string_append_pointer_size(gl_string(), "", 0, gl_string());
    _test_string_append_pointer_size(gl_string(), "12345", 3, gl_string("123"));
    _test_string_append_pointer_size(gl_string(), "12345", 4, gl_string("1234"));
    _test_string_append_pointer_size(gl_string(), "12345678901234567890", 0, gl_string());
    _test_string_append_pointer_size(gl_string(), "12345678901234567890", 1, gl_string("1"));
    _test_string_append_pointer_size(gl_string(), "12345678901234567890", 3, gl_string("123"));
    _test_string_append_pointer_size(gl_string(), "12345678901234567890", 20, gl_string("12345678901234567890"));

    _test_string_append_pointer_size(gl_string("12345"), "", 0, gl_string("12345"));
    _test_string_append_pointer_size(gl_string("12345"), "12345", 5, gl_string("1234512345"));
    _test_string_append_pointer_size(gl_string("12345"), "1234567890", 10, gl_string("123451234567890"));

    _test_string_append_pointer_size(gl_string("12345678901234567890"), "", 0, gl_string("12345678901234567890"));
    _test_string_append_pointer_size(gl_string("12345678901234567890"), "12345", 5, gl_string("1234567890123456789012345"));
    _test_string_append_pointer_size(gl_string("12345678901234567890"), "12345678901234567890", 20,
                                     gl_string("1234567890123456789012345678901234567890"));
  }

  void _test_string_append_push_back(gl_string s, char c, gl_string expected) {
    s.push_back(c);
    TS_ASSERT(s == expected);
  }

  void test_string_append_push_back() {
    _test_string_append_push_back(gl_string(), 'a', gl_string(1, 'a'));
    _test_string_append_push_back(gl_string("12345"), 'a', gl_string("12345a"));
    _test_string_append_push_back(gl_string("12345678901234567890"), 'a', gl_string("12345678901234567890a"));
  }
  
  void _test_string_append_size_char(gl_string s, size_t n, char c, gl_string expected) {
    s.append(n, c);
    TS_ASSERT(s == expected);
  }

  void test_string_append_size_char() {
    _test_string_append_size_char(gl_string(), 0, 'a', gl_string());
    _test_string_append_size_char(gl_string(), 1, 'a', gl_string(1, 'a'));
    _test_string_append_size_char(gl_string(), 10, 'a', gl_string(10, 'a'));
    _test_string_append_size_char(gl_string(), 100, 'a', gl_string(100, 'a'));

    _test_string_append_size_char(gl_string("12345"), 0, 'a', gl_string("12345"));
    _test_string_append_size_char(gl_string("12345"), 1, 'a', gl_string("12345a"));
    _test_string_append_size_char(gl_string("12345"), 10, 'a', gl_string("12345aaaaaaaaaa"));

    _test_string_append_size_char(gl_string("12345678901234567890"), 0, 'a', gl_string("12345678901234567890"));
    _test_string_append_size_char(gl_string("12345678901234567890"), 1, 'a', gl_string("12345678901234567890a"));
    _test_string_append_size_char(gl_string("12345678901234567890"), 10, 'a', gl_string("12345678901234567890aaaaaaaaaa"));
  }

  void _test_string_append_string(gl_string s, gl_string str, gl_string expected) {
    s.append(str);
    TS_ASSERT(s == expected);
  }

  void test_string_append_string() {
    _test_string_append_string(gl_string(), gl_string(), gl_string());
    _test_string_append_string(gl_string(), gl_string("12345"), gl_string("12345"));
    _test_string_append_string(gl_string(), gl_string("1234567890"), gl_string("1234567890"));
    _test_string_append_string(gl_string(), gl_string("12345678901234567890"), gl_string("12345678901234567890"));

    _test_string_append_string(gl_string("12345"), gl_string(), gl_string("12345"));
    _test_string_append_string(gl_string("12345"), gl_string("12345"), gl_string("1234512345"));
    _test_string_append_string(gl_string("12345"), gl_string("1234567890"), gl_string("123451234567890"));
    _test_string_append_string(gl_string("12345"), gl_string("12345678901234567890"), gl_string("1234512345678901234567890"));

    _test_string_append_string(gl_string("1234567890"), gl_string(), gl_string("1234567890"));
    _test_string_append_string(gl_string("1234567890"), gl_string("12345"), gl_string("123456789012345"));
    _test_string_append_string(gl_string("1234567890"), gl_string("1234567890"), gl_string("12345678901234567890"));
    _test_string_append_string(gl_string("1234567890"), gl_string("12345678901234567890"), gl_string("123456789012345678901234567890"));

    _test_string_append_string(gl_string("12345678901234567890"), gl_string(), gl_string("12345678901234567890"));
    _test_string_append_string(gl_string("12345678901234567890"), gl_string("12345"), gl_string("1234567890123456789012345"));
    _test_string_append_string(gl_string("12345678901234567890"), gl_string("1234567890"), gl_string("123456789012345678901234567890"));
    _test_string_append_string(gl_string("12345678901234567890"), gl_string("12345678901234567890"),
                               gl_string("1234567890123456789012345678901234567890"));
  }

  void _test_string_append_string_size_size(gl_string s, gl_string str, size_t pos, size_t n, gl_string expected) {
#ifndef NDEBUG
    auto f = [&](){s.append(str, pos, n);};

    if(pos > str.size()) {
      TS_ASSERT_THROWS_ANYTHING(f());
    } else {
      f();
      TS_ASSERT(s == expected);
    }
#endif
  }
  
  void test_string_append_string_size_size_npos(gl_string s, gl_string str, size_t pos, gl_string expected) {
#ifndef NDEBUG
    auto f = [&](){s.append(str, pos);};

    if(pos > str.size()) {
      TS_ASSERT_THROWS_ANYTHING(f());
    } else {
      f();
      TS_ASSERT(s == expected);
    }
#endif
  }
  
  void test_string_append_string_size_size() {
    _test_string_append_string_size_size(gl_string(), gl_string(), 0, 0, gl_string());
    _test_string_append_string_size_size(gl_string(), gl_string(), 1, 0, gl_string());
    _test_string_append_string_size_size(gl_string(), gl_string("12345"), 0, 3, gl_string("123"));
    _test_string_append_string_size_size(gl_string(), gl_string("12345"), 1, 4, gl_string("2345"));
    _test_string_append_string_size_size(gl_string(), gl_string("12345"), 3, 15, gl_string("45"));
    _test_string_append_string_size_size(gl_string(), gl_string("12345"), 5, 15, gl_string(""));
    _test_string_append_string_size_size(gl_string(), gl_string("12345"), 6, 15, gl_string("not happening"));
    _test_string_append_string_size_size(gl_string(), gl_string("12345678901234567890"), 0, 0, gl_string());
    _test_string_append_string_size_size(gl_string(), gl_string("12345678901234567890"), 1, 1, gl_string("2"));
    _test_string_append_string_size_size(gl_string(), gl_string("12345678901234567890"), 2, 3, gl_string("345"));
    _test_string_append_string_size_size(gl_string(), gl_string("12345678901234567890"), 12, 13, gl_string("34567890"));
    _test_string_append_string_size_size(gl_string(), gl_string("12345678901234567890"), 21, 13, gl_string("not happening"));

    _test_string_append_string_size_size(gl_string("12345"), gl_string(), 0, 0, gl_string("12345"));
    _test_string_append_string_size_size(gl_string("12345"), gl_string("12345"), 2, 2, gl_string("1234534"));
    _test_string_append_string_size_size(gl_string("12345"), gl_string("1234567890"), 0, 100, gl_string("123451234567890"));

    _test_string_append_string_size_size(gl_string("12345678901234567890"), gl_string(), 0, 0, gl_string("12345678901234567890"));
    _test_string_append_string_size_size(gl_string("12345678901234567890"), gl_string("12345"), 1, 3, gl_string("12345678901234567890234"));
    _test_string_append_string_size_size(gl_string("12345678901234567890"), gl_string("12345678901234567890"), 5, 10,
                                         gl_string("123456789012345678906789012345"));

    
    test_string_append_string_size_size_npos(gl_string(), gl_string(), 0, gl_string());
    test_string_append_string_size_size_npos(gl_string(), gl_string(), 1, gl_string());
    test_string_append_string_size_size_npos(gl_string(), gl_string("12345"), 0, gl_string("12345"));
    test_string_append_string_size_size_npos(gl_string(), gl_string("12345"), 1, gl_string("2345"));
    test_string_append_string_size_size_npos(gl_string(), gl_string("12345"), 3, gl_string("45"));
    test_string_append_string_size_size_npos(gl_string(), gl_string("12345"), 5, gl_string(""));
    test_string_append_string_size_size_npos(gl_string(), gl_string("12345"), 6, gl_string("not happening"));

  }

  void test_string_assign_initializer_list() {
    std::string s("123");
    s.assign({'a', 'b', 'c'});
    TS_ASSERT(s == "abc");
  }

  template < class It>
  void _test_string_assign_iterator(gl_string s, It first, It last, gl_string expected) {
    s.assign(first, last);
    
    TS_ASSERT(s == expected);
  }

  void test_string_assign_iterator() {
    
    const char* s = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
    _test_string_assign_iterator(gl_string(), s, s, gl_string());
    _test_string_assign_iterator(gl_string(), s, s+1, gl_string("A"));
    _test_string_assign_iterator(gl_string(), s, s+10, gl_string("ABCDEFGHIJ"));
    _test_string_assign_iterator(gl_string(), s, s+52, gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_assign_iterator(gl_string("12345"), s, s, gl_string());
    _test_string_assign_iterator(gl_string("12345"), s, s+1, gl_string("A"));
    _test_string_assign_iterator(gl_string("12345"), s, s+10, gl_string("ABCDEFGHIJ"));
    _test_string_assign_iterator(gl_string("12345"), s, s+52, gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_assign_iterator(gl_string("1234567890"), s, s, gl_string());
    _test_string_assign_iterator(gl_string("1234567890"), s, s+1, gl_string("A"));
    _test_string_assign_iterator(gl_string("1234567890"), s, s+10, gl_string("ABCDEFGHIJ"));
    _test_string_assign_iterator(gl_string("1234567890"), s, s+52, gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_assign_iterator(gl_string("12345678901234567890"), s, s, gl_string());
    _test_string_assign_iterator(gl_string("12345678901234567890"), s, s+1, gl_string("A"));
    _test_string_assign_iterator(gl_string("12345678901234567890"), s, s+10, gl_string("ABCDEFGHIJ"));
    _test_string_assign_iterator(gl_string("12345678901234567890"), s, s+52,
                                 gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_assign_iterator(gl_string(), (const char*)(s), (const char*)(s), gl_string());
    _test_string_assign_iterator(gl_string(), (const char*)(s), (const char*)(s+1), gl_string("A"));
    _test_string_assign_iterator(gl_string(), (const char*)(s), (const char*)(s+10),
                                 gl_string("ABCDEFGHIJ"));
    _test_string_assign_iterator(gl_string(), (const char*)(s), (const char*)(s+52),
                                 gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_assign_iterator(gl_string("12345"), (const char*)(s), (const char*)(s),
                                 gl_string());
    _test_string_assign_iterator(gl_string("12345"), (const char*)(s), (const char*)(s+1),
                                 gl_string("A"));
    _test_string_assign_iterator(gl_string("12345"), (const char*)(s), (const char*)(s+10),
                                 gl_string("ABCDEFGHIJ"));
    _test_string_assign_iterator(gl_string("12345"), (const char*)(s), (const char*)(s+52),
                                 gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_assign_iterator(gl_string("1234567890"), (const char*)(s), (const char*)(s),
                                 gl_string());
    _test_string_assign_iterator(gl_string("1234567890"), (const char*)(s), (const char*)(s+1),
                                 gl_string("A"));
    _test_string_assign_iterator(gl_string("1234567890"), (const char*)(s), (const char*)(s+10),
                                 gl_string("ABCDEFGHIJ"));
    _test_string_assign_iterator(gl_string("1234567890"), (const char*)(s), (const char*)(s+52),
                                 gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_assign_iterator(gl_string("12345678901234567890"), (const char*)(s), (const char*)(s),
                                 gl_string());
    _test_string_assign_iterator(gl_string("12345678901234567890"), (const char*)(s), (const char*)(s+1),
                                 gl_string("A"));
    _test_string_assign_iterator(gl_string("12345678901234567890"), (const char*)(s), (const char*)(s+10),
                                 gl_string("ABCDEFGHIJ"));
    _test_string_assign_iterator(gl_string("12345678901234567890"), (const char*)(s), (const char*)(s+52),
                                 gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));
  }

  
  void _test_string_assign_pointer(gl_string s, const char* str, gl_string expected) {
    s.assign(str);
    
    TS_ASSERT(s == expected);
  }

  void test_string_assign_pointer() {
    _test_string_assign_pointer(gl_string(), "", gl_string());
    _test_string_assign_pointer(gl_string(), "12345", gl_string("12345"));
    _test_string_assign_pointer(gl_string(), "12345678901234567890", gl_string("12345678901234567890"));

    _test_string_assign_pointer(gl_string("12345"), "", gl_string());
    _test_string_assign_pointer(gl_string("12345"), "12345", gl_string("12345"));
    _test_string_assign_pointer(gl_string("12345"), "1234567890", gl_string("1234567890"));

    _test_string_assign_pointer(gl_string("12345678901234567890"), "", gl_string());
    _test_string_assign_pointer(gl_string("12345678901234567890"), "12345", gl_string("12345"));
    _test_string_assign_pointer(gl_string("12345678901234567890"), "12345678901234567890",
                                gl_string("12345678901234567890"));
  }


  void _test_string_assign_pointer_size(gl_string s, const char* str, size_t n, gl_string expected) {
    s.assign(str, n);
    
    TS_ASSERT(s == expected);
  }

  void test_string_assign_pointer_size() {
    
    _test_string_assign_pointer_size(gl_string(), "", 0, gl_string());
    _test_string_assign_pointer_size(gl_string(), "12345", 3, gl_string("123"));
    _test_string_assign_pointer_size(gl_string(), "12345", 4, gl_string("1234"));
    _test_string_assign_pointer_size(gl_string(), "12345678901234567890", 0, gl_string());
    _test_string_assign_pointer_size(gl_string(), "12345678901234567890", 1, gl_string("1"));
    _test_string_assign_pointer_size(gl_string(), "12345678901234567890", 3, gl_string("123"));
    _test_string_assign_pointer_size(gl_string(), "12345678901234567890", 20, gl_string("12345678901234567890"));

    _test_string_assign_pointer_size(gl_string("12345"), "", 0, gl_string());
    _test_string_assign_pointer_size(gl_string("12345"), "12345", 5, gl_string("12345"));
    _test_string_assign_pointer_size(gl_string("12345"), "1234567890", 10, gl_string("1234567890"));

    _test_string_assign_pointer_size(gl_string("12345678901234567890"), "", 0, gl_string());
    _test_string_assign_pointer_size(gl_string("12345678901234567890"), "12345", 5, gl_string("12345"));
    _test_string_assign_pointer_size(gl_string("12345678901234567890"), "12345678901234567890", 20,
                                     gl_string("12345678901234567890"));
  }

  void _test_string_assign_rv_string(gl_string s, gl_string str, gl_string expected) {
    s.assign(std::move(str));
    
    TS_ASSERT(s == expected);
  }

  void test_string_assign_rv_string() {
    _test_string_assign_rv_string(gl_string(), gl_string(), gl_string());
    _test_string_assign_rv_string(gl_string(), gl_string("12345"), gl_string("12345"));
    _test_string_assign_rv_string(gl_string(), gl_string("1234567890"), gl_string("1234567890"));
    _test_string_assign_rv_string(gl_string(), gl_string("12345678901234567890"), gl_string("12345678901234567890"));

    _test_string_assign_rv_string(gl_string("12345"), gl_string(), gl_string());
    _test_string_assign_rv_string(gl_string("12345"), gl_string("12345"), gl_string("12345"));
    _test_string_assign_rv_string(gl_string("12345"), gl_string("1234567890"), gl_string("1234567890"));
    _test_string_assign_rv_string(gl_string("12345"), gl_string("12345678901234567890"), gl_string("12345678901234567890"));

    _test_string_assign_rv_string(gl_string("1234567890"), gl_string(), gl_string());
    _test_string_assign_rv_string(gl_string("1234567890"), gl_string("12345"), gl_string("12345"));
    _test_string_assign_rv_string(gl_string("1234567890"), gl_string("1234567890"), gl_string("1234567890"));
    _test_string_assign_rv_string(gl_string("1234567890"), gl_string("12345678901234567890"), gl_string("12345678901234567890"));

    _test_string_assign_rv_string(gl_string("12345678901234567890"), gl_string(), gl_string());
    _test_string_assign_rv_string(gl_string("12345678901234567890"), gl_string("12345"), gl_string("12345"));
    _test_string_assign_rv_string(gl_string("12345678901234567890"), gl_string("1234567890"), gl_string("1234567890"));
    _test_string_assign_rv_string(gl_string("12345678901234567890"), gl_string("12345678901234567890"),
                                  gl_string("12345678901234567890"));
  }

  void _test_string_assign_size_char(gl_string s, size_t n, char c, gl_string expected) {
    s.assign(n, c);
    
    TS_ASSERT(s == expected);
  }

  void test_string_assign_size_char() {
    _test_string_assign_size_char(gl_string(), 0, 'a', gl_string());
    _test_string_assign_size_char(gl_string(), 1, 'a', gl_string(1, 'a'));
    _test_string_assign_size_char(gl_string(), 10, 'a', gl_string(10, 'a'));
    _test_string_assign_size_char(gl_string(), 100, 'a', gl_string(100, 'a'));

    _test_string_assign_size_char(gl_string("12345"), 0, 'a', gl_string());
    _test_string_assign_size_char(gl_string("12345"), 1, 'a', gl_string(1, 'a'));
    _test_string_assign_size_char(gl_string("12345"), 10, 'a', gl_string(10, 'a'));

    _test_string_assign_size_char(gl_string("12345678901234567890"), 0, 'a', gl_string());
    _test_string_assign_size_char(gl_string("12345678901234567890"), 1, 'a', gl_string(1, 'a'));
    _test_string_assign_size_char(gl_string("12345678901234567890"), 10, 'a', gl_string(10, 'a'));
  }

  void _test_string_assign_string(gl_string s, gl_string str, gl_string expected) {
    s.assign(str);
    
    TS_ASSERT(s == expected);
  }

  void test_string_assign_string() {
    _test_string_assign_string(gl_string(), gl_string(), gl_string());
    _test_string_assign_string(gl_string(), gl_string("12345"), gl_string("12345"));
    _test_string_assign_string(gl_string(), gl_string("1234567890"), gl_string("1234567890"));
    _test_string_assign_string(gl_string(), gl_string("12345678901234567890"), gl_string("12345678901234567890"));

    _test_string_assign_string(gl_string("12345"), gl_string(), gl_string());
    _test_string_assign_string(gl_string("12345"), gl_string("12345"), gl_string("12345"));
    _test_string_assign_string(gl_string("12345"), gl_string("1234567890"), gl_string("1234567890"));
    _test_string_assign_string(gl_string("12345"), gl_string("12345678901234567890"), gl_string("12345678901234567890"));

    _test_string_assign_string(gl_string("1234567890"), gl_string(), gl_string());
    _test_string_assign_string(gl_string("1234567890"), gl_string("12345"), gl_string("12345"));
    _test_string_assign_string(gl_string("1234567890"), gl_string("1234567890"), gl_string("1234567890"));
    _test_string_assign_string(gl_string("1234567890"), gl_string("12345678901234567890"), gl_string("12345678901234567890"));

    _test_string_assign_string(gl_string("12345678901234567890"), gl_string(), gl_string());
    _test_string_assign_string(gl_string("12345678901234567890"), gl_string("12345"), gl_string("12345"));
    _test_string_assign_string(gl_string("12345678901234567890"), gl_string("1234567890"), gl_string("1234567890"));
    _test_string_assign_string(gl_string("12345678901234567890"), gl_string("12345678901234567890"),
                               gl_string("12345678901234567890"));
  }

  void _test_string_assign_string_size_size(gl_string s, gl_string str, size_t pos, size_t n, gl_string expected) {
#ifndef NDEBUG
    try
    {
      s.assign(str, pos, n);
        
      TS_ASSERT(pos <= str.size());
      TS_ASSERT(s == expected);
    }
    catch (...)
    {
      TS_ASSERT(pos > str.size());
    }
#endif
  }


  void test_string_assign_string_size_size_npos(gl_string s, gl_string str, size_t pos, gl_string expected) {
#ifndef NDEBUG
    try
    {
      s.assign(str, pos);
        
      TS_ASSERT(pos <= str.size());
      TS_ASSERT(s == expected);
    }
    catch (...)
    {
      TS_ASSERT(pos > str.size());
    }
#endif
  }

  void test_string_assign_string_size_size() {
    _test_string_assign_string_size_size(gl_string(), gl_string(), 0, 0, gl_string());
    _test_string_assign_string_size_size(gl_string(), gl_string(), 1, 0, gl_string());
    _test_string_assign_string_size_size(gl_string(), gl_string("12345"), 0, 3, gl_string("123"));
    _test_string_assign_string_size_size(gl_string(), gl_string("12345"), 1, 4, gl_string("2345"));
    _test_string_assign_string_size_size(gl_string(), gl_string("12345"), 3, 15, gl_string("45"));
    _test_string_assign_string_size_size(gl_string(), gl_string("12345"), 5, 15, gl_string(""));
    _test_string_assign_string_size_size(gl_string(), gl_string("12345"), 6, 15, gl_string("not happening"));
    _test_string_assign_string_size_size(gl_string(), gl_string("12345678901234567890"), 0, 0, gl_string());
    _test_string_assign_string_size_size(gl_string(), gl_string("12345678901234567890"), 1, 1, gl_string("2"));
    _test_string_assign_string_size_size(gl_string(), gl_string("12345678901234567890"), 2, 3, gl_string("345"));
    _test_string_assign_string_size_size(gl_string(), gl_string("12345678901234567890"), 12, 13, gl_string("34567890"));
    _test_string_assign_string_size_size(gl_string(), gl_string("12345678901234567890"), 21, 13, gl_string("not happening"));

    _test_string_assign_string_size_size(gl_string("12345"), gl_string(), 0, 0, gl_string());
    _test_string_assign_string_size_size(gl_string("12345"), gl_string("12345"), 2, 2, gl_string("34"));
    _test_string_assign_string_size_size(gl_string("12345"), gl_string("1234567890"), 0, 100, gl_string("1234567890"));

    _test_string_assign_string_size_size(gl_string("12345678901234567890"), gl_string(), 0, 0, gl_string());
    _test_string_assign_string_size_size(gl_string("12345678901234567890"), gl_string("12345"), 1, 3, gl_string("234"));
    _test_string_assign_string_size_size(gl_string("12345678901234567890"), gl_string("12345678901234567890"), 5, 10,
                                         gl_string("6789012345"));
    
    test_string_assign_string_size_size_npos(gl_string(), gl_string(), 0, gl_string());
    test_string_assign_string_size_size_npos(gl_string(), gl_string(), 1, gl_string());
    test_string_assign_string_size_size_npos(gl_string(), gl_string("12345"), 0, gl_string("12345"));
    test_string_assign_string_size_size_npos(gl_string(), gl_string("12345"), 1, gl_string("2345"));
    test_string_assign_string_size_size_npos(gl_string(), gl_string("12345"), 3, gl_string("45"));
    test_string_assign_string_size_size_npos(gl_string(), gl_string("12345"), 5, gl_string(""));
    test_string_assign_string_size_size_npos(gl_string(), gl_string("12345"), 6, gl_string("not happening"));
  }


  void  _test_string_copy_copy(gl_string str, char* s, size_t n, size_t pos) {
#ifndef NDEBUG
    try
    {
      const gl_string& cs = str;
      size_t r = cs.copy(s, n, pos);
      TS_ASSERT(pos <= cs.size());
      size_t rlen = std::min(n, cs.size() - pos);
      TS_ASSERT(r == rlen);
      for (r = 0; r < rlen; ++r)
        TS_ASSERT(cs[pos+r] == s[r]);
    }
    catch (...)
    {
      TS_ASSERT(pos > str.size());
    }
#endif
  }

  void test_string_copy_copy() {
    
    char s[50];
    _test_string_copy_copy(gl_string(""), s, 0, 0);
    _test_string_copy_copy(gl_string(""), s, 0, 1);
    _test_string_copy_copy(gl_string(""), s, 1, 0);
    _test_string_copy_copy(gl_string("abcde"), s, 0, 0);
    _test_string_copy_copy(gl_string("abcde"), s, 0, 1);
    _test_string_copy_copy(gl_string("abcde"), s, 0, 2);
    _test_string_copy_copy(gl_string("abcde"), s, 0, 4);
    _test_string_copy_copy(gl_string("abcde"), s, 0, 5);
    _test_string_copy_copy(gl_string("abcde"), s, 0, 6);
    _test_string_copy_copy(gl_string("abcde"), s, 1, 0);
    _test_string_copy_copy(gl_string("abcde"), s, 1, 1);
    _test_string_copy_copy(gl_string("abcde"), s, 1, 2);
    _test_string_copy_copy(gl_string("abcde"), s, 1, 4);
    _test_string_copy_copy(gl_string("abcde"), s, 1, 5);
    _test_string_copy_copy(gl_string("abcde"), s, 2, 0);
    _test_string_copy_copy(gl_string("abcde"), s, 2, 1);
    _test_string_copy_copy(gl_string("abcde"), s, 2, 2);
    _test_string_copy_copy(gl_string("abcde"), s, 2, 4);
    _test_string_copy_copy(gl_string("abcde"), s, 4, 0);
    _test_string_copy_copy(gl_string("abcde"), s, 4, 1);
    _test_string_copy_copy(gl_string("abcde"), s, 4, 2);
    _test_string_copy_copy(gl_string("abcde"), s, 5, 0);
    _test_string_copy_copy(gl_string("abcde"), s, 5, 1);
    _test_string_copy_copy(gl_string("abcde"), s, 6, 0);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 0, 0);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 0, 1);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 0, 2);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 0, 10);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 0, 19);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 0, 20);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 0, 21);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 1, 0);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 1, 1);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 1, 2);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 1, 9);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 1, 18);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 1, 19);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 1, 20);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 2, 0);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 2, 1);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 2, 2);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 2, 9);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 2, 17);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 2, 18);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 2, 19);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 10, 0);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 10, 1);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 10, 2);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 10, 5);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 10, 9);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 10, 10);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 10, 11);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 19, 0);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 19, 1);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 19, 2);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 20, 0);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 20, 1);
    _test_string_copy_copy(gl_string("abcdefghijklmnopqrst"), s, 21, 0);
  }

  
  void test_string_erase_erase_iter_db1() {
    gl_string l1("123");
    gl_string::const_iterator i = l1.end();
#ifndef NDEBUG
    TS_ASSERT_THROWS_ANYTHING(l1.erase(i));
#endif 
  }

  void test_string_erase_erase_iter_db2() {
    gl_string l1("123");
    gl_string l2("123");
    gl_string::const_iterator i = l2.begin();
#ifndef NDEBUG
    TS_ASSERT_THROWS_ANYTHING(l1.erase(i));
#endif 
  }
  
  void test_string_erase_erase_iter_iter_db1() {
    gl_string l1("123");
    gl_string l2("123");
#ifndef NDEBUG
    auto f = [&](){l1.erase(l2.cbegin(), l1.cbegin()+1);};
    TS_ASSERT_THROWS_ANYTHING(f());
#endif 
  }
  
  void test_string_erase_erase_iter_iter_db2() {
    gl_string l1("123");
    gl_string l2("123");
#ifndef NDEBUG
    auto f = [&](){l1.erase(l1.cbegin(), l2.cbegin()+1);};
    TS_ASSERT_THROWS_ANYTHING(f());
#endif 
  }
  
  void test_string_erase_erase_iter_iter_db3() {
    gl_string l1("123");
    gl_string l2("123");
#ifndef NDEBUG
    auto f = [&](){l1.erase(l2.cbegin(), l2.cbegin()+1);};
    TS_ASSERT_THROWS_ANYTHING(f());
#endif 
  }
  
  void test_string_erase_erase_iter_iter_db4() {
    gl_string l1("123");
#ifndef NDEBUG
    auto f = [&](){l1.erase(l1.cbegin()+1, l1.cbegin());};
    TS_ASSERT_THROWS_ANYTHING(f());
#endif 
  }
  void _test_string_erase_iter_iter(gl_string s, gl_string::difference_type pos, gl_string::difference_type n, gl_string expected) {
    gl_string::const_iterator first = s.cbegin() + pos;
    gl_string::const_iterator last = s.cbegin() + pos + n;
    gl_string::iterator i = s.erase(first, last);
    
    TS_ASSERT(s == expected);
    TS_ASSERT(i - s.begin() == pos);
  }

  void test_string_erase_iter_iter() {
    _test_string_erase_iter_iter(gl_string(""), 0, 0, gl_string(""));
    _test_string_erase_iter_iter(gl_string("abcde"), 0, 0, gl_string("abcde"));
    _test_string_erase_iter_iter(gl_string("abcde"), 0, 1, gl_string("bcde"));
    _test_string_erase_iter_iter(gl_string("abcde"), 0, 2, gl_string("cde"));
    _test_string_erase_iter_iter(gl_string("abcde"), 0, 4, gl_string("e"));
    _test_string_erase_iter_iter(gl_string("abcde"), 0, 5, gl_string(""));
    _test_string_erase_iter_iter(gl_string("abcde"), 1, 0, gl_string("abcde"));
    _test_string_erase_iter_iter(gl_string("abcde"), 1, 1, gl_string("acde"));
    _test_string_erase_iter_iter(gl_string("abcde"), 1, 2, gl_string("ade"));
    _test_string_erase_iter_iter(gl_string("abcde"), 1, 3, gl_string("ae"));
    _test_string_erase_iter_iter(gl_string("abcde"), 1, 4, gl_string("a"));
    _test_string_erase_iter_iter(gl_string("abcde"), 2, 0, gl_string("abcde"));
    _test_string_erase_iter_iter(gl_string("abcde"), 2, 1, gl_string("abde"));
    _test_string_erase_iter_iter(gl_string("abcde"), 2, 2, gl_string("abe"));
    _test_string_erase_iter_iter(gl_string("abcde"), 2, 3, gl_string("ab"));
    _test_string_erase_iter_iter(gl_string("abcde"), 4, 0, gl_string("abcde"));
    _test_string_erase_iter_iter(gl_string("abcde"), 4, 1, gl_string("abcd"));
    _test_string_erase_iter_iter(gl_string("abcde"), 5, 0, gl_string("abcde"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 0, 0, gl_string("abcdefghij"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 0, 1, gl_string("bcdefghij"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 0, 5, gl_string("fghij"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 0, 9, gl_string("j"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 0, 10, gl_string(""));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 1, 0, gl_string("abcdefghij"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 1, 1, gl_string("acdefghij"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 1, 4, gl_string("afghij"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 1, 8, gl_string("aj"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 1, 9, gl_string("a"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 5, 0, gl_string("abcdefghij"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 5, 1, gl_string("abcdeghij"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 5, 2, gl_string("abcdehij"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 5, 4, gl_string("abcdej"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 5, 5, gl_string("abcde"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 9, 0, gl_string("abcdefghij"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 9, 1, gl_string("abcdefghi"));
    _test_string_erase_iter_iter(gl_string("abcdefghij"), 10, 0, gl_string("abcdefghij"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("bcdefghijklmnopqrst"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("klmnopqrst"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("t"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string(""));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("acdefghijklmnopqrst"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("aklmnopqrst"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("at"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("a"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("abcdefghijlmnopqrst"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("abcdefghijpqrst"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("abcdefghijt"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("abcdefghij"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("abcdefghijklmnopqrs"));
    _test_string_erase_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("abcdefghijklmnopqrst"));
  }

  
  void _test_string_erase_iter(gl_string s, gl_string::difference_type pos, gl_string expected) {
    gl_string::const_iterator p = s.begin() + pos;
    gl_string::iterator i = s.erase(p);
    
    TS_ASSERT(s == expected);
    TS_ASSERT(i - s.begin() == pos);
  }

  void test_string_erase_iter() {
    _test_string_erase_iter(gl_string("abcde"), 0, gl_string("bcde"));
    _test_string_erase_iter(gl_string("abcde"), 1, gl_string("acde"));
    _test_string_erase_iter(gl_string("abcde"), 2, gl_string("abde"));
    _test_string_erase_iter(gl_string("abcde"), 4, gl_string("abcd"));
    _test_string_erase_iter(gl_string("abcdefghij"), 0, gl_string("bcdefghij"));
    _test_string_erase_iter(gl_string("abcdefghij"), 1, gl_string("acdefghij"));
    _test_string_erase_iter(gl_string("abcdefghij"), 5, gl_string("abcdeghij"));
    _test_string_erase_iter(gl_string("abcdefghij"), 9, gl_string("abcdefghi"));
    _test_string_erase_iter(gl_string("abcdefghijklmnopqrst"), 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_erase_iter(gl_string("abcdefghijklmnopqrst"), 1, gl_string("acdefghijklmnopqrst"));
    _test_string_erase_iter(gl_string("abcdefghijklmnopqrst"), 10, gl_string("abcdefghijlmnopqrst"));
    _test_string_erase_iter(gl_string("abcdefghijklmnopqrst"), 19, gl_string("abcdefghijklmnopqrs"));
  }

  void _test_string_erase_pop_back(gl_string s, gl_string expected) {
    s.pop_back();
    
    TS_ASSERT(s == expected);
  }

  void test_string_erase_pop_back() {
    _test_string_erase_pop_back(gl_string("abcde"), gl_string("abcd"));
    _test_string_erase_pop_back(gl_string("abcdefghij"), gl_string("abcdefghi"));
    _test_string_erase_pop_back(gl_string("abcdefghijklmnopqrst"), gl_string("abcdefghijklmnopqrs"));
  }

  void _test_string_erase_size_size(gl_string s, size_t pos, size_t n, gl_string expected) {
#ifndef NDEBUG    
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.erase(pos, n);
        
      TS_ASSERT(pos <= old_size);
      TS_ASSERT(s == expected);
    }
    catch (...)
    {
      TS_ASSERT(pos > old_size);
      TS_ASSERT(s == s0);
    }
#endif
  }


  void _test_string_erase_size_size(gl_string s, size_t pos, gl_string expected) {
#ifndef NDEBUG    
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.erase(pos);
        
      TS_ASSERT(pos <= old_size);
      TS_ASSERT(s == expected);
    }
    catch (...) 
    {
      TS_ASSERT(pos > old_size);
      TS_ASSERT(s == s0);
    }
#endif
  }


  void _test_string_erase_size_size(gl_string s, gl_string expected) {
    s.erase();
    
    TS_ASSERT(s == expected);
  }

  void test_string_erase_size_size() {
    _test_string_erase_size_size(gl_string(""), 0, 0, gl_string(""));
    _test_string_erase_size_size(gl_string(""), 0, 1, gl_string(""));
    _test_string_erase_size_size(gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_erase_size_size(gl_string("abcde"), 0, 0, gl_string("abcde"));
    _test_string_erase_size_size(gl_string("abcde"), 0, 1, gl_string("bcde"));
    _test_string_erase_size_size(gl_string("abcde"), 0, 2, gl_string("cde"));
    _test_string_erase_size_size(gl_string("abcde"), 0, 4, gl_string("e"));
    _test_string_erase_size_size(gl_string("abcde"), 0, 5, gl_string(""));
    _test_string_erase_size_size(gl_string("abcde"), 0, 6, gl_string(""));
    _test_string_erase_size_size(gl_string("abcde"), 1, 0, gl_string("abcde"));
    _test_string_erase_size_size(gl_string("abcde"), 1, 1, gl_string("acde"));
    _test_string_erase_size_size(gl_string("abcde"), 1, 2, gl_string("ade"));
    _test_string_erase_size_size(gl_string("abcde"), 1, 3, gl_string("ae"));
    _test_string_erase_size_size(gl_string("abcde"), 1, 4, gl_string("a"));
    _test_string_erase_size_size(gl_string("abcde"), 1, 5, gl_string("a"));
    _test_string_erase_size_size(gl_string("abcde"), 2, 0, gl_string("abcde"));
    _test_string_erase_size_size(gl_string("abcde"), 2, 1, gl_string("abde"));
    _test_string_erase_size_size(gl_string("abcde"), 2, 2, gl_string("abe"));
    _test_string_erase_size_size(gl_string("abcde"), 2, 3, gl_string("ab"));
    _test_string_erase_size_size(gl_string("abcde"), 2, 4, gl_string("ab"));
    _test_string_erase_size_size(gl_string("abcde"), 4, 0, gl_string("abcde"));
    _test_string_erase_size_size(gl_string("abcde"), 4, 1, gl_string("abcd"));
    _test_string_erase_size_size(gl_string("abcde"), 4, 2, gl_string("abcd"));
    _test_string_erase_size_size(gl_string("abcde"), 5, 0, gl_string("abcde"));
    _test_string_erase_size_size(gl_string("abcde"), 5, 1, gl_string("abcde"));
    _test_string_erase_size_size(gl_string("abcde"), 6, 0, gl_string("can't happen"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 0, 0, gl_string("abcdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 0, 1, gl_string("bcdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 0, 5, gl_string("fghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 0, 9, gl_string("j"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 0, 10, gl_string(""));
    _test_string_erase_size_size(gl_string("abcdefghij"), 0, 11, gl_string(""));
    _test_string_erase_size_size(gl_string("abcdefghij"), 1, 0, gl_string("abcdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 1, 1, gl_string("acdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 1, 4, gl_string("afghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 1, 8, gl_string("aj"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 1, 9, gl_string("a"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 1, 10, gl_string("a"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 5, 0, gl_string("abcdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 5, 1, gl_string("abcdeghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 5, 2, gl_string("abcdehij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 5, 4, gl_string("abcdej"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 5, 5, gl_string("abcde"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 5, 6, gl_string("abcde"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 9, 0, gl_string("abcdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 9, 1, gl_string("abcdefghi"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 9, 2, gl_string("abcdefghi"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 10, 0, gl_string("abcdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 10, 1, gl_string("abcdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 11, 0, gl_string("can't happen"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("bcdefghijklmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("klmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("t"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string(""));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string(""));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("acdefghijklmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("aklmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("at"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("a"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("a"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("abcdefghijlmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("abcdefghijpqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("abcdefghijt"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("abcdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("abcdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("abcdefghijklmnopqrs"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("abcdefghijklmnopqrs"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("can't happen"));

    _test_string_erase_size_size(gl_string(""), 0, gl_string(""));
    _test_string_erase_size_size(gl_string(""), 1, gl_string("can't happen"));
    _test_string_erase_size_size(gl_string("abcde"), 0, gl_string(""));
    _test_string_erase_size_size(gl_string("abcde"), 1, gl_string("a"));
    _test_string_erase_size_size(gl_string("abcde"), 2, gl_string("ab"));
    _test_string_erase_size_size(gl_string("abcde"), 4, gl_string("abcd"));
    _test_string_erase_size_size(gl_string("abcde"), 5, gl_string("abcde"));
    _test_string_erase_size_size(gl_string("abcde"), 6, gl_string("can't happen"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 0, gl_string(""));
    _test_string_erase_size_size(gl_string("abcdefghij"), 1, gl_string("a"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 5, gl_string("abcde"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 9, gl_string("abcdefghi"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 10, gl_string("abcdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghij"), 11, gl_string("can't happen"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string(""));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("a"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("abcdefghij"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("abcdefghijklmnopqrs"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("abcdefghijklmnopqrst"));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("can't happen"));

    _test_string_erase_size_size(gl_string(""), gl_string(""));
    _test_string_erase_size_size(gl_string("abcde"), gl_string(""));
    _test_string_erase_size_size(gl_string("abcdefghij"), gl_string(""));
    _test_string_erase_size_size(gl_string("abcdefghijklmnopqrst"), gl_string(""));
  }

  
  void
  _test_string_insert_iter_char(gl_string& s, gl_string::const_iterator p, char c, gl_string expected)
  {
    bool sufficient_cap = s.size() < s.capacity();
    gl_string::difference_type pos = p - s.begin();
    gl_string::iterator i = s.insert(p, c);
    
    TS_ASSERT(s == expected);
    TS_ASSERT(i - s.begin() == pos);
    TS_ASSERT(*i == c);
    if (sufficient_cap)
      TS_ASSERT(i == p);
  }

  void test_string_insert_iter_char()
  {
    gl_string s;
    _test_string_insert_iter_char(s, s.begin(), '1', gl_string("1"));
    _test_string_insert_iter_char(s, s.begin(), 'a', gl_string("a1"));
    _test_string_insert_iter_char(s, s.end(), 'b', gl_string("a1b"));
    _test_string_insert_iter_char(s, s.end()-1, 'c', gl_string("a1cb"));
    _test_string_insert_iter_char(s, s.end()-2, 'd', gl_string("a1dcb"));
    _test_string_insert_iter_char(s, s.end()-3, '2', gl_string("a12dcb"));
    _test_string_insert_iter_char(s, s.end()-4, '3', gl_string("a132dcb"));
    _test_string_insert_iter_char(s, s.end()-5, '4', gl_string("a1432dcb"));
    _test_string_insert_iter_char(s, s.begin()+1, '5', gl_string("a51432dcb"));
    _test_string_insert_iter_char(s, s.begin()+2, '6', gl_string("a561432dcb"));
    _test_string_insert_iter_char(s, s.begin()+3, '7', gl_string("a5671432dcb"));
    _test_string_insert_iter_char(s, s.begin()+4, 'A', gl_string("a567A1432dcb"));
    _test_string_insert_iter_char(s, s.begin()+5, 'B', gl_string("a567AB1432dcb"));
    _test_string_insert_iter_char(s, s.begin()+6, 'C', gl_string("a567ABC1432dcb"));
  }

  void test_string_insert_iter_initializer_list()
  {
    gl_string s("123456");
    gl_string::iterator i = s.insert(s.begin() + 3, {'a', 'b', 'c'});
    TS_ASSERT(i - s.begin() == 3);
    TS_ASSERT(s == "123abc456");
  }


  
  template < class It>
  void
  _test_string_insert_iter_iter_iter(gl_string s, gl_string::difference_type pos, It first, It last, gl_string expected)
  {
    gl_string::const_iterator p = s.cbegin() + pos;
    gl_string::iterator i = s.insert(p, first, last);
    
    ASSERT_EQ(i - s.begin(), pos);
    TS_ASSERT(s == expected);
  }

  void test_string_insert_iter_iter_iter()
  {
    const char* s = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
    _test_string_insert_iter_iter_iter(gl_string(), 0, s, s, gl_string());
    _test_string_insert_iter_iter_iter(gl_string(), 0, s, s+1, gl_string("A"));
    _test_string_insert_iter_iter_iter(gl_string(), 0, s, s+10, gl_string("ABCDEFGHIJ"));
    _test_string_insert_iter_iter_iter(gl_string(), 0, s, s+52, gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_insert_iter_iter_iter(gl_string("12345"), 0, s, s, gl_string("12345"));
    _test_string_insert_iter_iter_iter(gl_string("12345"), 1, s, s+1, gl_string("1A2345"));
    _test_string_insert_iter_iter_iter(gl_string("12345"), 4, s, s+10, gl_string("1234ABCDEFGHIJ5"));
    _test_string_insert_iter_iter_iter(gl_string("12345"), 5, s, s+52, gl_string("12345ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_insert_iter_iter_iter(gl_string("1234567890"), 0, s, s, gl_string("1234567890"));
    _test_string_insert_iter_iter_iter(gl_string("1234567890"), 1, s, s+1, gl_string("1A234567890"));
    _test_string_insert_iter_iter_iter(gl_string("1234567890"), 10, s, s+10, gl_string("1234567890ABCDEFGHIJ"));
    _test_string_insert_iter_iter_iter(gl_string("1234567890"), 8, s, s+52, gl_string("12345678ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz90"));

    _test_string_insert_iter_iter_iter(gl_string("12345678901234567890"), 3, s, s, gl_string("12345678901234567890"));
    _test_string_insert_iter_iter_iter(gl_string("12345678901234567890"), 3, s, s+1, gl_string("123A45678901234567890"));
    _test_string_insert_iter_iter_iter(gl_string("12345678901234567890"), 15, s, s+10, gl_string("123456789012345ABCDEFGHIJ67890"));
    _test_string_insert_iter_iter_iter(gl_string("12345678901234567890"), 20, s, s+52,
                                       gl_string("12345678901234567890ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_insert_iter_iter_iter(gl_string(), 0, (const char*)(s), (const char*)(s), gl_string());
    _test_string_insert_iter_iter_iter(gl_string(), 0, (const char*)(s), (const char*)(s+1), gl_string("A"));
    _test_string_insert_iter_iter_iter(gl_string(), 0, (const char*)(s), (const char*)(s+10), gl_string("ABCDEFGHIJ"));
    _test_string_insert_iter_iter_iter(gl_string(), 0, (const char*)(s), (const char*)(s+52), gl_string("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_insert_iter_iter_iter(gl_string("12345"), 0, (const char*)(s), (const char*)(s), gl_string("12345"));
    _test_string_insert_iter_iter_iter(gl_string("12345"), 1, (const char*)(s), (const char*)(s+1), gl_string("1A2345"));
    _test_string_insert_iter_iter_iter(gl_string("12345"), 4, (const char*)(s), (const char*)(s+10), gl_string("1234ABCDEFGHIJ5"));
    _test_string_insert_iter_iter_iter(gl_string("12345"), 5, (const char*)(s), (const char*)(s+52), gl_string("12345ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));

    _test_string_insert_iter_iter_iter(gl_string("1234567890"), 0, (const char*)(s), (const char*)(s), gl_string("1234567890"));
    _test_string_insert_iter_iter_iter(gl_string("1234567890"), 1, (const char*)(s), (const char*)(s+1), gl_string("1A234567890"));
    _test_string_insert_iter_iter_iter(gl_string("1234567890"), 10, (const char*)(s), (const char*)(s+10), gl_string("1234567890ABCDEFGHIJ"));
    _test_string_insert_iter_iter_iter(gl_string("1234567890"), 8, (const char*)(s), (const char*)(s+52), gl_string("12345678ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz90"));

    _test_string_insert_iter_iter_iter(gl_string("12345678901234567890"), 3, (const char*)(s), (const char*)(s), gl_string("12345678901234567890"));
    _test_string_insert_iter_iter_iter(gl_string("12345678901234567890"), 3, (const char*)(s), (const char*)(s+1), gl_string("123A45678901234567890"));
    _test_string_insert_iter_iter_iter(gl_string("12345678901234567890"), 15, (const char*)(s), (const char*)(s+10), gl_string("123456789012345ABCDEFGHIJ67890"));
    _test_string_insert_iter_iter_iter(gl_string("12345678901234567890"), 20, (const char*)(s), (const char*)(s+52),
                                       gl_string("12345678901234567890ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"));
  }

  
  void
  _test_string_insert_iter_size_char(gl_string s, gl_string::difference_type pos, size_t n,
                                     char c, gl_string expected)
  {
    gl_string::const_iterator p = s.cbegin() + pos;
    gl_string::iterator i = s.insert(p, n, c);
    
    ASSERT_EQ(i - s.begin(), pos);
    TS_ASSERT(s == expected);
  }

  void test_string_insert_iter_size_char()
  {
    _test_string_insert_iter_size_char(gl_string(""), 0, 0, '1', gl_string(""));
    _test_string_insert_iter_size_char(gl_string(""), 0, 5, '1', gl_string("11111"));
    _test_string_insert_iter_size_char(gl_string(""), 0, 10, '1', gl_string("1111111111"));
    _test_string_insert_iter_size_char(gl_string(""), 0, 20, '1', gl_string("11111111111111111111"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 0, 0, '1', gl_string("abcde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 0, 5, '1', gl_string("11111abcde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 0, 10, '1', gl_string("1111111111abcde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 0, 20, '1', gl_string("11111111111111111111abcde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 1, 0, '1', gl_string("abcde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 1, 5, '1', gl_string("a11111bcde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 1, 10, '1', gl_string("a1111111111bcde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 1, 20, '1', gl_string("a11111111111111111111bcde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 2, 0, '1', gl_string("abcde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 2, 5, '1', gl_string("ab11111cde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 2, 10, '1', gl_string("ab1111111111cde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 2, 20, '1', gl_string("ab11111111111111111111cde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 4, 0, '1', gl_string("abcde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 4, 5, '1', gl_string("abcd11111e"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 4, 10, '1', gl_string("abcd1111111111e"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 4, 20, '1', gl_string("abcd11111111111111111111e"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 5, 0, '1', gl_string("abcde"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 5, 5, '1', gl_string("abcde11111"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 5, 10, '1', gl_string("abcde1111111111"));
    _test_string_insert_iter_size_char(gl_string("abcde"), 5, 20, '1', gl_string("abcde11111111111111111111"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 0, 0, '1', gl_string("abcdefghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 0, 5, '1', gl_string("11111abcdefghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 0, 10, '1', gl_string("1111111111abcdefghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 0, 20, '1', gl_string("11111111111111111111abcdefghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 1, 0, '1', gl_string("abcdefghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 1, 5, '1', gl_string("a11111bcdefghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 1, 10, '1', gl_string("a1111111111bcdefghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 1, 20, '1', gl_string("a11111111111111111111bcdefghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 5, 0, '1', gl_string("abcdefghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 5, 5, '1', gl_string("abcde11111fghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 5, 10, '1', gl_string("abcde1111111111fghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 5, 20, '1', gl_string("abcde11111111111111111111fghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 9, 0, '1', gl_string("abcdefghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 9, 5, '1', gl_string("abcdefghi11111j"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 9, 10, '1', gl_string("abcdefghi1111111111j"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 9, 20, '1', gl_string("abcdefghi11111111111111111111j"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 10, 0, '1', gl_string("abcdefghij"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 10, 5, '1', gl_string("abcdefghij11111"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 10, 10, '1', gl_string("abcdefghij1111111111"));
    _test_string_insert_iter_size_char(gl_string("abcdefghij"), 10, 20, '1', gl_string("abcdefghij11111111111111111111"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 0, '1', gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 5, '1', gl_string("11111abcdefghijklmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 10, '1', gl_string("1111111111abcdefghijklmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 20, '1', gl_string("11111111111111111111abcdefghijklmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 0, '1', gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 5, '1', gl_string("a11111bcdefghijklmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 10, '1', gl_string("a1111111111bcdefghijklmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 20, '1', gl_string("a11111111111111111111bcdefghijklmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 0, '1', gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 5, '1', gl_string("abcdefghij11111klmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 10, '1', gl_string("abcdefghij1111111111klmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 20, '1', gl_string("abcdefghij11111111111111111111klmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 0, '1', gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 5, '1', gl_string("abcdefghijklmnopqrs11111t"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 10, '1', gl_string("abcdefghijklmnopqrs1111111111t"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 20, '1', gl_string("abcdefghijklmnopqrs11111111111111111111t"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 20, 0, '1', gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 20, 5, '1', gl_string("abcdefghijklmnopqrst11111"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 20, 10, '1', gl_string("abcdefghijklmnopqrst1111111111"));
    _test_string_insert_iter_size_char(gl_string("abcdefghijklmnopqrst"), 20, 20, '1', gl_string("abcdefghijklmnopqrst11111111111111111111"));
  }

  
  void
  _test_string_insert_size_pointer(gl_string s, size_t pos, const char* str, gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.insert(pos, str);
        
      TS_ASSERT(pos <= old_size);
      TS_ASSERT(s == expected);
    }
    catch (...)
    {
      TS_ASSERT(pos > old_size);
      TS_ASSERT(s == s0);
    }
#endif
  }

  void test_string_insert_size_pointer()
  {
    _test_string_insert_size_pointer(gl_string(""), 0, "", gl_string(""));
    _test_string_insert_size_pointer(gl_string(""), 0, "12345", gl_string("12345"));
    _test_string_insert_size_pointer(gl_string(""), 0, "1234567890", gl_string("1234567890"));
    _test_string_insert_size_pointer(gl_string(""), 0, "12345678901234567890", gl_string("12345678901234567890"));
    _test_string_insert_size_pointer(gl_string(""), 1, "", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string(""), 1, "12345", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string(""), 1, "1234567890", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string(""), 1, "12345678901234567890", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcde"), 0, "", gl_string("abcde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 0, "12345", gl_string("12345abcde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 0, "1234567890", gl_string("1234567890abcde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 0, "12345678901234567890", gl_string("12345678901234567890abcde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 1, "", gl_string("abcde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 1, "12345", gl_string("a12345bcde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 1, "1234567890", gl_string("a1234567890bcde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 1, "12345678901234567890", gl_string("a12345678901234567890bcde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 2, "", gl_string("abcde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 2, "12345", gl_string("ab12345cde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 2, "1234567890", gl_string("ab1234567890cde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 2, "12345678901234567890", gl_string("ab12345678901234567890cde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 4, "", gl_string("abcde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 4, "12345", gl_string("abcd12345e"));
    _test_string_insert_size_pointer(gl_string("abcde"), 4, "1234567890", gl_string("abcd1234567890e"));
    _test_string_insert_size_pointer(gl_string("abcde"), 4, "12345678901234567890", gl_string("abcd12345678901234567890e"));
    _test_string_insert_size_pointer(gl_string("abcde"), 5, "", gl_string("abcde"));
    _test_string_insert_size_pointer(gl_string("abcde"), 5, "12345", gl_string("abcde12345"));
    _test_string_insert_size_pointer(gl_string("abcde"), 5, "1234567890", gl_string("abcde1234567890"));
    _test_string_insert_size_pointer(gl_string("abcde"), 5, "12345678901234567890", gl_string("abcde12345678901234567890"));
    _test_string_insert_size_pointer(gl_string("abcde"), 6, "", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcde"), 6, "12345", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcde"), 6, "1234567890", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcde"), 6, "12345678901234567890", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 0, "", gl_string("abcdefghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 0, "12345", gl_string("12345abcdefghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 0, "1234567890", gl_string("1234567890abcdefghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 0, "12345678901234567890", gl_string("12345678901234567890abcdefghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 1, "", gl_string("abcdefghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 1, "12345", gl_string("a12345bcdefghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 1, "1234567890", gl_string("a1234567890bcdefghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 1, "12345678901234567890", gl_string("a12345678901234567890bcdefghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 5, "", gl_string("abcdefghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 5, "12345", gl_string("abcde12345fghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 5, "1234567890", gl_string("abcde1234567890fghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 5, "12345678901234567890", gl_string("abcde12345678901234567890fghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 9, "", gl_string("abcdefghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 9, "12345", gl_string("abcdefghi12345j"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 9, "1234567890", gl_string("abcdefghi1234567890j"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 9, "12345678901234567890", gl_string("abcdefghi12345678901234567890j"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 10, "", gl_string("abcdefghij"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 10, "12345", gl_string("abcdefghij12345"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 10, "1234567890", gl_string("abcdefghij1234567890"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 10, "12345678901234567890", gl_string("abcdefghij12345678901234567890"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 11, "", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 11, "12345", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 11, "1234567890", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcdefghij"), 11, "12345678901234567890", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, "", gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, "12345", gl_string("12345abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, "1234567890", gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, "12345678901234567890", gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, "", gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, "12345", gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, "1234567890", gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, "12345678901234567890", gl_string("a12345678901234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, "", gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, "12345", gl_string("abcdefghij12345klmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, "1234567890", gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, "12345678901234567890", gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, "", gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, "12345", gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, "1234567890", gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, "12345678901234567890", gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, "", gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, "12345", gl_string("abcdefghijklmnopqrst12345"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, "1234567890", gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, "12345678901234567890", gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 21, "", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 21, "12345", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 21, "1234567890", gl_string("can't happen"));
    _test_string_insert_size_pointer(gl_string("abcdefghijklmnopqrst"), 21, "12345678901234567890", gl_string("can't happen"));
  }


  void
  _test_string_insert_size_pointer_size(gl_string s, size_t pos, const char* str,
                                        size_t n, gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.insert(pos, str, n);
        
      TS_ASSERT(pos <= old_size);
      TS_ASSERT(s == expected);
    }
    catch (...)
    {
      TS_ASSERT(pos > old_size);
      TS_ASSERT(s == s0);
    }
#endif
  }

  void test_string_insert_size_pointer_size()
  {
    _test_string_insert_size_pointer_size(gl_string(""), 0, "", 0, gl_string(""));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "12345", 0, gl_string(""));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "12345", 1, gl_string("1"));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "12345", 2, gl_string("12"));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "12345", 4, gl_string("1234"));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "12345", 5, gl_string("12345"));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "1234567890", 0, gl_string(""));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "1234567890", 1, gl_string("1"));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "1234567890", 5, gl_string("12345"));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "1234567890", 9, gl_string("123456789"));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "1234567890", 10, gl_string("1234567890"));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "12345678901234567890", 0, gl_string(""));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "12345678901234567890", 1, gl_string("1"));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "12345678901234567890", 10, gl_string("1234567890"));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "12345678901234567890", 19, gl_string("1234567890123456789"));
    _test_string_insert_size_pointer_size(gl_string(""), 0, "12345678901234567890", 20, gl_string("12345678901234567890"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "12345", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "12345", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "12345", 2, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "12345", 4, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "12345", 5, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "1234567890", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "1234567890", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "1234567890", 5, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "1234567890", 9, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "1234567890", 10, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "12345678901234567890", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "12345678901234567890", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "12345678901234567890", 10, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "12345678901234567890", 19, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string(""), 1, "12345678901234567890", 20, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "12345", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "12345", 1, gl_string("1abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "12345", 2, gl_string("12abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "12345", 4, gl_string("1234abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "12345", 5, gl_string("12345abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "1234567890", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "1234567890", 1, gl_string("1abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "1234567890", 5, gl_string("12345abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "1234567890", 9, gl_string("123456789abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "1234567890", 10, gl_string("1234567890abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "12345678901234567890", 1, gl_string("1abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "12345678901234567890", 10, gl_string("1234567890abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "12345678901234567890", 19, gl_string("1234567890123456789abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 0, "12345678901234567890", 20, gl_string("12345678901234567890abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "12345", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "12345", 1, gl_string("a1bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "12345", 2, gl_string("a12bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "12345", 4, gl_string("a1234bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "12345", 5, gl_string("a12345bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "1234567890", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "1234567890", 1, gl_string("a1bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "1234567890", 5, gl_string("a12345bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "1234567890", 9, gl_string("a123456789bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "1234567890", 10, gl_string("a1234567890bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "12345678901234567890", 1, gl_string("a1bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "12345678901234567890", 10, gl_string("a1234567890bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "12345678901234567890", 19, gl_string("a1234567890123456789bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 1, "12345678901234567890", 20, gl_string("a12345678901234567890bcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "12345", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "12345", 1, gl_string("ab1cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "12345", 2, gl_string("ab12cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "12345", 4, gl_string("ab1234cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "12345", 5, gl_string("ab12345cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "1234567890", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "1234567890", 1, gl_string("ab1cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "1234567890", 5, gl_string("ab12345cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "1234567890", 9, gl_string("ab123456789cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "1234567890", 10, gl_string("ab1234567890cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "12345678901234567890", 1, gl_string("ab1cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "12345678901234567890", 10, gl_string("ab1234567890cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "12345678901234567890", 19, gl_string("ab1234567890123456789cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 2, "12345678901234567890", 20, gl_string("ab12345678901234567890cde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "12345", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "12345", 1, gl_string("abcd1e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "12345", 2, gl_string("abcd12e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "12345", 4, gl_string("abcd1234e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "12345", 5, gl_string("abcd12345e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "1234567890", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "1234567890", 1, gl_string("abcd1e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "1234567890", 5, gl_string("abcd12345e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "1234567890", 9, gl_string("abcd123456789e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "1234567890", 10, gl_string("abcd1234567890e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "12345678901234567890", 1, gl_string("abcd1e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "12345678901234567890", 10, gl_string("abcd1234567890e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "12345678901234567890", 19, gl_string("abcd1234567890123456789e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 4, "12345678901234567890", 20, gl_string("abcd12345678901234567890e"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "12345", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "12345", 1, gl_string("abcde1"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "12345", 2, gl_string("abcde12"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "12345", 4, gl_string("abcde1234"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "12345", 5, gl_string("abcde12345"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "1234567890", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "1234567890", 1, gl_string("abcde1"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "1234567890", 5, gl_string("abcde12345"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "1234567890", 9, gl_string("abcde123456789"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "1234567890", 10, gl_string("abcde1234567890"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "12345678901234567890", 1, gl_string("abcde1"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "12345678901234567890", 10, gl_string("abcde1234567890"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "12345678901234567890", 19, gl_string("abcde1234567890123456789"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 5, "12345678901234567890", 20, gl_string("abcde12345678901234567890"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "12345", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "12345", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "12345", 2, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "12345", 4, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "12345", 5, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "1234567890", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "1234567890", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "1234567890", 5, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "1234567890", 9, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "1234567890", 10, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "12345678901234567890", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "12345678901234567890", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "12345678901234567890", 10, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "12345678901234567890", 19, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcde"), 6, "12345678901234567890", 20, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "12345", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "12345", 1, gl_string("1abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "12345", 2, gl_string("12abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "12345", 4, gl_string("1234abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "12345", 5, gl_string("12345abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "1234567890", 1, gl_string("1abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "1234567890", 5, gl_string("12345abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "1234567890", 9, gl_string("123456789abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "1234567890", 10, gl_string("1234567890abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "12345678901234567890", 1, gl_string("1abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "12345678901234567890", 10, gl_string("1234567890abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "12345678901234567890", 19, gl_string("1234567890123456789abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 0, "12345678901234567890", 20, gl_string("12345678901234567890abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "12345", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "12345", 1, gl_string("a1bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "12345", 2, gl_string("a12bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "12345", 4, gl_string("a1234bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "12345", 5, gl_string("a12345bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "1234567890", 1, gl_string("a1bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "1234567890", 5, gl_string("a12345bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "1234567890", 9, gl_string("a123456789bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "1234567890", 10, gl_string("a1234567890bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "12345678901234567890", 1, gl_string("a1bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "12345678901234567890", 10, gl_string("a1234567890bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "12345678901234567890", 19, gl_string("a1234567890123456789bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 1, "12345678901234567890", 20, gl_string("a12345678901234567890bcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "12345", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "12345", 1, gl_string("abcde1fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "12345", 2, gl_string("abcde12fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "12345", 4, gl_string("abcde1234fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "12345", 5, gl_string("abcde12345fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "1234567890", 1, gl_string("abcde1fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "1234567890", 5, gl_string("abcde12345fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "1234567890", 9, gl_string("abcde123456789fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "1234567890", 10, gl_string("abcde1234567890fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "12345678901234567890", 1, gl_string("abcde1fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "12345678901234567890", 10, gl_string("abcde1234567890fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "12345678901234567890", 19, gl_string("abcde1234567890123456789fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 5, "12345678901234567890", 20, gl_string("abcde12345678901234567890fghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "12345", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "12345", 1, gl_string("abcdefghi1j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "12345", 2, gl_string("abcdefghi12j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "12345", 4, gl_string("abcdefghi1234j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "12345", 5, gl_string("abcdefghi12345j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "1234567890", 1, gl_string("abcdefghi1j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "1234567890", 5, gl_string("abcdefghi12345j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "1234567890", 9, gl_string("abcdefghi123456789j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "1234567890", 10, gl_string("abcdefghi1234567890j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "12345678901234567890", 1, gl_string("abcdefghi1j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "12345678901234567890", 10, gl_string("abcdefghi1234567890j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "12345678901234567890", 19, gl_string("abcdefghi1234567890123456789j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 9, "12345678901234567890", 20, gl_string("abcdefghi12345678901234567890j"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "12345", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "12345", 1, gl_string("abcdefghij1"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "12345", 2, gl_string("abcdefghij12"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "12345", 4, gl_string("abcdefghij1234"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "12345", 5, gl_string("abcdefghij12345"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "1234567890", 1, gl_string("abcdefghij1"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "1234567890", 5, gl_string("abcdefghij12345"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "1234567890", 9, gl_string("abcdefghij123456789"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "1234567890", 10, gl_string("abcdefghij1234567890"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "12345678901234567890", 1, gl_string("abcdefghij1"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "12345678901234567890", 10, gl_string("abcdefghij1234567890"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "12345678901234567890", 19, gl_string("abcdefghij1234567890123456789"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 10, "12345678901234567890", 20, gl_string("abcdefghij12345678901234567890"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "12345", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "12345", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "12345", 2, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "12345", 4, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "12345", 5, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "1234567890", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "1234567890", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "1234567890", 5, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "1234567890", 9, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "1234567890", 10, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "12345678901234567890", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "12345678901234567890", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "12345678901234567890", 10, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "12345678901234567890", 19, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghij"), 11, "12345678901234567890", 20, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "12345", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "12345", 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "12345", 2, gl_string("12abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "12345", 4, gl_string("1234abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "12345", 5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "1234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "1234567890", 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "1234567890", 5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "1234567890", 9, gl_string("123456789abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "1234567890", 10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "12345678901234567890", 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "12345678901234567890", 10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "12345678901234567890", 19, gl_string("1234567890123456789abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, "12345678901234567890", 20, gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "12345", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "12345", 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "12345", 2, gl_string("a12bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "12345", 4, gl_string("a1234bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "12345", 5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "1234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "1234567890", 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "1234567890", 5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "1234567890", 9, gl_string("a123456789bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "1234567890", 10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "12345678901234567890", 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "12345678901234567890", 10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "12345678901234567890", 19, gl_string("a1234567890123456789bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, "12345678901234567890", 20, gl_string("a12345678901234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "12345", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "12345", 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "12345", 2, gl_string("abcdefghij12klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "12345", 4, gl_string("abcdefghij1234klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "12345", 5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "1234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "1234567890", 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "1234567890", 5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "1234567890", 9, gl_string("abcdefghij123456789klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "1234567890", 10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "12345678901234567890", 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "12345678901234567890", 10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "12345678901234567890", 19, gl_string("abcdefghij1234567890123456789klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, "12345678901234567890", 20, gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "12345", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "12345", 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "12345", 2, gl_string("abcdefghijklmnopqrs12t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "12345", 4, gl_string("abcdefghijklmnopqrs1234t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "12345", 5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "1234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "1234567890", 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "1234567890", 5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "1234567890", 9, gl_string("abcdefghijklmnopqrs123456789t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "1234567890", 10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "12345678901234567890", 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "12345678901234567890", 10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "12345678901234567890", 19, gl_string("abcdefghijklmnopqrs1234567890123456789t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, "12345678901234567890", 20, gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "12345", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "12345", 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "12345", 2, gl_string("abcdefghijklmnopqrst12"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "12345", 4, gl_string("abcdefghijklmnopqrst1234"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "12345", 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "1234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "1234567890", 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "1234567890", 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "1234567890", 9, gl_string("abcdefghijklmnopqrst123456789"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "1234567890", 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "12345678901234567890", 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "12345678901234567890", 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "12345678901234567890", 19, gl_string("abcdefghijklmnopqrst1234567890123456789"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, "12345678901234567890", 20, gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "12345", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "12345", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "12345", 2, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "12345", 4, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "12345", 5, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "1234567890", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "1234567890", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "1234567890", 5, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "1234567890", 9, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "1234567890", 10, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "12345678901234567890", 0, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "12345678901234567890", 1, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "12345678901234567890", 10, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "12345678901234567890", 19, gl_string("can't happen"));
    _test_string_insert_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, "12345678901234567890", 20, gl_string("can't happen"));
  }


  void
  _test_string_insert_size_size_char(gl_string s, size_t pos, size_t n,
                                     char str, gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.insert(pos, n, str);
        
      TS_ASSERT(pos <= old_size);
      TS_ASSERT(s == expected);
    }
    catch (...)
    {
      TS_ASSERT(pos > old_size);
      TS_ASSERT(s == s0);
    }
#endif
  }

  void test_string_insert_size_size_char()
  {
    _test_string_insert_size_size_char(gl_string(""), 0, 0, '1', gl_string(""));
    _test_string_insert_size_size_char(gl_string(""), 0, 5, '1', gl_string("11111"));
    _test_string_insert_size_size_char(gl_string(""), 0, 10, '1', gl_string("1111111111"));
    _test_string_insert_size_size_char(gl_string(""), 0, 20, '1', gl_string("11111111111111111111"));
    _test_string_insert_size_size_char(gl_string(""), 1, 0, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string(""), 1, 5, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string(""), 1, 10, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string(""), 1, 20, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcde"), 0, 0, '1', gl_string("abcde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 0, 5, '1', gl_string("11111abcde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 0, 10, '1', gl_string("1111111111abcde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 0, 20, '1', gl_string("11111111111111111111abcde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 1, 0, '1', gl_string("abcde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 1, 5, '1', gl_string("a11111bcde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 1, 10, '1', gl_string("a1111111111bcde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 1, 20, '1', gl_string("a11111111111111111111bcde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 2, 0, '1', gl_string("abcde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 2, 5, '1', gl_string("ab11111cde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 2, 10, '1', gl_string("ab1111111111cde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 2, 20, '1', gl_string("ab11111111111111111111cde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 4, 0, '1', gl_string("abcde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 4, 5, '1', gl_string("abcd11111e"));
    _test_string_insert_size_size_char(gl_string("abcde"), 4, 10, '1', gl_string("abcd1111111111e"));
    _test_string_insert_size_size_char(gl_string("abcde"), 4, 20, '1', gl_string("abcd11111111111111111111e"));
    _test_string_insert_size_size_char(gl_string("abcde"), 5, 0, '1', gl_string("abcde"));
    _test_string_insert_size_size_char(gl_string("abcde"), 5, 5, '1', gl_string("abcde11111"));
    _test_string_insert_size_size_char(gl_string("abcde"), 5, 10, '1', gl_string("abcde1111111111"));
    _test_string_insert_size_size_char(gl_string("abcde"), 5, 20, '1', gl_string("abcde11111111111111111111"));
    _test_string_insert_size_size_char(gl_string("abcde"), 6, 0, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcde"), 6, 5, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcde"), 6, 10, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcde"), 6, 20, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 0, 0, '1', gl_string("abcdefghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 0, 5, '1', gl_string("11111abcdefghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 0, 10, '1', gl_string("1111111111abcdefghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 0, 20, '1', gl_string("11111111111111111111abcdefghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 1, 0, '1', gl_string("abcdefghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 1, 5, '1', gl_string("a11111bcdefghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 1, 10, '1', gl_string("a1111111111bcdefghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 1, 20, '1', gl_string("a11111111111111111111bcdefghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 5, 0, '1', gl_string("abcdefghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 5, 5, '1', gl_string("abcde11111fghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 5, 10, '1', gl_string("abcde1111111111fghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 5, 20, '1', gl_string("abcde11111111111111111111fghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 9, 0, '1', gl_string("abcdefghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 9, 5, '1', gl_string("abcdefghi11111j"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 9, 10, '1', gl_string("abcdefghi1111111111j"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 9, 20, '1', gl_string("abcdefghi11111111111111111111j"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 10, 0, '1', gl_string("abcdefghij"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 10, 5, '1', gl_string("abcdefghij11111"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 10, 10, '1', gl_string("abcdefghij1111111111"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 10, 20, '1', gl_string("abcdefghij11111111111111111111"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 11, 0, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 11, 5, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 11, 10, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcdefghij"), 11, 20, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 0, '1', gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 5, '1', gl_string("11111abcdefghijklmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 10, '1', gl_string("1111111111abcdefghijklmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 20, '1', gl_string("11111111111111111111abcdefghijklmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 0, '1', gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 5, '1', gl_string("a11111bcdefghijklmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 10, '1', gl_string("a1111111111bcdefghijklmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 20, '1', gl_string("a11111111111111111111bcdefghijklmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 0, '1', gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 5, '1', gl_string("abcdefghij11111klmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 10, '1', gl_string("abcdefghij1111111111klmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 20, '1', gl_string("abcdefghij11111111111111111111klmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 0, '1', gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 5, '1', gl_string("abcdefghijklmnopqrs11111t"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 10, '1', gl_string("abcdefghijklmnopqrs1111111111t"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 20, '1', gl_string("abcdefghijklmnopqrs11111111111111111111t"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 0, '1', gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 5, '1', gl_string("abcdefghijklmnopqrst11111"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 10, '1', gl_string("abcdefghijklmnopqrst1111111111"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 20, '1', gl_string("abcdefghijklmnopqrst11111111111111111111"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 21, 0, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 21, 5, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 21, 10, '1', gl_string("can't happen"));
    _test_string_insert_size_size_char(gl_string("abcdefghijklmnopqrst"), 21, 20, '1', gl_string("can't happen"));
  }


  void
  _test_string_insert_size_string(gl_string s, size_t pos, gl_string str, gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.insert(pos, str);
        
      TS_ASSERT(pos <= old_size);
      TS_ASSERT(s == expected);
    }
    catch (...)
    {
      TS_ASSERT(pos > old_size);
      TS_ASSERT(s == s0);
    }
#endif
  }

  void test_string_insert_size_string()
  {
    _test_string_insert_size_string(gl_string(""), 0, gl_string(""), gl_string(""));
    _test_string_insert_size_string(gl_string(""), 0, gl_string("12345"), gl_string("12345"));
    _test_string_insert_size_string(gl_string(""), 0, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_insert_size_string(gl_string(""), 0, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_insert_size_string(gl_string(""), 1, gl_string(""), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string(""), 1, gl_string("12345"), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string(""), 1, gl_string("1234567890"), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string(""), 1, gl_string("12345678901234567890"), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcde"), 0, gl_string(""), gl_string("abcde"));
    _test_string_insert_size_string(gl_string("abcde"), 0, gl_string("12345"), gl_string("12345abcde"));
    _test_string_insert_size_string(gl_string("abcde"), 0, gl_string("1234567890"), gl_string("1234567890abcde"));
    _test_string_insert_size_string(gl_string("abcde"), 0, gl_string("12345678901234567890"), gl_string("12345678901234567890abcde"));
    _test_string_insert_size_string(gl_string("abcde"), 1, gl_string(""), gl_string("abcde"));
    _test_string_insert_size_string(gl_string("abcde"), 1, gl_string("12345"), gl_string("a12345bcde"));
    _test_string_insert_size_string(gl_string("abcde"), 1, gl_string("1234567890"), gl_string("a1234567890bcde"));
    _test_string_insert_size_string(gl_string("abcde"), 1, gl_string("12345678901234567890"), gl_string("a12345678901234567890bcde"));
    _test_string_insert_size_string(gl_string("abcde"), 2, gl_string(""), gl_string("abcde"));
    _test_string_insert_size_string(gl_string("abcde"), 2, gl_string("12345"), gl_string("ab12345cde"));
    _test_string_insert_size_string(gl_string("abcde"), 2, gl_string("1234567890"), gl_string("ab1234567890cde"));
    _test_string_insert_size_string(gl_string("abcde"), 2, gl_string("12345678901234567890"), gl_string("ab12345678901234567890cde"));
    _test_string_insert_size_string(gl_string("abcde"), 4, gl_string(""), gl_string("abcde"));
    _test_string_insert_size_string(gl_string("abcde"), 4, gl_string("12345"), gl_string("abcd12345e"));
    _test_string_insert_size_string(gl_string("abcde"), 4, gl_string("1234567890"), gl_string("abcd1234567890e"));
    _test_string_insert_size_string(gl_string("abcde"), 4, gl_string("12345678901234567890"), gl_string("abcd12345678901234567890e"));
    _test_string_insert_size_string(gl_string("abcde"), 5, gl_string(""), gl_string("abcde"));
    _test_string_insert_size_string(gl_string("abcde"), 5, gl_string("12345"), gl_string("abcde12345"));
    _test_string_insert_size_string(gl_string("abcde"), 5, gl_string("1234567890"), gl_string("abcde1234567890"));
    _test_string_insert_size_string(gl_string("abcde"), 5, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890"));
    _test_string_insert_size_string(gl_string("abcde"), 6, gl_string(""), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcde"), 6, gl_string("12345"), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcde"), 6, gl_string("1234567890"), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcde"), 6, gl_string("12345678901234567890"), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 0, gl_string(""), gl_string("abcdefghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 0, gl_string("12345"), gl_string("12345abcdefghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 0, gl_string("1234567890"), gl_string("1234567890abcdefghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), gl_string("12345678901234567890abcdefghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 1, gl_string(""), gl_string("abcdefghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 1, gl_string("12345"), gl_string("a12345bcdefghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 1, gl_string("1234567890"), gl_string("a1234567890bcdefghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), gl_string("a12345678901234567890bcdefghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 5, gl_string(""), gl_string("abcdefghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 5, gl_string("12345"), gl_string("abcde12345fghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 5, gl_string("1234567890"), gl_string("abcde1234567890fghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890fghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 9, gl_string(""), gl_string("abcdefghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 9, gl_string("12345"), gl_string("abcdefghi12345j"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 9, gl_string("1234567890"), gl_string("abcdefghi1234567890j"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), gl_string("abcdefghi12345678901234567890j"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 10, gl_string(""), gl_string("abcdefghij"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 10, gl_string("12345"), gl_string("abcdefghij12345"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 10, gl_string("1234567890"), gl_string("abcdefghij1234567890"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 11, gl_string(""), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 11, gl_string("12345"), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 11, gl_string("1234567890"), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 0, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), gl_string("12345abcdefghijklmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 1, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), gl_string("a12345678901234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 10, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), gl_string("abcdefghij12345klmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 19, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 20, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), gl_string("abcdefghijklmnopqrst12345"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 21, gl_string(""), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), gl_string("can't happen"));
    _test_string_insert_size_string(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), gl_string("can't happen"));
  }

  void
  _test_string_insert_size_string_size_size(gl_string s, size_t pos1, gl_string str, size_t pos2,
                                            size_t n, gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.insert(pos1, str, pos2, n);
        
      TS_ASSERT(pos1 <= old_size && pos2 <= str.size());
      TS_ASSERT(s == expected);
    }
    catch (...)
    {
      TS_ASSERT(pos1 > old_size || pos2 > str.size());
      TS_ASSERT(s == s0);
    }
#endif
  }


  void
  test_string_insert_size_string_size_size_npos(gl_string s, size_t pos1, gl_string str, size_t pos2, gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.insert(pos1, str, pos2);
        
      TS_ASSERT(pos1 <= old_size && pos2 <= str.size());
      TS_ASSERT(s == expected);
    }
    catch (...)
    {
      TS_ASSERT(pos1 > old_size || pos2 > str.size());
      TS_ASSERT(s == s0);
    }
#endif
  }



  void test_string_insert_size_string_size_size0()
  {
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string(""), 0, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string(""), 0, 1, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 0, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 0, 1, gl_string("1"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 0, 2, gl_string("12"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 0, 4, gl_string("1234"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 0, 5, gl_string("12345"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 0, 6, gl_string("12345"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 1, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 1, 1, gl_string("2"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 1, 2, gl_string("23"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 1, 3, gl_string("234"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 1, 4, gl_string("2345"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 1, 5, gl_string("2345"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 2, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 2, 1, gl_string("3"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 2, 2, gl_string("34"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 2, 3, gl_string("345"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 2, 4, gl_string("345"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 4, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 4, 1, gl_string("5"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 4, 2, gl_string("5"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 5, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 5, 1, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 0, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 0, 1, gl_string("1"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 0, 5, gl_string("12345"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 0, 9, gl_string("123456789"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 0, 10, gl_string("1234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 0, 11, gl_string("1234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 1, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 1, 1, gl_string("2"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 1, 4, gl_string("2345"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 1, 8, gl_string("23456789"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 1, 9, gl_string("234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 1, 10, gl_string("234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 5, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 5, 1, gl_string("6"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 5, 2, gl_string("67"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 5, 4, gl_string("6789"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 5, 5, gl_string("67890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 5, 6, gl_string("67890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 9, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 9, 1, gl_string("0"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 9, 2, gl_string("0"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 10, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 10, 1, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
  }


  void test_string_insert_size_string_size_size1()
  {
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 0, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 0, 1, gl_string("1"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 1, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 1, 1, gl_string("2"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 1, 9, gl_string("234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 10, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 10, 1, gl_string("1"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 10, 5, gl_string("12345"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 10, 9, gl_string("123456789"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 19, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 19, 1, gl_string("0"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 19, 2, gl_string("0"));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 20, 0, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 20, 1, gl_string(""));
    _test_string_insert_size_string_size_size(gl_string(""), 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string(""), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string(""), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 0, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 0, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 0, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 0, 6, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 1, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 1, 3, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 1, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 1, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 2, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 2, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 2, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 2, 3, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 2, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 4, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 4, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 4, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 5, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 5, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
  }


  void test_string_insert_size_string_size_size2()
  {
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 0, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 0, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 0, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 0, 11, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 1, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 1, 8, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 1, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 1, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 5, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 5, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 5, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 5, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 5, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 5, 6, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 9, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 9, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 9, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 10, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 10, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 0, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 0, 19, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 0, 20, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 0, 21, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 1, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 1, 18, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 1, 19, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 1, 20, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 10, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 10, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 10, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 10, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 10, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 10, 11, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 19, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 19, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 19, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 20, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 20, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string(""), 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string(""), 0, 1, gl_string("abcde"));
  }


  void test_string_insert_size_string_size_size3()
  {
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 0, 1, gl_string("1abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 0, 2, gl_string("12abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 0, 4, gl_string("1234abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 0, 5, gl_string("12345abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 0, 6, gl_string("12345abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 1, 1, gl_string("2abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 1, 2, gl_string("23abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 1, 3, gl_string("234abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 1, 4, gl_string("2345abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 1, 5, gl_string("2345abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 2, 1, gl_string("3abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 2, 2, gl_string("34abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 2, 3, gl_string("345abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 2, 4, gl_string("345abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 4, 1, gl_string("5abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 4, 2, gl_string("5abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 0, 1, gl_string("1abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 0, 5, gl_string("12345abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 0, 9, gl_string("123456789abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 0, 10, gl_string("1234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 0, 11, gl_string("1234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 1, 1, gl_string("2abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 1, 4, gl_string("2345abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 1, 8, gl_string("23456789abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 1, 9, gl_string("234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 1, 10, gl_string("234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 5, 1, gl_string("6abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 5, 2, gl_string("67abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 5, 4, gl_string("6789abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 5, 5, gl_string("67890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 5, 6, gl_string("67890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 9, 1, gl_string("0abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 9, 2, gl_string("0abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 0, 1, gl_string("1abcde"));
  }


  void test_string_insert_size_string_size_size4()
  {
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 1, 1, gl_string("2abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 1, 9, gl_string("234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 10, 1, gl_string("1abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 10, 5, gl_string("12345abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 10, 9, gl_string("123456789abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 19, 1, gl_string("0abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 19, 2, gl_string("0abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 0, 1, gl_string("a1bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 0, 2, gl_string("a12bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 0, 4, gl_string("a1234bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 0, 5, gl_string("a12345bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 0, 6, gl_string("a12345bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 1, 1, gl_string("a2bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 1, 2, gl_string("a23bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 1, 3, gl_string("a234bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 1, 4, gl_string("a2345bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 1, 5, gl_string("a2345bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 2, 1, gl_string("a3bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 2, 2, gl_string("a34bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 2, 3, gl_string("a345bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 2, 4, gl_string("a345bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 4, 1, gl_string("a5bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 4, 2, gl_string("a5bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 0, 1, gl_string("a1bcde"));
  }


  void test_string_insert_size_string_size_size5()
  {
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 0, 5, gl_string("a12345bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 0, 9, gl_string("a123456789bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 0, 10, gl_string("a1234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 0, 11, gl_string("a1234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 1, 1, gl_string("a2bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 1, 4, gl_string("a2345bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 1, 8, gl_string("a23456789bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 1, 9, gl_string("a234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 1, 10, gl_string("a234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 5, 1, gl_string("a6bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 5, 2, gl_string("a67bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 5, 4, gl_string("a6789bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 5, 5, gl_string("a67890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 5, 6, gl_string("a67890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 9, 1, gl_string("a0bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 9, 2, gl_string("a0bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 0, 1, gl_string("a1bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 1, 1, gl_string("a2bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 10, 1, gl_string("a1bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 10, 5, gl_string("a12345bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 19, 1, gl_string("a0bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 19, 2, gl_string("a0bcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 0, 0, gl_string("abcde"));
  }


  void test_string_insert_size_string_size_size6()
  {
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 0, 1, gl_string("ab1cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 0, 2, gl_string("ab12cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 0, 4, gl_string("ab1234cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 0, 5, gl_string("ab12345cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 0, 6, gl_string("ab12345cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 1, 1, gl_string("ab2cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 1, 2, gl_string("ab23cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 1, 3, gl_string("ab234cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 1, 4, gl_string("ab2345cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 1, 5, gl_string("ab2345cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 2, 1, gl_string("ab3cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 2, 2, gl_string("ab34cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 2, 3, gl_string("ab345cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 2, 4, gl_string("ab345cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 4, 1, gl_string("ab5cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 4, 2, gl_string("ab5cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 0, 1, gl_string("ab1cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 0, 5, gl_string("ab12345cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 0, 9, gl_string("ab123456789cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 0, 10, gl_string("ab1234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 0, 11, gl_string("ab1234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 1, 1, gl_string("ab2cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 1, 4, gl_string("ab2345cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 1, 8, gl_string("ab23456789cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 1, 9, gl_string("ab234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 1, 10, gl_string("ab234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 5, 1, gl_string("ab6cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 5, 2, gl_string("ab67cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 5, 4, gl_string("ab6789cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 5, 5, gl_string("ab67890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 5, 6, gl_string("ab67890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 9, 1, gl_string("ab0cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 9, 2, gl_string("ab0cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 0, 1, gl_string("ab1cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 0, 10, gl_string("ab1234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 0, 19, gl_string("ab1234567890123456789cde"));
  }


  void test_string_insert_size_string_size_size7()
  {
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 0, 20, gl_string("ab12345678901234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 0, 21, gl_string("ab12345678901234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 1, 1, gl_string("ab2cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 1, 9, gl_string("ab234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 1, 18, gl_string("ab234567890123456789cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 1, 19, gl_string("ab2345678901234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 1, 20, gl_string("ab2345678901234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 10, 1, gl_string("ab1cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 10, 5, gl_string("ab12345cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 10, 9, gl_string("ab123456789cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 10, 10, gl_string("ab1234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 10, 11, gl_string("ab1234567890cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 19, 1, gl_string("ab0cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 19, 2, gl_string("ab0cde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 2, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 0, 1, gl_string("abcd1e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 0, 2, gl_string("abcd12e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 0, 4, gl_string("abcd1234e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 0, 5, gl_string("abcd12345e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 0, 6, gl_string("abcd12345e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 1, 1, gl_string("abcd2e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 1, 2, gl_string("abcd23e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 1, 3, gl_string("abcd234e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 1, 4, gl_string("abcd2345e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 1, 5, gl_string("abcd2345e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 2, 1, gl_string("abcd3e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 2, 2, gl_string("abcd34e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 2, 3, gl_string("abcd345e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 2, 4, gl_string("abcd345e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 4, 1, gl_string("abcd5e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 4, 2, gl_string("abcd5e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 0, 1, gl_string("abcd1e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 0, 5, gl_string("abcd12345e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 0, 9, gl_string("abcd123456789e"));
  }


  void test_string_insert_size_string_size_size8()
  {
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 0, 10, gl_string("abcd1234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 0, 11, gl_string("abcd1234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 1, 1, gl_string("abcd2e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 1, 4, gl_string("abcd2345e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 1, 8, gl_string("abcd23456789e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 1, 9, gl_string("abcd234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 1, 10, gl_string("abcd234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 5, 1, gl_string("abcd6e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 5, 2, gl_string("abcd67e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 5, 4, gl_string("abcd6789e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 5, 5, gl_string("abcd67890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 5, 6, gl_string("abcd67890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 9, 1, gl_string("abcd0e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 9, 2, gl_string("abcd0e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 0, 1, gl_string("abcd1e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 0, 10, gl_string("abcd1234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 0, 19, gl_string("abcd1234567890123456789e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 0, 20, gl_string("abcd12345678901234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 0, 21, gl_string("abcd12345678901234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 1, 1, gl_string("abcd2e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 1, 9, gl_string("abcd234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 1, 18, gl_string("abcd234567890123456789e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 1, 19, gl_string("abcd2345678901234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 1, 20, gl_string("abcd2345678901234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 10, 1, gl_string("abcd1e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 10, 5, gl_string("abcd12345e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 10, 9, gl_string("abcd123456789e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 10, 10, gl_string("abcd1234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 10, 11, gl_string("abcd1234567890e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 19, 1, gl_string("abcd0e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 19, 2, gl_string("abcd0e"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 4, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 0, 1, gl_string("abcde1"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 0, 2, gl_string("abcde12"));
  }


  void test_string_insert_size_string_size_size9()
  {
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 0, 4, gl_string("abcde1234"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 0, 5, gl_string("abcde12345"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 0, 6, gl_string("abcde12345"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 1, 1, gl_string("abcde2"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 1, 2, gl_string("abcde23"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 1, 3, gl_string("abcde234"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 1, 4, gl_string("abcde2345"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 1, 5, gl_string("abcde2345"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 2, 1, gl_string("abcde3"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 2, 2, gl_string("abcde34"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 2, 3, gl_string("abcde345"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 2, 4, gl_string("abcde345"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 4, 1, gl_string("abcde5"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 4, 2, gl_string("abcde5"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 0, 1, gl_string("abcde1"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 0, 5, gl_string("abcde12345"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 0, 9, gl_string("abcde123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 0, 10, gl_string("abcde1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 0, 11, gl_string("abcde1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 1, 1, gl_string("abcde2"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 1, 4, gl_string("abcde2345"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 1, 8, gl_string("abcde23456789"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 1, 9, gl_string("abcde234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 1, 10, gl_string("abcde234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 5, 1, gl_string("abcde6"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 5, 2, gl_string("abcde67"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 5, 4, gl_string("abcde6789"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 5, 5, gl_string("abcde67890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 5, 6, gl_string("abcde67890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 9, 1, gl_string("abcde0"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 9, 2, gl_string("abcde0"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 0, 1, gl_string("abcde1"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 0, 10, gl_string("abcde1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 0, 19, gl_string("abcde1234567890123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 0, 20, gl_string("abcde12345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 0, 21, gl_string("abcde12345678901234567890"));
  }


  void test_string_insert_size_string_size_size10()
  {
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 1, 1, gl_string("abcde2"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 1, 9, gl_string("abcde234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 1, 18, gl_string("abcde234567890123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 1, 19, gl_string("abcde2345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 1, 20, gl_string("abcde2345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 10, 1, gl_string("abcde1"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 10, 5, gl_string("abcde12345"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 10, 9, gl_string("abcde123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 10, 10, gl_string("abcde1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 10, 11, gl_string("abcde1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 19, 1, gl_string("abcde0"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 19, 2, gl_string("abcde0"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 5, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string(""), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string(""), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 0, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 0, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 0, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 0, 6, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 1, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 1, 3, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 1, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 1, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 2, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 2, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 2, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 2, 3, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 2, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 4, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 4, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 4, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 5, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 5, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 0, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 0, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 0, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 0, 11, gl_string("can't happen"));
  }


  void test_string_insert_size_string_size_size11()
  {
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 1, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 1, 8, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 1, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 1, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 5, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 5, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 5, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 5, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 5, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 5, 6, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 9, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 9, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 9, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 10, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 10, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 0, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 0, 19, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 0, 20, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 0, 21, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 1, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 1, 18, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 1, 19, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 1, 20, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 10, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 10, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 10, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 10, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 10, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 10, 11, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 19, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 19, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 19, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 20, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 20, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcde"), 6, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 0, 1, gl_string("1abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 0, 2, gl_string("12abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 0, 4, gl_string("1234abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 0, 5, gl_string("12345abcdefghij"));
  }


  void test_string_insert_size_string_size_size12()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 0, 6, gl_string("12345abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 1, 1, gl_string("2abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 1, 2, gl_string("23abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 1, 3, gl_string("234abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 1, 4, gl_string("2345abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 1, 5, gl_string("2345abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 2, 1, gl_string("3abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 2, 2, gl_string("34abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 2, 3, gl_string("345abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 2, 4, gl_string("345abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 4, 1, gl_string("5abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 4, 2, gl_string("5abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 0, 1, gl_string("1abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 0, 5, gl_string("12345abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 0, 9, gl_string("123456789abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 0, 10, gl_string("1234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 0, 11, gl_string("1234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 1, 1, gl_string("2abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 1, 4, gl_string("2345abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 1, 8, gl_string("23456789abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 1, 9, gl_string("234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 1, 10, gl_string("234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 5, 1, gl_string("6abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 5, 2, gl_string("67abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 5, 4, gl_string("6789abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 5, 5, gl_string("67890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 5, 6, gl_string("67890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 9, 1, gl_string("0abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 9, 2, gl_string("0abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 0, 1, gl_string("1abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 1, 1, gl_string("2abcdefghij"));
  }


  void test_string_insert_size_string_size_size13()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 1, 9, gl_string("234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 10, 1, gl_string("1abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 10, 5, gl_string("12345abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 10, 9, gl_string("123456789abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 19, 1, gl_string("0abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 19, 2, gl_string("0abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 0, 1, gl_string("a1bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 0, 2, gl_string("a12bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 0, 4, gl_string("a1234bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 0, 5, gl_string("a12345bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 0, 6, gl_string("a12345bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 1, 1, gl_string("a2bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 1, 2, gl_string("a23bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 1, 3, gl_string("a234bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 1, 4, gl_string("a2345bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 1, 5, gl_string("a2345bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 2, 1, gl_string("a3bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 2, 2, gl_string("a34bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 2, 3, gl_string("a345bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 2, 4, gl_string("a345bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 4, 1, gl_string("a5bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 4, 2, gl_string("a5bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 0, 1, gl_string("a1bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 0, 5, gl_string("a12345bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 0, 9, gl_string("a123456789bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 0, 10, gl_string("a1234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 0, 11, gl_string("a1234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 1, 1, gl_string("a2bcdefghij"));
  }


  void test_string_insert_size_string_size_size14()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 1, 4, gl_string("a2345bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 1, 8, gl_string("a23456789bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 1, 9, gl_string("a234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 1, 10, gl_string("a234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 5, 1, gl_string("a6bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 5, 2, gl_string("a67bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 5, 4, gl_string("a6789bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 5, 5, gl_string("a67890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 5, 6, gl_string("a67890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 9, 1, gl_string("a0bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 9, 2, gl_string("a0bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 0, 1, gl_string("a1bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 1, 1, gl_string("a2bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 10, 1, gl_string("a1bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 10, 5, gl_string("a12345bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 19, 1, gl_string("a0bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 19, 2, gl_string("a0bcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 0, 1, gl_string("abcde1fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 0, 2, gl_string("abcde12fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 0, 4, gl_string("abcde1234fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 0, 5, gl_string("abcde12345fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 0, 6, gl_string("abcde12345fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
  }


  void test_string_insert_size_string_size_size15()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 1, 1, gl_string("abcde2fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 1, 2, gl_string("abcde23fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 1, 3, gl_string("abcde234fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 1, 4, gl_string("abcde2345fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 1, 5, gl_string("abcde2345fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 2, 1, gl_string("abcde3fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 2, 2, gl_string("abcde34fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 2, 3, gl_string("abcde345fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 2, 4, gl_string("abcde345fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 4, 1, gl_string("abcde5fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 4, 2, gl_string("abcde5fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 0, 1, gl_string("abcde1fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 0, 5, gl_string("abcde12345fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 0, 9, gl_string("abcde123456789fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 0, 10, gl_string("abcde1234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 0, 11, gl_string("abcde1234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 1, 1, gl_string("abcde2fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 1, 4, gl_string("abcde2345fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 1, 8, gl_string("abcde23456789fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 1, 9, gl_string("abcde234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 1, 10, gl_string("abcde234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 5, 1, gl_string("abcde6fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 5, 2, gl_string("abcde67fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 5, 4, gl_string("abcde6789fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 5, 5, gl_string("abcde67890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 5, 6, gl_string("abcde67890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 9, 1, gl_string("abcde0fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 9, 2, gl_string("abcde0fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 0, 1, gl_string("abcde1fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 0, 10, gl_string("abcde1234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 0, 19, gl_string("abcde1234567890123456789fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 0, 20, gl_string("abcde12345678901234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 0, 21, gl_string("abcde12345678901234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 1, 1, gl_string("abcde2fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 1, 9, gl_string("abcde234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 1, 18, gl_string("abcde234567890123456789fghij"));
  }


  void test_string_insert_size_string_size_size16()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 1, 19, gl_string("abcde2345678901234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 1, 20, gl_string("abcde2345678901234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 10, 1, gl_string("abcde1fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 10, 5, gl_string("abcde12345fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 10, 9, gl_string("abcde123456789fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 10, 10, gl_string("abcde1234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 10, 11, gl_string("abcde1234567890fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 19, 1, gl_string("abcde0fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 19, 2, gl_string("abcde0fghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 5, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 0, 1, gl_string("abcdefghi1j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 0, 2, gl_string("abcdefghi12j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 0, 4, gl_string("abcdefghi1234j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 0, 5, gl_string("abcdefghi12345j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 0, 6, gl_string("abcdefghi12345j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 1, 1, gl_string("abcdefghi2j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 1, 2, gl_string("abcdefghi23j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 1, 3, gl_string("abcdefghi234j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 1, 4, gl_string("abcdefghi2345j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 1, 5, gl_string("abcdefghi2345j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 2, 1, gl_string("abcdefghi3j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 2, 2, gl_string("abcdefghi34j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 2, 3, gl_string("abcdefghi345j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 2, 4, gl_string("abcdefghi345j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 4, 1, gl_string("abcdefghi5j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 4, 2, gl_string("abcdefghi5j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 0, 1, gl_string("abcdefghi1j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 0, 5, gl_string("abcdefghi12345j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 0, 9, gl_string("abcdefghi123456789j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 0, 10, gl_string("abcdefghi1234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 0, 11, gl_string("abcdefghi1234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 1, 1, gl_string("abcdefghi2j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 1, 4, gl_string("abcdefghi2345j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 1, 8, gl_string("abcdefghi23456789j"));
  }


  void test_string_insert_size_string_size_size17()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 1, 9, gl_string("abcdefghi234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 1, 10, gl_string("abcdefghi234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 5, 1, gl_string("abcdefghi6j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 5, 2, gl_string("abcdefghi67j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 5, 4, gl_string("abcdefghi6789j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 5, 5, gl_string("abcdefghi67890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 5, 6, gl_string("abcdefghi67890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 9, 1, gl_string("abcdefghi0j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 9, 2, gl_string("abcdefghi0j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghi1j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghi1234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghi1234567890123456789j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghi12345678901234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghi12345678901234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghi2j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghi234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghi234567890123456789j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghi2345678901234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghi2345678901234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghi1j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghi12345j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghi123456789j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghi1234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghi1234567890j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghi0j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghi0j"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 9, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 0, 1, gl_string("abcdefghij1"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 0, 2, gl_string("abcdefghij12"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 0, 4, gl_string("abcdefghij1234"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 0, 5, gl_string("abcdefghij12345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 0, 6, gl_string("abcdefghij12345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 1, 1, gl_string("abcdefghij2"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 1, 2, gl_string("abcdefghij23"));
  }


  void test_string_insert_size_string_size_size18()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 1, 3, gl_string("abcdefghij234"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 1, 4, gl_string("abcdefghij2345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 1, 5, gl_string("abcdefghij2345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 2, 1, gl_string("abcdefghij3"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 2, 2, gl_string("abcdefghij34"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 2, 3, gl_string("abcdefghij345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 2, 4, gl_string("abcdefghij345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 4, 1, gl_string("abcdefghij5"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 4, 2, gl_string("abcdefghij5"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 0, 1, gl_string("abcdefghij1"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 0, 5, gl_string("abcdefghij12345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 0, 9, gl_string("abcdefghij123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 0, 10, gl_string("abcdefghij1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 0, 11, gl_string("abcdefghij1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 1, 1, gl_string("abcdefghij2"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 1, 4, gl_string("abcdefghij2345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 1, 8, gl_string("abcdefghij23456789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 1, 9, gl_string("abcdefghij234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 1, 10, gl_string("abcdefghij234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 5, 1, gl_string("abcdefghij6"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 5, 2, gl_string("abcdefghij67"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 5, 4, gl_string("abcdefghij6789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 5, 5, gl_string("abcdefghij67890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 5, 6, gl_string("abcdefghij67890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 9, 1, gl_string("abcdefghij0"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 9, 2, gl_string("abcdefghij0"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghij1"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghij1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghij1234567890123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghij12345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghij12345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghij2"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghij234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghij234567890123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghij2345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghij2345678901234567890"));
  }


  void test_string_insert_size_string_size_size19()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghij1"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghij12345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghij123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghij1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghij1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghij0"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghij0"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 10, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string(""), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string(""), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 0, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 0, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 0, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 0, 6, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 1, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 1, 3, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 1, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 1, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 2, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 2, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 2, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 2, 3, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 2, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 4, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 4, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 4, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 5, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 5, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 0, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 0, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 0, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 0, 11, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 1, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 1, 8, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 1, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 1, 10, gl_string("can't happen"));
  }


  void test_string_insert_size_string_size_size20()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 5, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 5, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 5, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 5, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 5, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 5, 6, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 9, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 9, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 9, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 10, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 10, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 0, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 0, 19, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 0, 20, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 0, 21, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 1, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 1, 18, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 1, 19, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 1, 20, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 10, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 10, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 10, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 10, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 10, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 10, 11, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 19, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 19, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 19, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 20, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 20, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghij"), 11, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 0, 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 0, 2, gl_string("12abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 0, 4, gl_string("1234abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 0, 5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 0, 6, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 1, 1, gl_string("2abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 1, 2, gl_string("23abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 1, 3, gl_string("234abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 1, 4, gl_string("2345abcdefghijklmnopqrst"));
  }


  void test_string_insert_size_string_size_size21()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 1, 5, gl_string("2345abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 2, 1, gl_string("3abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 2, 2, gl_string("34abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 2, 3, gl_string("345abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 2, 4, gl_string("345abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 4, 1, gl_string("5abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 4, 2, gl_string("5abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 0, 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 0, 5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 0, 9, gl_string("123456789abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 0, 10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 0, 11, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 1, 1, gl_string("2abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 1, 4, gl_string("2345abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 1, 8, gl_string("23456789abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 1, 9, gl_string("234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 1, 10, gl_string("234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 5, 1, gl_string("6abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 5, 2, gl_string("67abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 5, 4, gl_string("6789abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 5, 5, gl_string("67890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 5, 6, gl_string("67890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 9, 1, gl_string("0abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 9, 2, gl_string("0abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 0, 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 1, 1, gl_string("2abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 1, 9, gl_string("234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 10, 1, gl_string("1abcdefghijklmnopqrst"));
  }


  void test_string_insert_size_string_size_size22()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 10, 5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 10, 9, gl_string("123456789abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 19, 1, gl_string("0abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 19, 2, gl_string("0abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 0, 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 0, 2, gl_string("a12bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 0, 4, gl_string("a1234bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 0, 5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 0, 6, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 1, 1, gl_string("a2bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 1, 2, gl_string("a23bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 1, 3, gl_string("a234bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 1, 4, gl_string("a2345bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 1, 5, gl_string("a2345bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 2, 1, gl_string("a3bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 2, 2, gl_string("a34bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 2, 3, gl_string("a345bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 2, 4, gl_string("a345bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 4, 1, gl_string("a5bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 4, 2, gl_string("a5bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 0, 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 0, 5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 0, 9, gl_string("a123456789bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 0, 10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 0, 11, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 1, 1, gl_string("a2bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 1, 4, gl_string("a2345bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 1, 8, gl_string("a23456789bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 1, 9, gl_string("a234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 1, 10, gl_string("a234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 5, 1, gl_string("a6bcdefghijklmnopqrst"));
  }


  void test_string_insert_size_string_size_size23()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 5, 2, gl_string("a67bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 5, 4, gl_string("a6789bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 5, 5, gl_string("a67890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 5, 6, gl_string("a67890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 9, 1, gl_string("a0bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 9, 2, gl_string("a0bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 0, 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 1, 1, gl_string("a2bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 10, 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 10, 5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 19, 1, gl_string("a0bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 19, 2, gl_string("a0bcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 0, 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 0, 2, gl_string("abcdefghij12klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 0, 4, gl_string("abcdefghij1234klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 0, 5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 0, 6, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 1, 1, gl_string("abcdefghij2klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 1, 2, gl_string("abcdefghij23klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 1, 3, gl_string("abcdefghij234klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 1, 4, gl_string("abcdefghij2345klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 1, 5, gl_string("abcdefghij2345klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrst"));
  }


  void test_string_insert_size_string_size_size24()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 2, 1, gl_string("abcdefghij3klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 2, 2, gl_string("abcdefghij34klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 2, 3, gl_string("abcdefghij345klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 2, 4, gl_string("abcdefghij345klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 4, 1, gl_string("abcdefghij5klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 4, 2, gl_string("abcdefghij5klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 0, 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 0, 5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 0, 9, gl_string("abcdefghij123456789klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 0, 10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 0, 11, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 1, 1, gl_string("abcdefghij2klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 1, 4, gl_string("abcdefghij2345klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 1, 8, gl_string("abcdefghij23456789klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 1, 9, gl_string("abcdefghij234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 1, 10, gl_string("abcdefghij234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 5, 1, gl_string("abcdefghij6klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 5, 2, gl_string("abcdefghij67klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 5, 4, gl_string("abcdefghij6789klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 5, 5, gl_string("abcdefghij67890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 5, 6, gl_string("abcdefghij67890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 9, 1, gl_string("abcdefghij0klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 9, 2, gl_string("abcdefghij0klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghij1234567890123456789klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghij2klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghij234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghij234567890123456789klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghij2345678901234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghij2345678901234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghij123456789klmnopqrst"));
  }


  void test_string_insert_size_string_size_size25()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghij0klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghij0klmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 0, 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 0, 2, gl_string("abcdefghijklmnopqrs12t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 0, 4, gl_string("abcdefghijklmnopqrs1234t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 0, 5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 0, 6, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 1, 1, gl_string("abcdefghijklmnopqrs2t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 1, 2, gl_string("abcdefghijklmnopqrs23t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 1, 3, gl_string("abcdefghijklmnopqrs234t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 1, 4, gl_string("abcdefghijklmnopqrs2345t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 1, 5, gl_string("abcdefghijklmnopqrs2345t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 2, 1, gl_string("abcdefghijklmnopqrs3t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 2, 2, gl_string("abcdefghijklmnopqrs34t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 2, 3, gl_string("abcdefghijklmnopqrs345t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 2, 4, gl_string("abcdefghijklmnopqrs345t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 4, 1, gl_string("abcdefghijklmnopqrs5t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 4, 2, gl_string("abcdefghijklmnopqrs5t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 0, 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 0, 5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 0, 9, gl_string("abcdefghijklmnopqrs123456789t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 0, 10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 0, 11, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 1, 1, gl_string("abcdefghijklmnopqrs2t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 1, 4, gl_string("abcdefghijklmnopqrs2345t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 1, 8, gl_string("abcdefghijklmnopqrs23456789t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 1, 9, gl_string("abcdefghijklmnopqrs234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 1, 10, gl_string("abcdefghijklmnopqrs234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 5, 1, gl_string("abcdefghijklmnopqrs6t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 5, 2, gl_string("abcdefghijklmnopqrs67t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 5, 4, gl_string("abcdefghijklmnopqrs6789t"));
  }


  void test_string_insert_size_string_size_size26()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 5, 5, gl_string("abcdefghijklmnopqrs67890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 5, 6, gl_string("abcdefghijklmnopqrs67890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 9, 1, gl_string("abcdefghijklmnopqrs0t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 9, 2, gl_string("abcdefghijklmnopqrs0t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghijklmnopqrs1234567890123456789t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghijklmnopqrs2t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghijklmnopqrs234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghijklmnopqrs234567890123456789t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghijklmnopqrs2345678901234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghijklmnopqrs2345678901234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghijklmnopqrs123456789t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghijklmnopqrs0t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghijklmnopqrs0t"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 0, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 0, 2, gl_string("abcdefghijklmnopqrst12"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 0, 4, gl_string("abcdefghijklmnopqrst1234"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 0, 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 0, 6, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 1, 1, gl_string("abcdefghijklmnopqrst2"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 1, 2, gl_string("abcdefghijklmnopqrst23"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 1, 3, gl_string("abcdefghijklmnopqrst234"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 1, 4, gl_string("abcdefghijklmnopqrst2345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 1, 5, gl_string("abcdefghijklmnopqrst2345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 2, 1, gl_string("abcdefghijklmnopqrst3"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 2, 2, gl_string("abcdefghijklmnopqrst34"));
  }


  void test_string_insert_size_string_size_size27()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 2, 3, gl_string("abcdefghijklmnopqrst345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 2, 4, gl_string("abcdefghijklmnopqrst345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 4, 1, gl_string("abcdefghijklmnopqrst5"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 4, 2, gl_string("abcdefghijklmnopqrst5"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 0, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 0, 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 0, 9, gl_string("abcdefghijklmnopqrst123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 0, 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 0, 11, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 1, 1, gl_string("abcdefghijklmnopqrst2"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 1, 4, gl_string("abcdefghijklmnopqrst2345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 1, 8, gl_string("abcdefghijklmnopqrst23456789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 1, 9, gl_string("abcdefghijklmnopqrst234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 1, 10, gl_string("abcdefghijklmnopqrst234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 5, 1, gl_string("abcdefghijklmnopqrst6"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 5, 2, gl_string("abcdefghijklmnopqrst67"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 5, 4, gl_string("abcdefghijklmnopqrst6789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 5, 5, gl_string("abcdefghijklmnopqrst67890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 5, 6, gl_string("abcdefghijklmnopqrst67890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 9, 1, gl_string("abcdefghijklmnopqrst0"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 9, 2, gl_string("abcdefghijklmnopqrst0"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghijklmnopqrst1234567890123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghijklmnopqrst2"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghijklmnopqrst234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghijklmnopqrst234567890123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghijklmnopqrst2345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghijklmnopqrst2345678901234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghijklmnopqrst123456789"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghijklmnopqrst1234567890"));
  }


  void test_string_insert_size_string_size_size28()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghijklmnopqrst0"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghijklmnopqrst0"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string(""), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string(""), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 0, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 0, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 0, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 0, 6, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 1, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 1, 3, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 1, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 1, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 2, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 2, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 2, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 2, 3, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 2, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 4, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 4, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 4, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 5, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 5, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 0, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 0, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 0, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 0, 11, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 1, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 1, 8, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 1, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 1, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 5, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 5, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 5, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 5, 4, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 5, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 5, 6, gl_string("can't happen"));
  }


  void test_string_insert_size_string_size_size29()
  {
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 9, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 9, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 9, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 10, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 10, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 0, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 0, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 0, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 0, 19, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 0, 20, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 0, 21, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 1, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 1, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 1, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 1, 18, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 1, 19, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 1, 20, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 10, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 10, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 10, 5, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 10, 9, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 10, 10, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 10, 11, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 19, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 19, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 19, 2, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 20, 0, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 20, 1, gl_string("can't happen"));
    _test_string_insert_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
  }


  void test_string_insert_size_string_size_size30()
  {
    test_string_insert_size_string_size_size_npos(gl_string(""), 0, gl_string("12345678901234567890"),  0, gl_string("12345678901234567890"));
    test_string_insert_size_string_size_size_npos(gl_string(""), 0, gl_string("12345678901234567890"),  1, gl_string( "2345678901234567890"));
    test_string_insert_size_string_size_size_npos(gl_string(""), 0, gl_string("12345678901234567890"),  2, gl_string(  "345678901234567890"));
    test_string_insert_size_string_size_size_npos(gl_string(""), 0, gl_string("12345678901234567890"),  3, gl_string(   "45678901234567890"));
    test_string_insert_size_string_size_size_npos(gl_string(""), 0, gl_string("12345678901234567890"),  5, gl_string(     "678901234567890"));
    test_string_insert_size_string_size_size_npos(gl_string(""), 0, gl_string("12345678901234567890"), 10, gl_string(          "1234567890"));
    test_string_insert_size_string_size_size_npos(gl_string(""), 0, gl_string("12345678901234567890"), 21, gl_string("can't happen"));
    test_string_insert_size_string_size_size_npos(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 0, gl_string("abcdefghij12345klmnopqrst"));
    test_string_insert_size_string_size_size_npos(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 1, gl_string("abcdefghij2345klmnopqrst"));
    test_string_insert_size_string_size_size_npos(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 3, gl_string("abcdefghij45klmnopqrst"));
    test_string_insert_size_string_size_size_npos(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 5, gl_string("abcdefghijklmnopqrst"));
    test_string_insert_size_string_size_size_npos(gl_string("abcdefghijklmnopqrst"), 10, gl_string("12345"), 6, gl_string("can't happen"));
  }

  void
  _test_string_op_plus_equal_char(gl_string s, char str, gl_string expected)
  {
    s += str;
    
    TS_ASSERT(s == expected);
  }

  void test_string_op_plus_equal_char()
  {
    _test_string_op_plus_equal_char(gl_string(), 'a', gl_string("a"));
    _test_string_op_plus_equal_char(gl_string("12345"), 'a', gl_string("12345a"));
    _test_string_op_plus_equal_char(gl_string("1234567890"), 'a', gl_string("1234567890a"));
    _test_string_op_plus_equal_char(gl_string("12345678901234567890"), 'a', gl_string("12345678901234567890a"));
  }

  void test_string_op_plus_equal_initializer_list()
  {
    gl_string s("123");
    s += {'a', 'b', 'c'};
    TS_ASSERT(s == "123abc");
  }

  void
  _test_string_op_plus_equal_pointer(gl_string s, const char* str, gl_string expected)
  {
    s += str;
    
    TS_ASSERT(s == expected);
  }

  void test_string_op_plus_equal_pointer()
  {
    _test_string_op_plus_equal_pointer(gl_string(), "", gl_string());
    _test_string_op_plus_equal_pointer(gl_string(), "12345", gl_string("12345"));
    _test_string_op_plus_equal_pointer(gl_string(), "1234567890", gl_string("1234567890"));
    _test_string_op_plus_equal_pointer(gl_string(), "12345678901234567890", gl_string("12345678901234567890"));

    _test_string_op_plus_equal_pointer(gl_string("12345"), "", gl_string("12345"));
    _test_string_op_plus_equal_pointer(gl_string("12345"), "12345", gl_string("1234512345"));
    _test_string_op_plus_equal_pointer(gl_string("12345"), "1234567890", gl_string("123451234567890"));
    _test_string_op_plus_equal_pointer(gl_string("12345"), "12345678901234567890", gl_string("1234512345678901234567890"));

    _test_string_op_plus_equal_pointer(gl_string("1234567890"), "", gl_string("1234567890"));
    _test_string_op_plus_equal_pointer(gl_string("1234567890"), "12345", gl_string("123456789012345"));
    _test_string_op_plus_equal_pointer(gl_string("1234567890"), "1234567890", gl_string("12345678901234567890"));
    _test_string_op_plus_equal_pointer(gl_string("1234567890"), "12345678901234567890", gl_string("123456789012345678901234567890"));

    _test_string_op_plus_equal_pointer(gl_string("12345678901234567890"), "", gl_string("12345678901234567890"));
    _test_string_op_plus_equal_pointer(gl_string("12345678901234567890"), "12345", gl_string("1234567890123456789012345"));
    _test_string_op_plus_equal_pointer(gl_string("12345678901234567890"), "1234567890", gl_string("123456789012345678901234567890"));
    _test_string_op_plus_equal_pointer(gl_string("12345678901234567890"), "12345678901234567890",
                                       gl_string("1234567890123456789012345678901234567890"));
  }


  void
  _test_string_op_plus_equal_string(gl_string s, gl_string str, gl_string expected)
  {
    s += str;
    
    TS_ASSERT(s == expected);
  }

  void test_string_op_plus_equal_string()
  {
    _test_string_op_plus_equal_string(gl_string(), gl_string(), gl_string());
    _test_string_op_plus_equal_string(gl_string(), gl_string("12345"), gl_string("12345"));
    _test_string_op_plus_equal_string(gl_string(), gl_string("1234567890"), gl_string("1234567890"));
    _test_string_op_plus_equal_string(gl_string(), gl_string("12345678901234567890"), gl_string("12345678901234567890"));

    _test_string_op_plus_equal_string(gl_string("12345"), gl_string(), gl_string("12345"));
    _test_string_op_plus_equal_string(gl_string("12345"), gl_string("12345"), gl_string("1234512345"));
    _test_string_op_plus_equal_string(gl_string("12345"), gl_string("1234567890"), gl_string("123451234567890"));
    _test_string_op_plus_equal_string(gl_string("12345"), gl_string("12345678901234567890"), gl_string("1234512345678901234567890"));

    _test_string_op_plus_equal_string(gl_string("1234567890"), gl_string(), gl_string("1234567890"));
    _test_string_op_plus_equal_string(gl_string("1234567890"), gl_string("12345"), gl_string("123456789012345"));
    _test_string_op_plus_equal_string(gl_string("1234567890"), gl_string("1234567890"), gl_string("12345678901234567890"));
    _test_string_op_plus_equal_string(gl_string("1234567890"), gl_string("12345678901234567890"), gl_string("123456789012345678901234567890"));

    _test_string_op_plus_equal_string(gl_string("12345678901234567890"), gl_string(), gl_string("12345678901234567890"));
    _test_string_op_plus_equal_string(gl_string("12345678901234567890"), gl_string("12345"), gl_string("1234567890123456789012345"));
    _test_string_op_plus_equal_string(gl_string("12345678901234567890"), gl_string("1234567890"), gl_string("123456789012345678901234567890"));
    _test_string_op_plus_equal_string(gl_string("12345678901234567890"), gl_string("12345678901234567890"),
                                      gl_string("1234567890123456789012345678901234567890"));
  }

  void test_string_replace_iter_iter_initializer_list()
  {
    gl_string s("123def456");
    s.replace(s.cbegin() + 3, s.cbegin() + 6, {'a', 'b', 'c'});
    TS_ASSERT(s == "123abc456");
  }
  template < class It>
  void
  _test_string_replace_iter_iter_iter_iter(gl_string s, size_t pos1, size_t n1, It f, It l, gl_string expected)
  {
    size_t old_size = s.size();
    gl_string::const_iterator first = s.begin() + pos1;
    gl_string::const_iterator last = s.begin() + pos1 + n1;
    size_t xlen = last - first;
    s.replace(first, last, f, l);
    
    ASSERT_TRUE(s == expected);
    size_t rlen = std::distance(f, l);
    TS_ASSERT(s.size() == old_size - xlen + rlen);
  }

  void test_string_replace_iter_iter_iter_iter0()
  {
    static const char* str = "12345678901234567890";
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+2, gl_string("12"));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+4, gl_string("1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+5, gl_string("12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+5, gl_string("12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+9, gl_string("123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+10, gl_string("1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+10, gl_string("1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+19, gl_string("1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string(""), 0, 0, str, str+20, gl_string("12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+1, gl_string("1abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+2, gl_string("12abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+4, gl_string("1234abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+5, gl_string("12345abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+1, gl_string("1abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+5, gl_string("12345abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+9, gl_string("123456789abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+10, gl_string("1234567890abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+1, gl_string("1abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+10, gl_string("1234567890abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+19, gl_string("1234567890123456789abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 0, str, str+20, gl_string("12345678901234567890abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+0, gl_string("bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+0, gl_string("bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+1, gl_string("1bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+2, gl_string("12bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+4, gl_string("1234bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+5, gl_string("12345bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+0, gl_string("bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+1, gl_string("1bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+5, gl_string("12345bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+9, gl_string("123456789bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+10, gl_string("1234567890bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+0, gl_string("bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+1, gl_string("1bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+10, gl_string("1234567890bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+19, gl_string("1234567890123456789bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 1, str, str+20, gl_string("12345678901234567890bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+0, gl_string("cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+0, gl_string("cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+1, gl_string("1cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+2, gl_string("12cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+4, gl_string("1234cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+5, gl_string("12345cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+0, gl_string("cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+1, gl_string("1cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+5, gl_string("12345cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+9, gl_string("123456789cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+10, gl_string("1234567890cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+0, gl_string("cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+1, gl_string("1cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+10, gl_string("1234567890cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+19, gl_string("1234567890123456789cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 2, str, str+20, gl_string("12345678901234567890cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+0, gl_string("e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+0, gl_string("e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+1, gl_string("1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+2, gl_string("12e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+4, gl_string("1234e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+5, gl_string("12345e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+0, gl_string("e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+1, gl_string("1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+5, gl_string("12345e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+9, gl_string("123456789e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+10, gl_string("1234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+0, gl_string("e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+1, gl_string("1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+10, gl_string("1234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+19, gl_string("1234567890123456789e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 4, str, str+20, gl_string("12345678901234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+2, gl_string("12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+4, gl_string("1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+5, gl_string("12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+5, gl_string("12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+9, gl_string("123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+10, gl_string("1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+10, gl_string("1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+19, gl_string("1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 0, 5, str, str+20, gl_string("12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+1, gl_string("a1bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+2, gl_string("a12bcde"));
  }


  void test_string_replace_iter_iter_iter_iter1()
  {
    static const char* str = "12345678901234567890";
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+4, gl_string("a1234bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+5, gl_string("a12345bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+1, gl_string("a1bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+5, gl_string("a12345bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+9, gl_string("a123456789bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+10, gl_string("a1234567890bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+1, gl_string("a1bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+10, gl_string("a1234567890bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+19, gl_string("a1234567890123456789bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 0, str, str+20, gl_string("a12345678901234567890bcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+0, gl_string("acde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+0, gl_string("acde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+1, gl_string("a1cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+2, gl_string("a12cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+4, gl_string("a1234cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+5, gl_string("a12345cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+0, gl_string("acde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+1, gl_string("a1cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+5, gl_string("a12345cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+9, gl_string("a123456789cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+10, gl_string("a1234567890cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+0, gl_string("acde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+1, gl_string("a1cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+10, gl_string("a1234567890cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+19, gl_string("a1234567890123456789cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 1, str, str+20, gl_string("a12345678901234567890cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+0, gl_string("ade"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+0, gl_string("ade"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+1, gl_string("a1de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+2, gl_string("a12de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+4, gl_string("a1234de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+5, gl_string("a12345de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+0, gl_string("ade"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+1, gl_string("a1de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+5, gl_string("a12345de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+9, gl_string("a123456789de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+10, gl_string("a1234567890de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+0, gl_string("ade"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+1, gl_string("a1de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+10, gl_string("a1234567890de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+19, gl_string("a1234567890123456789de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 2, str, str+20, gl_string("a12345678901234567890de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+0, gl_string("ae"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+0, gl_string("ae"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+1, gl_string("a1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+2, gl_string("a12e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+4, gl_string("a1234e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+5, gl_string("a12345e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+0, gl_string("ae"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+1, gl_string("a1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+5, gl_string("a12345e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+9, gl_string("a123456789e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+10, gl_string("a1234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+0, gl_string("ae"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+1, gl_string("a1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+10, gl_string("a1234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+19, gl_string("a1234567890123456789e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 3, str, str+20, gl_string("a12345678901234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+1, gl_string("a1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+2, gl_string("a12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+4, gl_string("a1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+5, gl_string("a12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+1, gl_string("a1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+5, gl_string("a12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+9, gl_string("a123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+10, gl_string("a1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+1, gl_string("a1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+10, gl_string("a1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+19, gl_string("a1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 1, 4, str, str+20, gl_string("a12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+1, gl_string("ab1cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+2, gl_string("ab12cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+4, gl_string("ab1234cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+5, gl_string("ab12345cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+1, gl_string("ab1cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+5, gl_string("ab12345cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+9, gl_string("ab123456789cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+10, gl_string("ab1234567890cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+1, gl_string("ab1cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+10, gl_string("ab1234567890cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+19, gl_string("ab1234567890123456789cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 0, str, str+20, gl_string("ab12345678901234567890cde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+0, gl_string("abde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+0, gl_string("abde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+1, gl_string("ab1de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+2, gl_string("ab12de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+4, gl_string("ab1234de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+5, gl_string("ab12345de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+0, gl_string("abde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+1, gl_string("ab1de"));
  }


  void test_string_replace_iter_iter_iter_iter2()
  {
    static const char* str = "12345678901234567890";
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+5, gl_string("ab12345de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+9, gl_string("ab123456789de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+10, gl_string("ab1234567890de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+0, gl_string("abde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+1, gl_string("ab1de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+10, gl_string("ab1234567890de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+19, gl_string("ab1234567890123456789de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 1, str, str+20, gl_string("ab12345678901234567890de"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+0, gl_string("abe"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+0, gl_string("abe"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+1, gl_string("ab1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+2, gl_string("ab12e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+4, gl_string("ab1234e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+5, gl_string("ab12345e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+0, gl_string("abe"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+1, gl_string("ab1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+5, gl_string("ab12345e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+9, gl_string("ab123456789e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+10, gl_string("ab1234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+0, gl_string("abe"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+1, gl_string("ab1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+10, gl_string("ab1234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+19, gl_string("ab1234567890123456789e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 2, str, str+20, gl_string("ab12345678901234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+0, gl_string("ab"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+0, gl_string("ab"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+1, gl_string("ab1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+2, gl_string("ab12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+4, gl_string("ab1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+5, gl_string("ab12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+0, gl_string("ab"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+1, gl_string("ab1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+5, gl_string("ab12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+9, gl_string("ab123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+10, gl_string("ab1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+0, gl_string("ab"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+1, gl_string("ab1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+10, gl_string("ab1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+19, gl_string("ab1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 2, 3, str, str+20, gl_string("ab12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+1, gl_string("abcd1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+2, gl_string("abcd12e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+4, gl_string("abcd1234e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+5, gl_string("abcd12345e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+1, gl_string("abcd1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+5, gl_string("abcd12345e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+9, gl_string("abcd123456789e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+10, gl_string("abcd1234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+1, gl_string("abcd1e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+10, gl_string("abcd1234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+19, gl_string("abcd1234567890123456789e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 0, str, str+20, gl_string("abcd12345678901234567890e"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+0, gl_string("abcd"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+0, gl_string("abcd"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+1, gl_string("abcd1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+2, gl_string("abcd12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+4, gl_string("abcd1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+5, gl_string("abcd12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+0, gl_string("abcd"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+1, gl_string("abcd1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+5, gl_string("abcd12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+9, gl_string("abcd123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+10, gl_string("abcd1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+0, gl_string("abcd"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+1, gl_string("abcd1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+10, gl_string("abcd1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+19, gl_string("abcd1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 4, 1, str, str+20, gl_string("abcd12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+1, gl_string("abcde1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+2, gl_string("abcde12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+4, gl_string("abcde1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+5, gl_string("abcde12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+1, gl_string("abcde1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+5, gl_string("abcde12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+9, gl_string("abcde123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+10, gl_string("abcde1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+1, gl_string("abcde1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+10, gl_string("abcde1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+19, gl_string("abcde1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcde"), 5, 0, str, str+20, gl_string("abcde12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+1, gl_string("1abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+2, gl_string("12abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+4, gl_string("1234abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+5, gl_string("12345abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+1, gl_string("1abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+5, gl_string("12345abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+9, gl_string("123456789abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+10, gl_string("1234567890abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+0, gl_string("abcdefghij"));
  }


  void test_string_replace_iter_iter_iter_iter3()
  {
    static const char* str = "12345678901234567890";
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+1, gl_string("1abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+10, gl_string("1234567890abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+19, gl_string("1234567890123456789abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 0, str, str+20, gl_string("12345678901234567890abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+0, gl_string("bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+0, gl_string("bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+1, gl_string("1bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+2, gl_string("12bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+4, gl_string("1234bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+5, gl_string("12345bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+0, gl_string("bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+1, gl_string("1bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+5, gl_string("12345bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+9, gl_string("123456789bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+10, gl_string("1234567890bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+0, gl_string("bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+1, gl_string("1bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+10, gl_string("1234567890bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+19, gl_string("1234567890123456789bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 1, str, str+20, gl_string("12345678901234567890bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+0, gl_string("fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+0, gl_string("fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+1, gl_string("1fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+2, gl_string("12fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+4, gl_string("1234fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+5, gl_string("12345fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+0, gl_string("fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+1, gl_string("1fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+5, gl_string("12345fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+9, gl_string("123456789fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+10, gl_string("1234567890fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+0, gl_string("fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+1, gl_string("1fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+10, gl_string("1234567890fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+19, gl_string("1234567890123456789fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 5, str, str+20, gl_string("12345678901234567890fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+0, gl_string("j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+0, gl_string("j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+1, gl_string("1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+2, gl_string("12j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+4, gl_string("1234j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+5, gl_string("12345j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+0, gl_string("j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+1, gl_string("1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+5, gl_string("12345j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+9, gl_string("123456789j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+10, gl_string("1234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+0, gl_string("j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+1, gl_string("1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+10, gl_string("1234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+19, gl_string("1234567890123456789j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 9, str, str+20, gl_string("12345678901234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+2, gl_string("12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+4, gl_string("1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+5, gl_string("12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+5, gl_string("12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+9, gl_string("123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+10, gl_string("1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+10, gl_string("1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+19, gl_string("1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 0, 10, str, str+20, gl_string("12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+1, gl_string("a1bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+2, gl_string("a12bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+4, gl_string("a1234bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+5, gl_string("a12345bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+1, gl_string("a1bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+5, gl_string("a12345bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+9, gl_string("a123456789bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+10, gl_string("a1234567890bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+1, gl_string("a1bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+10, gl_string("a1234567890bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+19, gl_string("a1234567890123456789bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 0, str, str+20, gl_string("a12345678901234567890bcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+0, gl_string("acdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+0, gl_string("acdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+1, gl_string("a1cdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+2, gl_string("a12cdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+4, gl_string("a1234cdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+5, gl_string("a12345cdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+0, gl_string("acdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+1, gl_string("a1cdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+5, gl_string("a12345cdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+9, gl_string("a123456789cdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+10, gl_string("a1234567890cdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+0, gl_string("acdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+1, gl_string("a1cdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+10, gl_string("a1234567890cdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+19, gl_string("a1234567890123456789cdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 1, str, str+20, gl_string("a12345678901234567890cdefghij"));
  }


  void test_string_replace_iter_iter_iter_iter4()
  {
    static const char* str = "12345678901234567890";
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+0, gl_string("afghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+0, gl_string("afghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+1, gl_string("a1fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+2, gl_string("a12fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+4, gl_string("a1234fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+5, gl_string("a12345fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+0, gl_string("afghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+1, gl_string("a1fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+5, gl_string("a12345fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+9, gl_string("a123456789fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+10, gl_string("a1234567890fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+0, gl_string("afghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+1, gl_string("a1fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+10, gl_string("a1234567890fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+19, gl_string("a1234567890123456789fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 4, str, str+20, gl_string("a12345678901234567890fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+0, gl_string("aj"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+0, gl_string("aj"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+1, gl_string("a1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+2, gl_string("a12j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+4, gl_string("a1234j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+5, gl_string("a12345j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+0, gl_string("aj"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+1, gl_string("a1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+5, gl_string("a12345j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+9, gl_string("a123456789j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+10, gl_string("a1234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+0, gl_string("aj"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+1, gl_string("a1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+10, gl_string("a1234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+19, gl_string("a1234567890123456789j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 8, str, str+20, gl_string("a12345678901234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+1, gl_string("a1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+2, gl_string("a12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+4, gl_string("a1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+5, gl_string("a12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+1, gl_string("a1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+5, gl_string("a12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+9, gl_string("a123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+10, gl_string("a1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+1, gl_string("a1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+10, gl_string("a1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+19, gl_string("a1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 1, 9, str, str+20, gl_string("a12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+1, gl_string("abcde1fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+2, gl_string("abcde12fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+4, gl_string("abcde1234fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+5, gl_string("abcde12345fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+1, gl_string("abcde1fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+5, gl_string("abcde12345fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+9, gl_string("abcde123456789fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+10, gl_string("abcde1234567890fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+1, gl_string("abcde1fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+10, gl_string("abcde1234567890fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+19, gl_string("abcde1234567890123456789fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 0, str, str+20, gl_string("abcde12345678901234567890fghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+0, gl_string("abcdeghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+0, gl_string("abcdeghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+1, gl_string("abcde1ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+2, gl_string("abcde12ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+4, gl_string("abcde1234ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+5, gl_string("abcde12345ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+0, gl_string("abcdeghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+1, gl_string("abcde1ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+5, gl_string("abcde12345ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+9, gl_string("abcde123456789ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+10, gl_string("abcde1234567890ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+0, gl_string("abcdeghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+1, gl_string("abcde1ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+10, gl_string("abcde1234567890ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+19, gl_string("abcde1234567890123456789ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 1, str, str+20, gl_string("abcde12345678901234567890ghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+0, gl_string("abcdehij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+0, gl_string("abcdehij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+1, gl_string("abcde1hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+2, gl_string("abcde12hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+4, gl_string("abcde1234hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+5, gl_string("abcde12345hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+0, gl_string("abcdehij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+1, gl_string("abcde1hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+5, gl_string("abcde12345hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+9, gl_string("abcde123456789hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+10, gl_string("abcde1234567890hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+0, gl_string("abcdehij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+1, gl_string("abcde1hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+10, gl_string("abcde1234567890hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+19, gl_string("abcde1234567890123456789hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 2, str, str+20, gl_string("abcde12345678901234567890hij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+0, gl_string("abcdej"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+0, gl_string("abcdej"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+1, gl_string("abcde1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+2, gl_string("abcde12j"));
  }


  void test_string_replace_iter_iter_iter_iter5()
  {
    static const char* str = "12345678901234567890";
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+4, gl_string("abcde1234j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+5, gl_string("abcde12345j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+0, gl_string("abcdej"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+1, gl_string("abcde1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+5, gl_string("abcde12345j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+9, gl_string("abcde123456789j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+10, gl_string("abcde1234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+0, gl_string("abcdej"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+1, gl_string("abcde1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+10, gl_string("abcde1234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+19, gl_string("abcde1234567890123456789j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 4, str, str+20, gl_string("abcde12345678901234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+1, gl_string("abcde1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+2, gl_string("abcde12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+4, gl_string("abcde1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+5, gl_string("abcde12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+1, gl_string("abcde1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+5, gl_string("abcde12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+9, gl_string("abcde123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+10, gl_string("abcde1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+0, gl_string("abcde"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+1, gl_string("abcde1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+10, gl_string("abcde1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+19, gl_string("abcde1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 5, 5, str, str+20, gl_string("abcde12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+1, gl_string("abcdefghi1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+2, gl_string("abcdefghi12j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+4, gl_string("abcdefghi1234j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+5, gl_string("abcdefghi12345j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+1, gl_string("abcdefghi1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+5, gl_string("abcdefghi12345j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+9, gl_string("abcdefghi123456789j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+10, gl_string("abcdefghi1234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+1, gl_string("abcdefghi1j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+10, gl_string("abcdefghi1234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+19, gl_string("abcdefghi1234567890123456789j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 0, str, str+20, gl_string("abcdefghi12345678901234567890j"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+0, gl_string("abcdefghi"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+0, gl_string("abcdefghi"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+1, gl_string("abcdefghi1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+2, gl_string("abcdefghi12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+4, gl_string("abcdefghi1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+5, gl_string("abcdefghi12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+0, gl_string("abcdefghi"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+1, gl_string("abcdefghi1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+5, gl_string("abcdefghi12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+9, gl_string("abcdefghi123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+10, gl_string("abcdefghi1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+0, gl_string("abcdefghi"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+1, gl_string("abcdefghi1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+10, gl_string("abcdefghi1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+19, gl_string("abcdefghi1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 9, 1, str, str+20, gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+1, gl_string("abcdefghij1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+2, gl_string("abcdefghij12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+4, gl_string("abcdefghij1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+5, gl_string("abcdefghij12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+1, gl_string("abcdefghij1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+5, gl_string("abcdefghij12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+9, gl_string("abcdefghij123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+10, gl_string("abcdefghij1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+1, gl_string("abcdefghij1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+10, gl_string("abcdefghij1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+19, gl_string("abcdefghij1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghij"), 10, 0, str, str+20, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+2, gl_string("12abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+4, gl_string("1234abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+9, gl_string("123456789abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+19, gl_string("1234567890123456789abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 0, str, str+20, gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+1, gl_string("1bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+2, gl_string("12bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+4, gl_string("1234bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+5, gl_string("12345bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+1, gl_string("1bcdefghijklmnopqrst"));
  }


  void test_string_replace_iter_iter_iter_iter6()
  {
    static const char* str = "12345678901234567890";
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+5, gl_string("12345bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+9, gl_string("123456789bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+10, gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+1, gl_string("1bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+10, gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+19, gl_string("1234567890123456789bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 1, str, str+20, gl_string("12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+0, gl_string("klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+0, gl_string("klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+1, gl_string("1klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+2, gl_string("12klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+4, gl_string("1234klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+5, gl_string("12345klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+0, gl_string("klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+1, gl_string("1klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+5, gl_string("12345klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+9, gl_string("123456789klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+10, gl_string("1234567890klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+0, gl_string("klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+1, gl_string("1klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+10, gl_string("1234567890klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+19, gl_string("1234567890123456789klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 10, str, str+20, gl_string("12345678901234567890klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+0, gl_string("t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+0, gl_string("t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+1, gl_string("1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+2, gl_string("12t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+4, gl_string("1234t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+5, gl_string("12345t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+0, gl_string("t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+1, gl_string("1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+5, gl_string("12345t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+9, gl_string("123456789t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+10, gl_string("1234567890t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+0, gl_string("t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+1, gl_string("1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+10, gl_string("1234567890t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+19, gl_string("1234567890123456789t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 19, str, str+20, gl_string("12345678901234567890t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+2, gl_string("12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+4, gl_string("1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+5, gl_string("12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+5, gl_string("12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+9, gl_string("123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+10, gl_string("1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+0, gl_string(""));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+1, gl_string("1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+10, gl_string("1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+19, gl_string("1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 0, 20, str, str+20, gl_string("12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+2, gl_string("a12bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+4, gl_string("a1234bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+9, gl_string("a123456789bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+19, gl_string("a1234567890123456789bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 0, str, str+20, gl_string("a12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+1, gl_string("a1cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+2, gl_string("a12cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+4, gl_string("a1234cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+5, gl_string("a12345cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+1, gl_string("a1cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+5, gl_string("a12345cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+9, gl_string("a123456789cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+10, gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+1, gl_string("a1cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+10, gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+19, gl_string("a1234567890123456789cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 1, str, str+20, gl_string("a12345678901234567890cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+0, gl_string("aklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+0, gl_string("aklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+1, gl_string("a1klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+2, gl_string("a12klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+4, gl_string("a1234klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+5, gl_string("a12345klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+0, gl_string("aklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+1, gl_string("a1klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+5, gl_string("a12345klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+9, gl_string("a123456789klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+10, gl_string("a1234567890klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+0, gl_string("aklmnopqrst"));
  }


  void test_string_replace_iter_iter_iter_iter7()
  {
    static const char* str = "12345678901234567890";
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+1, gl_string("a1klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+10, gl_string("a1234567890klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+19, gl_string("a1234567890123456789klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 9, str, str+20, gl_string("a12345678901234567890klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+0, gl_string("at"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+0, gl_string("at"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+1, gl_string("a1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+2, gl_string("a12t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+4, gl_string("a1234t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+5, gl_string("a12345t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+0, gl_string("at"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+1, gl_string("a1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+5, gl_string("a12345t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+9, gl_string("a123456789t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+10, gl_string("a1234567890t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+0, gl_string("at"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+1, gl_string("a1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+10, gl_string("a1234567890t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+19, gl_string("a1234567890123456789t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 18, str, str+20, gl_string("a12345678901234567890t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+1, gl_string("a1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+2, gl_string("a12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+4, gl_string("a1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+5, gl_string("a12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+1, gl_string("a1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+5, gl_string("a12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+9, gl_string("a123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+10, gl_string("a1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+0, gl_string("a"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+1, gl_string("a1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+10, gl_string("a1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+19, gl_string("a1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 1, 19, str, str+20, gl_string("a12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+2, gl_string("abcdefghij12klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+4, gl_string("abcdefghij1234klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+9, gl_string("abcdefghij123456789klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+19, gl_string("abcdefghij1234567890123456789klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 0, str, str+20, gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+1, gl_string("abcdefghij1lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+2, gl_string("abcdefghij12lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+4, gl_string("abcdefghij1234lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+5, gl_string("abcdefghij12345lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+1, gl_string("abcdefghij1lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+5, gl_string("abcdefghij12345lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+9, gl_string("abcdefghij123456789lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+10, gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+1, gl_string("abcdefghij1lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+10, gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+19, gl_string("abcdefghij1234567890123456789lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 1, str, str+20, gl_string("abcdefghij12345678901234567890lmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+0, gl_string("abcdefghijpqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+0, gl_string("abcdefghijpqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+1, gl_string("abcdefghij1pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+2, gl_string("abcdefghij12pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+4, gl_string("abcdefghij1234pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+5, gl_string("abcdefghij12345pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+0, gl_string("abcdefghijpqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+1, gl_string("abcdefghij1pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+5, gl_string("abcdefghij12345pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+9, gl_string("abcdefghij123456789pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+10, gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+0, gl_string("abcdefghijpqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+1, gl_string("abcdefghij1pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+10, gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+19, gl_string("abcdefghij1234567890123456789pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 5, str, str+20, gl_string("abcdefghij12345678901234567890pqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+0, gl_string("abcdefghijt"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+0, gl_string("abcdefghijt"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+1, gl_string("abcdefghij1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+2, gl_string("abcdefghij12t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+4, gl_string("abcdefghij1234t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+5, gl_string("abcdefghij12345t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+0, gl_string("abcdefghijt"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+1, gl_string("abcdefghij1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+5, gl_string("abcdefghij12345t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+9, gl_string("abcdefghij123456789t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+10, gl_string("abcdefghij1234567890t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+0, gl_string("abcdefghijt"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+1, gl_string("abcdefghij1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+10, gl_string("abcdefghij1234567890t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+19, gl_string("abcdefghij1234567890123456789t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 9, str, str+20, gl_string("abcdefghij12345678901234567890t"));
  }


  void test_string_replace_iter_iter_iter_iter8()
  {
    static const char* str = "12345678901234567890";
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+1, gl_string("abcdefghij1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+2, gl_string("abcdefghij12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+4, gl_string("abcdefghij1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+5, gl_string("abcdefghij12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+1, gl_string("abcdefghij1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+5, gl_string("abcdefghij12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+9, gl_string("abcdefghij123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+10, gl_string("abcdefghij1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+0, gl_string("abcdefghij"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+1, gl_string("abcdefghij1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+10, gl_string("abcdefghij1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+19, gl_string("abcdefghij1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 10, 10, str, str+20, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+2, gl_string("abcdefghijklmnopqrs12t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+4, gl_string("abcdefghijklmnopqrs1234t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+9, gl_string("abcdefghijklmnopqrs123456789t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+19, gl_string("abcdefghijklmnopqrs1234567890123456789t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 0, str, str+20, gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+2, gl_string("abcdefghijklmnopqrs12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+4, gl_string("abcdefghijklmnopqrs1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+9, gl_string("abcdefghijklmnopqrs123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+19, gl_string("abcdefghijklmnopqrs1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 19, 1, str, str+20, gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+2, gl_string("abcdefghijklmnopqrst12"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+4, gl_string("abcdefghijklmnopqrst1234"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+9, gl_string("abcdefghijklmnopqrst123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+19, gl_string("abcdefghijklmnopqrst1234567890123456789"));
    _test_string_replace_iter_iter_iter_iter(gl_string("abcdefghijklmnopqrst"), 20, 0, str, str+20, gl_string("abcdefghijklmnopqrst12345678901234567890"));
  }

  void
  _test_string_replace_iter_iter_size_char(gl_string s, size_t pos1, size_t n1, size_t n2,
                                           char c, gl_string expected)
  {
    size_t old_size = s.size();
    gl_string::const_iterator first = s.begin() + pos1;
    gl_string::const_iterator last = s.begin() + pos1 + n1;
    size_t xlen = last - first;
    s.replace(first, last, n2, c);
    
    ASSERT_TRUE(s == expected);
    size_t rlen = n2;
    TS_ASSERT(s.size() == old_size - xlen + rlen);
  }


  void test_string_replace_iter_iter_size_char0()
  {
    _test_string_replace_iter_iter_size_char(gl_string(""), 0, 0, 0, '3', gl_string(""));
    _test_string_replace_iter_iter_size_char(gl_string(""), 0, 0, 5, '3', gl_string("33333"));
    _test_string_replace_iter_iter_size_char(gl_string(""), 0, 0, 10, '3', gl_string("3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string(""), 0, 0, 20, '3', gl_string("33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 0, 0, '3', gl_string("abcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 0, 5, '3', gl_string("33333abcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 0, 10, '3', gl_string("3333333333abcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 0, 20, '3', gl_string("33333333333333333333abcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 1, 0, '3', gl_string("bcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 1, 5, '3', gl_string("33333bcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 1, 10, '3', gl_string("3333333333bcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 1, 20, '3', gl_string("33333333333333333333bcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 2, 0, '3', gl_string("cde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 2, 5, '3', gl_string("33333cde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 2, 10, '3', gl_string("3333333333cde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 2, 20, '3', gl_string("33333333333333333333cde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 4, 0, '3', gl_string("e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 4, 5, '3', gl_string("33333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 4, 10, '3', gl_string("3333333333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 4, 20, '3', gl_string("33333333333333333333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 5, 0, '3', gl_string(""));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 5, 5, '3', gl_string("33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 5, 10, '3', gl_string("3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 0, 5, 20, '3', gl_string("33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 0, 0, '3', gl_string("abcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 0, 5, '3', gl_string("a33333bcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 0, 10, '3', gl_string("a3333333333bcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 0, 20, '3', gl_string("a33333333333333333333bcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 1, 0, '3', gl_string("acde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 1, 5, '3', gl_string("a33333cde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 1, 10, '3', gl_string("a3333333333cde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 1, 20, '3', gl_string("a33333333333333333333cde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 2, 0, '3', gl_string("ade"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 2, 5, '3', gl_string("a33333de"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 2, 10, '3', gl_string("a3333333333de"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 2, 20, '3', gl_string("a33333333333333333333de"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 3, 0, '3', gl_string("ae"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 3, 5, '3', gl_string("a33333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 3, 10, '3', gl_string("a3333333333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 3, 20, '3', gl_string("a33333333333333333333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 4, 0, '3', gl_string("a"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 4, 5, '3', gl_string("a33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 4, 10, '3', gl_string("a3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 1, 4, 20, '3', gl_string("a33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 0, 0, '3', gl_string("abcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 0, 5, '3', gl_string("ab33333cde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 0, 10, '3', gl_string("ab3333333333cde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 0, 20, '3', gl_string("ab33333333333333333333cde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 1, 0, '3', gl_string("abde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 1, 5, '3', gl_string("ab33333de"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 1, 10, '3', gl_string("ab3333333333de"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 1, 20, '3', gl_string("ab33333333333333333333de"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 2, 0, '3', gl_string("abe"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 2, 5, '3', gl_string("ab33333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 2, 10, '3', gl_string("ab3333333333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 2, 20, '3', gl_string("ab33333333333333333333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 3, 0, '3', gl_string("ab"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 3, 5, '3', gl_string("ab33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 3, 10, '3', gl_string("ab3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 2, 3, 20, '3', gl_string("ab33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 4, 0, 0, '3', gl_string("abcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 4, 0, 5, '3', gl_string("abcd33333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 4, 0, 10, '3', gl_string("abcd3333333333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 4, 0, 20, '3', gl_string("abcd33333333333333333333e"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 4, 1, 0, '3', gl_string("abcd"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 4, 1, 5, '3', gl_string("abcd33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 4, 1, 10, '3', gl_string("abcd3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 4, 1, 20, '3', gl_string("abcd33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 5, 0, 0, '3', gl_string("abcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 5, 0, 5, '3', gl_string("abcde33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 5, 0, 10, '3', gl_string("abcde3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcde"), 5, 0, 20, '3', gl_string("abcde33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 0, 0, '3', gl_string("abcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 0, 5, '3', gl_string("33333abcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 0, 10, '3', gl_string("3333333333abcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 0, 20, '3', gl_string("33333333333333333333abcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 1, 0, '3', gl_string("bcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 1, 5, '3', gl_string("33333bcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 1, 10, '3', gl_string("3333333333bcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 1, 20, '3', gl_string("33333333333333333333bcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 5, 0, '3', gl_string("fghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 5, 5, '3', gl_string("33333fghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 5, 10, '3', gl_string("3333333333fghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 5, 20, '3', gl_string("33333333333333333333fghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 9, 0, '3', gl_string("j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 9, 5, '3', gl_string("33333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 9, 10, '3', gl_string("3333333333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 9, 20, '3', gl_string("33333333333333333333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 10, 0, '3', gl_string(""));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 10, 5, '3', gl_string("33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 10, 10, '3', gl_string("3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 0, 10, 20, '3', gl_string("33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 0, 0, '3', gl_string("abcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 0, 5, '3', gl_string("a33333bcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 0, 10, '3', gl_string("a3333333333bcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 0, 20, '3', gl_string("a33333333333333333333bcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 1, 0, '3', gl_string("acdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 1, 5, '3', gl_string("a33333cdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 1, 10, '3', gl_string("a3333333333cdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 1, 20, '3', gl_string("a33333333333333333333cdefghij"));
  }


  void test_string_replace_iter_iter_size_char1()
  {
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 4, 0, '3', gl_string("afghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 4, 5, '3', gl_string("a33333fghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 4, 10, '3', gl_string("a3333333333fghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 4, 20, '3', gl_string("a33333333333333333333fghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 8, 0, '3', gl_string("aj"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 8, 5, '3', gl_string("a33333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 8, 10, '3', gl_string("a3333333333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 8, 20, '3', gl_string("a33333333333333333333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 9, 0, '3', gl_string("a"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 9, 5, '3', gl_string("a33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 9, 10, '3', gl_string("a3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 1, 9, 20, '3', gl_string("a33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 0, 0, '3', gl_string("abcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 0, 5, '3', gl_string("abcde33333fghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 0, 10, '3', gl_string("abcde3333333333fghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 0, 20, '3', gl_string("abcde33333333333333333333fghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 1, 0, '3', gl_string("abcdeghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 1, 5, '3', gl_string("abcde33333ghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 1, 10, '3', gl_string("abcde3333333333ghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 1, 20, '3', gl_string("abcde33333333333333333333ghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 2, 0, '3', gl_string("abcdehij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 2, 5, '3', gl_string("abcde33333hij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 2, 10, '3', gl_string("abcde3333333333hij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 2, 20, '3', gl_string("abcde33333333333333333333hij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 4, 0, '3', gl_string("abcdej"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 4, 5, '3', gl_string("abcde33333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 4, 10, '3', gl_string("abcde3333333333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 4, 20, '3', gl_string("abcde33333333333333333333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 5, 0, '3', gl_string("abcde"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 5, 5, '3', gl_string("abcde33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 5, 10, '3', gl_string("abcde3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 5, 5, 20, '3', gl_string("abcde33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 9, 0, 0, '3', gl_string("abcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 9, 0, 5, '3', gl_string("abcdefghi33333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 9, 0, 10, '3', gl_string("abcdefghi3333333333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 9, 0, 20, '3', gl_string("abcdefghi33333333333333333333j"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 9, 1, 0, '3', gl_string("abcdefghi"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 9, 1, 5, '3', gl_string("abcdefghi33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 9, 1, 10, '3', gl_string("abcdefghi3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 9, 1, 20, '3', gl_string("abcdefghi33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 10, 0, 0, '3', gl_string("abcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 10, 0, 5, '3', gl_string("abcdefghij33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 10, 0, 10, '3', gl_string("abcdefghij3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghij"), 10, 0, 20, '3', gl_string("abcdefghij33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 0, 0, '3', gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 0, 5, '3', gl_string("33333abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 0, 10, '3', gl_string("3333333333abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 0, 20, '3', gl_string("33333333333333333333abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 1, 0, '3', gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 1, 5, '3', gl_string("33333bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 1, 10, '3', gl_string("3333333333bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 1, 20, '3', gl_string("33333333333333333333bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 10, 0, '3', gl_string("klmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 10, 5, '3', gl_string("33333klmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 10, 10, '3', gl_string("3333333333klmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 10, 20, '3', gl_string("33333333333333333333klmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 19, 0, '3', gl_string("t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 19, 5, '3', gl_string("33333t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 19, 10, '3', gl_string("3333333333t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 19, 20, '3', gl_string("33333333333333333333t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 20, 0, '3', gl_string(""));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 20, 5, '3', gl_string("33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 20, 10, '3', gl_string("3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 0, 20, 20, '3', gl_string("33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 0, 0, '3', gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 0, 5, '3', gl_string("a33333bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 0, 10, '3', gl_string("a3333333333bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 0, 20, '3', gl_string("a33333333333333333333bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 1, 0, '3', gl_string("acdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 1, 5, '3', gl_string("a33333cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 1, 10, '3', gl_string("a3333333333cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 1, 20, '3', gl_string("a33333333333333333333cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 9, 0, '3', gl_string("aklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 9, 5, '3', gl_string("a33333klmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 9, 10, '3', gl_string("a3333333333klmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 9, 20, '3', gl_string("a33333333333333333333klmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 18, 0, '3', gl_string("at"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 18, 5, '3', gl_string("a33333t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 18, 10, '3', gl_string("a3333333333t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 18, 20, '3', gl_string("a33333333333333333333t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 19, 0, '3', gl_string("a"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 19, 5, '3', gl_string("a33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 19, 10, '3', gl_string("a3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 1, 19, 20, '3', gl_string("a33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 0, 0, '3', gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 0, 5, '3', gl_string("abcdefghij33333klmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 0, 10, '3', gl_string("abcdefghij3333333333klmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 0, 20, '3', gl_string("abcdefghij33333333333333333333klmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 1, 0, '3', gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 1, 5, '3', gl_string("abcdefghij33333lmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 1, 10, '3', gl_string("abcdefghij3333333333lmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 1, 20, '3', gl_string("abcdefghij33333333333333333333lmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 5, 0, '3', gl_string("abcdefghijpqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 5, 5, '3', gl_string("abcdefghij33333pqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 5, 10, '3', gl_string("abcdefghij3333333333pqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 5, 20, '3', gl_string("abcdefghij33333333333333333333pqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 9, 0, '3', gl_string("abcdefghijt"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 9, 5, '3', gl_string("abcdefghij33333t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 9, 10, '3', gl_string("abcdefghij3333333333t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 9, 20, '3', gl_string("abcdefghij33333333333333333333t"));
  }


  void test_string_replace_iter_iter_size_char2()
  {
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 10, 0, '3', gl_string("abcdefghij"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 10, 5, '3', gl_string("abcdefghij33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 10, 10, '3', gl_string("abcdefghij3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 10, 10, 20, '3', gl_string("abcdefghij33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 0, 0, '3', gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 0, 5, '3', gl_string("abcdefghijklmnopqrs33333t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 0, 10, '3', gl_string("abcdefghijklmnopqrs3333333333t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 0, 20, '3', gl_string("abcdefghijklmnopqrs33333333333333333333t"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 1, 0, '3', gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 1, 5, '3', gl_string("abcdefghijklmnopqrs33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 1, 10, '3', gl_string("abcdefghijklmnopqrs3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 19, 1, 20, '3', gl_string("abcdefghijklmnopqrs33333333333333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 20, 0, 0, '3', gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 20, 0, 5, '3', gl_string("abcdefghijklmnopqrst33333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 20, 0, 10, '3', gl_string("abcdefghijklmnopqrst3333333333"));
    _test_string_replace_iter_iter_size_char(gl_string("abcdefghijklmnopqrst"), 20, 0, 20, '3', gl_string("abcdefghijklmnopqrst33333333333333333333"));
  }


  void
  _test_string_replace_iter_iter_string(gl_string s, size_t pos1, size_t n1, gl_string str, gl_string expected)
  {
    size_t old_size = s.size();
    gl_string::const_iterator first = s.begin() + pos1;
    gl_string::const_iterator last = s.begin() + pos1 + n1;
    size_t xlen = last - first;
    s.replace(first, last, str);
    
    TS_ASSERT(s == expected);
    size_t rlen = str.size();
    TS_ASSERT(s.size() == old_size - xlen + rlen);
  }


  void test_string_replace_iter_iter_string0()
  {
    _test_string_replace_iter_iter_string(gl_string(""), 0, 0, gl_string(""), gl_string(""));
    _test_string_replace_iter_iter_string(gl_string(""), 0, 0, gl_string("12345"), gl_string("12345"));
    _test_string_replace_iter_iter_string(gl_string(""), 0, 0, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_iter_iter_string(gl_string(""), 0, 0, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 0, gl_string(""), gl_string("abcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 0, gl_string("12345"), gl_string("12345abcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 0, gl_string("1234567890"), gl_string("1234567890abcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), gl_string("12345678901234567890abcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 1, gl_string(""), gl_string("bcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 1, gl_string("12345"), gl_string("12345bcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 1, gl_string("1234567890"), gl_string("1234567890bcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), gl_string("12345678901234567890bcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 2, gl_string(""), gl_string("cde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 2, gl_string("12345"), gl_string("12345cde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 2, gl_string("1234567890"), gl_string("1234567890cde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), gl_string("12345678901234567890cde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 4, gl_string(""), gl_string("e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 4, gl_string("12345"), gl_string("12345e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 4, gl_string("1234567890"), gl_string("1234567890e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), gl_string("12345678901234567890e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 5, gl_string(""), gl_string(""));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 5, gl_string("12345"), gl_string("12345"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 5, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 0, gl_string(""), gl_string("abcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 0, gl_string("12345"), gl_string("a12345bcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 0, gl_string("1234567890"), gl_string("a1234567890bcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), gl_string("a12345678901234567890bcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 1, gl_string(""), gl_string("acde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 1, gl_string("12345"), gl_string("a12345cde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 1, gl_string("1234567890"), gl_string("a1234567890cde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), gl_string("a12345678901234567890cde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 2, gl_string(""), gl_string("ade"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 2, gl_string("12345"), gl_string("a12345de"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 2, gl_string("1234567890"), gl_string("a1234567890de"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), gl_string("a12345678901234567890de"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 3, gl_string(""), gl_string("ae"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 3, gl_string("12345"), gl_string("a12345e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 3, gl_string("1234567890"), gl_string("a1234567890e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), gl_string("a12345678901234567890e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 4, gl_string(""), gl_string("a"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 4, gl_string("12345"), gl_string("a12345"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 4, gl_string("1234567890"), gl_string("a1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), gl_string("a12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 0, gl_string(""), gl_string("abcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 0, gl_string("12345"), gl_string("ab12345cde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 0, gl_string("1234567890"), gl_string("ab1234567890cde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), gl_string("ab12345678901234567890cde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 1, gl_string(""), gl_string("abde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 1, gl_string("12345"), gl_string("ab12345de"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 1, gl_string("1234567890"), gl_string("ab1234567890de"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), gl_string("ab12345678901234567890de"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 2, gl_string(""), gl_string("abe"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 2, gl_string("12345"), gl_string("ab12345e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 2, gl_string("1234567890"), gl_string("ab1234567890e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), gl_string("ab12345678901234567890e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 3, gl_string(""), gl_string("ab"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 3, gl_string("12345"), gl_string("ab12345"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 3, gl_string("1234567890"), gl_string("ab1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), gl_string("ab12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 4, 0, gl_string(""), gl_string("abcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 4, 0, gl_string("12345"), gl_string("abcd12345e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 4, 0, gl_string("1234567890"), gl_string("abcd1234567890e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), gl_string("abcd12345678901234567890e"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 4, 1, gl_string(""), gl_string("abcd"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 4, 1, gl_string("12345"), gl_string("abcd12345"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 4, 1, gl_string("1234567890"), gl_string("abcd1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), gl_string("abcd12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 5, 0, gl_string(""), gl_string("abcde"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 5, 0, gl_string("12345"), gl_string("abcde12345"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 5, 0, gl_string("1234567890"), gl_string("abcde1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 0, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 0, gl_string("12345"), gl_string("12345abcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), gl_string("1234567890abcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), gl_string("12345678901234567890abcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 1, gl_string(""), gl_string("bcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 1, gl_string("12345"), gl_string("12345bcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), gl_string("1234567890bcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), gl_string("12345678901234567890bcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 5, gl_string(""), gl_string("fghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 5, gl_string("12345"), gl_string("12345fghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), gl_string("1234567890fghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), gl_string("12345678901234567890fghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 9, gl_string(""), gl_string("j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 9, gl_string("12345"), gl_string("12345j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), gl_string("1234567890j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), gl_string("12345678901234567890j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 10, gl_string(""), gl_string(""));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 10, gl_string("12345"), gl_string("12345"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 0, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 0, gl_string("12345"), gl_string("a12345bcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), gl_string("a1234567890bcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), gl_string("a12345678901234567890bcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 1, gl_string(""), gl_string("acdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 1, gl_string("12345"), gl_string("a12345cdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), gl_string("a1234567890cdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), gl_string("a12345678901234567890cdefghij"));
  }


  void test_string_replace_iter_iter_string1()
  {
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 4, gl_string(""), gl_string("afghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 4, gl_string("12345"), gl_string("a12345fghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), gl_string("a1234567890fghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), gl_string("a12345678901234567890fghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 8, gl_string(""), gl_string("aj"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 8, gl_string("12345"), gl_string("a12345j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), gl_string("a1234567890j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), gl_string("a12345678901234567890j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 9, gl_string(""), gl_string("a"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 9, gl_string("12345"), gl_string("a12345"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), gl_string("a1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), gl_string("a12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 0, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 0, gl_string("12345"), gl_string("abcde12345fghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), gl_string("abcde1234567890fghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890fghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 1, gl_string(""), gl_string("abcdeghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 1, gl_string("12345"), gl_string("abcde12345ghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), gl_string("abcde1234567890ghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890ghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 2, gl_string(""), gl_string("abcdehij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 2, gl_string("12345"), gl_string("abcde12345hij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), gl_string("abcde1234567890hij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890hij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 4, gl_string(""), gl_string("abcdej"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 4, gl_string("12345"), gl_string("abcde12345j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), gl_string("abcde1234567890j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 5, gl_string(""), gl_string("abcde"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 5, gl_string("12345"), gl_string("abcde12345"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), gl_string("abcde1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 9, 0, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 9, 0, gl_string("12345"), gl_string("abcdefghi12345j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), gl_string("abcdefghi1234567890j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), gl_string("abcdefghi12345678901234567890j"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 9, 1, gl_string(""), gl_string("abcdefghi"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 9, 1, gl_string("12345"), gl_string("abcdefghi12345"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), gl_string("abcdefghi1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 10, 0, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 10, 0, gl_string("12345"), gl_string("abcdefghij12345"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), gl_string("abcdefghij1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), gl_string("12345abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string(""), gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), gl_string("12345bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), gl_string("12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string(""), gl_string("klmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), gl_string("12345klmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), gl_string("1234567890klmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), gl_string("12345678901234567890klmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string(""), gl_string("t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), gl_string("12345t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), gl_string("1234567890t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), gl_string("12345678901234567890t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string(""), gl_string(""));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), gl_string("12345"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), gl_string("a12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string(""), gl_string("acdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), gl_string("a12345cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), gl_string("a12345678901234567890cdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string(""), gl_string("aklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), gl_string("a12345klmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), gl_string("a1234567890klmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), gl_string("a12345678901234567890klmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string(""), gl_string("at"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), gl_string("a12345t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), gl_string("a1234567890t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), gl_string("a12345678901234567890t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string(""), gl_string("a"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), gl_string("a12345"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), gl_string("a1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), gl_string("a12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), gl_string("abcdefghij12345klmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string(""), gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), gl_string("abcdefghij12345lmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890lmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string(""), gl_string("abcdefghijpqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), gl_string("abcdefghij12345pqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890pqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string(""), gl_string("abcdefghijt"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), gl_string("abcdefghij12345t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), gl_string("abcdefghij1234567890t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890t"));
  }


  void test_string_replace_iter_iter_string2()
  {
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), gl_string("abcdefghij12345"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), gl_string("abcdefghij1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string(""), gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_iter_iter_string(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), gl_string("abcdefghijklmnopqrst12345678901234567890"));
  }


  void
  _test_string_replace_size_size_pointer(gl_string s, size_t pos, size_t n1,
                                         const char* str, gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.replace(pos, n1, str);
        
      TS_ASSERT(pos <= old_size);
      TS_ASSERT(s == expected);
      size_t xlen = std::min(n1, old_size - pos);
      size_t rlen = std::strlen(str);
      TS_ASSERT(s.size() == old_size - xlen + rlen);
    }
    catch (...)
    {
      TS_ASSERT(pos > old_size);
      TS_ASSERT(s == s0);
    }
#endif
  }


  void test_string_replace_size_size_pointer0()
  {
    _test_string_replace_size_size_pointer(gl_string(""), 0, 0, "", gl_string(""));
    _test_string_replace_size_size_pointer(gl_string(""), 0, 0, "12345", gl_string("12345"));
    _test_string_replace_size_size_pointer(gl_string(""), 0, 0, "1234567890", gl_string("1234567890"));
    _test_string_replace_size_size_pointer(gl_string(""), 0, 0, "12345678901234567890", gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string(""), 0, 1, "", gl_string(""));
    _test_string_replace_size_size_pointer(gl_string(""), 0, 1, "12345", gl_string("12345"));
    _test_string_replace_size_size_pointer(gl_string(""), 0, 1, "1234567890", gl_string("1234567890"));
    _test_string_replace_size_size_pointer(gl_string(""), 0, 1, "12345678901234567890", gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string(""), 1, 0, "", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string(""), 1, 0, "12345", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string(""), 1, 0, "1234567890", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string(""), 1, 0, "12345678901234567890", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 0, "", gl_string("abcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 0, "12345", gl_string("12345abcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 0, "1234567890", gl_string("1234567890abcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 0, "12345678901234567890", gl_string("12345678901234567890abcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 1, "", gl_string("bcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 1, "12345", gl_string("12345bcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 1, "1234567890", gl_string("1234567890bcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 1, "12345678901234567890", gl_string("12345678901234567890bcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 2, "", gl_string("cde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 2, "12345", gl_string("12345cde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 2, "1234567890", gl_string("1234567890cde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 2, "12345678901234567890", gl_string("12345678901234567890cde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 4, "", gl_string("e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 4, "12345", gl_string("12345e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 4, "1234567890", gl_string("1234567890e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 4, "12345678901234567890", gl_string("12345678901234567890e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 5, "", gl_string(""));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 5, "12345", gl_string("12345"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 5, "1234567890", gl_string("1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 5, "12345678901234567890", gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 6, "", gl_string(""));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 6, "12345", gl_string("12345"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 6, "1234567890", gl_string("1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 0, 6, "12345678901234567890", gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 0, "", gl_string("abcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 0, "12345", gl_string("a12345bcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 0, "1234567890", gl_string("a1234567890bcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 0, "12345678901234567890", gl_string("a12345678901234567890bcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 1, "", gl_string("acde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 1, "12345", gl_string("a12345cde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 1, "1234567890", gl_string("a1234567890cde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 1, "12345678901234567890", gl_string("a12345678901234567890cde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 2, "", gl_string("ade"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 2, "12345", gl_string("a12345de"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 2, "1234567890", gl_string("a1234567890de"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 2, "12345678901234567890", gl_string("a12345678901234567890de"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 3, "", gl_string("ae"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 3, "12345", gl_string("a12345e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 3, "1234567890", gl_string("a1234567890e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 3, "12345678901234567890", gl_string("a12345678901234567890e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 4, "", gl_string("a"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 4, "12345", gl_string("a12345"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 4, "1234567890", gl_string("a1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 4, "12345678901234567890", gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 5, "", gl_string("a"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 5, "12345", gl_string("a12345"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 5, "1234567890", gl_string("a1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 1, 5, "12345678901234567890", gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 0, "", gl_string("abcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 0, "12345", gl_string("ab12345cde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 0, "1234567890", gl_string("ab1234567890cde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 0, "12345678901234567890", gl_string("ab12345678901234567890cde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 1, "", gl_string("abde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 1, "12345", gl_string("ab12345de"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 1, "1234567890", gl_string("ab1234567890de"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 1, "12345678901234567890", gl_string("ab12345678901234567890de"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 2, "", gl_string("abe"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 2, "12345", gl_string("ab12345e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 2, "1234567890", gl_string("ab1234567890e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 2, "12345678901234567890", gl_string("ab12345678901234567890e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 3, "", gl_string("ab"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 3, "12345", gl_string("ab12345"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 3, "1234567890", gl_string("ab1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 3, "12345678901234567890", gl_string("ab12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 4, "", gl_string("ab"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 4, "12345", gl_string("ab12345"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 4, "1234567890", gl_string("ab1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 2, 4, "12345678901234567890", gl_string("ab12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 0, "", gl_string("abcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 0, "12345", gl_string("abcd12345e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 0, "1234567890", gl_string("abcd1234567890e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 0, "12345678901234567890", gl_string("abcd12345678901234567890e"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 1, "", gl_string("abcd"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 1, "12345", gl_string("abcd12345"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 1, "1234567890", gl_string("abcd1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 1, "12345678901234567890", gl_string("abcd12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 2, "", gl_string("abcd"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 2, "12345", gl_string("abcd12345"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 2, "1234567890", gl_string("abcd1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 4, 2, "12345678901234567890", gl_string("abcd12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 5, 0, "", gl_string("abcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 5, 0, "12345", gl_string("abcde12345"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 5, 0, "1234567890", gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 5, 0, "12345678901234567890", gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 5, 1, "", gl_string("abcde"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 5, 1, "12345", gl_string("abcde12345"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 5, 1, "1234567890", gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 5, 1, "12345678901234567890", gl_string("abcde12345678901234567890"));
  }


  void test_string_replace_size_size_pointer1()
  {
    _test_string_replace_size_size_pointer(gl_string("abcde"), 6, 0, "", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 6, 0, "12345", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 6, 0, "1234567890", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string("abcde"), 6, 0, "12345678901234567890", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 0, "", gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 0, "12345", gl_string("12345abcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 0, "1234567890", gl_string("1234567890abcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 0, "12345678901234567890", gl_string("12345678901234567890abcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 1, "", gl_string("bcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 1, "12345", gl_string("12345bcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 1, "1234567890", gl_string("1234567890bcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 1, "12345678901234567890", gl_string("12345678901234567890bcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 5, "", gl_string("fghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 5, "12345", gl_string("12345fghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 5, "1234567890", gl_string("1234567890fghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 5, "12345678901234567890", gl_string("12345678901234567890fghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 9, "", gl_string("j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 9, "12345", gl_string("12345j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 9, "1234567890", gl_string("1234567890j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 9, "12345678901234567890", gl_string("12345678901234567890j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 10, "", gl_string(""));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 10, "12345", gl_string("12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 10, "1234567890", gl_string("1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 10, "12345678901234567890", gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 11, "", gl_string(""));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 11, "12345", gl_string("12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 11, "1234567890", gl_string("1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 0, 11, "12345678901234567890", gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 0, "", gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 0, "12345", gl_string("a12345bcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 0, "1234567890", gl_string("a1234567890bcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 0, "12345678901234567890", gl_string("a12345678901234567890bcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 1, "", gl_string("acdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 1, "12345", gl_string("a12345cdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 1, "1234567890", gl_string("a1234567890cdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 1, "12345678901234567890", gl_string("a12345678901234567890cdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 4, "", gl_string("afghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 4, "12345", gl_string("a12345fghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 4, "1234567890", gl_string("a1234567890fghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 4, "12345678901234567890", gl_string("a12345678901234567890fghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 8, "", gl_string("aj"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 8, "12345", gl_string("a12345j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 8, "1234567890", gl_string("a1234567890j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 8, "12345678901234567890", gl_string("a12345678901234567890j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 9, "", gl_string("a"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 9, "12345", gl_string("a12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 9, "1234567890", gl_string("a1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 9, "12345678901234567890", gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 10, "", gl_string("a"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 10, "12345", gl_string("a12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 10, "1234567890", gl_string("a1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 1, 10, "12345678901234567890", gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 0, "", gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 0, "12345", gl_string("abcde12345fghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 0, "1234567890", gl_string("abcde1234567890fghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 0, "12345678901234567890", gl_string("abcde12345678901234567890fghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 1, "", gl_string("abcdeghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 1, "12345", gl_string("abcde12345ghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 1, "1234567890", gl_string("abcde1234567890ghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 1, "12345678901234567890", gl_string("abcde12345678901234567890ghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 2, "", gl_string("abcdehij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 2, "12345", gl_string("abcde12345hij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 2, "1234567890", gl_string("abcde1234567890hij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 2, "12345678901234567890", gl_string("abcde12345678901234567890hij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 4, "", gl_string("abcdej"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 4, "12345", gl_string("abcde12345j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 4, "1234567890", gl_string("abcde1234567890j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 4, "12345678901234567890", gl_string("abcde12345678901234567890j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 5, "", gl_string("abcde"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 5, "12345", gl_string("abcde12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 5, "1234567890", gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 5, "12345678901234567890", gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 6, "", gl_string("abcde"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 6, "12345", gl_string("abcde12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 6, "1234567890", gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 5, 6, "12345678901234567890", gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 0, "", gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 0, "12345", gl_string("abcdefghi12345j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 0, "1234567890", gl_string("abcdefghi1234567890j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 0, "12345678901234567890", gl_string("abcdefghi12345678901234567890j"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 1, "", gl_string("abcdefghi"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 1, "12345", gl_string("abcdefghi12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 1, "1234567890", gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 1, "12345678901234567890", gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 2, "", gl_string("abcdefghi"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 2, "12345", gl_string("abcdefghi12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 2, "1234567890", gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 9, 2, "12345678901234567890", gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 10, 0, "", gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 10, 0, "12345", gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 10, 0, "1234567890", gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 10, 0, "12345678901234567890", gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 10, 1, "", gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 10, 1, "12345", gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 10, 1, "1234567890", gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 10, 1, "12345678901234567890", gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 11, 0, "", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 11, 0, "12345", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 11, 0, "1234567890", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghij"), 11, 0, "12345678901234567890", gl_string("can't happen"));
  }


  void test_string_replace_size_size_pointer2()
  {
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 0, "", gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345", gl_string("12345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 0, "1234567890", gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345678901234567890", gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 1, "", gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345", gl_string("12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 1, "1234567890", gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345678901234567890", gl_string("12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 10, "", gl_string("klmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345", gl_string("12345klmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 10, "1234567890", gl_string("1234567890klmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345678901234567890", gl_string("12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 19, "", gl_string("t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345", gl_string("12345t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 19, "1234567890", gl_string("1234567890t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345678901234567890", gl_string("12345678901234567890t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 20, "", gl_string(""));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345", gl_string("12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 20, "1234567890", gl_string("1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345678901234567890", gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 21, "", gl_string(""));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345", gl_string("12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 21, "1234567890", gl_string("1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345678901234567890", gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 0, "", gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345", gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 0, "1234567890", gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345678901234567890", gl_string("a12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 1, "", gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345", gl_string("a12345cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 1, "1234567890", gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345678901234567890", gl_string("a12345678901234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 9, "", gl_string("aklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345", gl_string("a12345klmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 9, "1234567890", gl_string("a1234567890klmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345678901234567890", gl_string("a12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 18, "", gl_string("at"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345", gl_string("a12345t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 18, "1234567890", gl_string("a1234567890t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345678901234567890", gl_string("a12345678901234567890t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 19, "", gl_string("a"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345", gl_string("a12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 19, "1234567890", gl_string("a1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345678901234567890", gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 20, "", gl_string("a"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345", gl_string("a12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 20, "1234567890", gl_string("a1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345678901234567890", gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 0, "", gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345", gl_string("abcdefghij12345klmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 0, "1234567890", gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345678901234567890", gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 1, "", gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345", gl_string("abcdefghij12345lmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 1, "1234567890", gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345678901234567890", gl_string("abcdefghij12345678901234567890lmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 5, "", gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345", gl_string("abcdefghij12345pqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 5, "1234567890", gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345678901234567890", gl_string("abcdefghij12345678901234567890pqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 9, "", gl_string("abcdefghijt"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345", gl_string("abcdefghij12345t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 9, "1234567890", gl_string("abcdefghij1234567890t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345678901234567890", gl_string("abcdefghij12345678901234567890t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 10, "", gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345", gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 10, "1234567890", gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345678901234567890", gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 11, "", gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345", gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 11, "1234567890", gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345678901234567890", gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 0, "", gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345", gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 0, "1234567890", gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345678901234567890", gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 1, "", gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345", gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 1, "1234567890", gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345678901234567890", gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 2, "", gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345", gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 2, "1234567890", gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345678901234567890", gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, 0, "", gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345", gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, 0, "1234567890", gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345678901234567890", gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, 1, "", gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345", gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, 1, "1234567890", gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345678901234567890", gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 21, 0, "", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 21, 0, "1234567890", gl_string("can't happen"));
    _test_string_replace_size_size_pointer(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345678901234567890", gl_string("can't happen"));
  }

  void
  _test_string_replace_size_size_pointer_size(gl_string s, size_t pos, size_t n1,
                                              const char* str, size_t n2,
                                              gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.replace(pos, n1, str, n2);
        
      TS_ASSERT(pos <= old_size);
      TS_ASSERT(s == expected);
      size_t xlen = std::min(n1, old_size - pos);
      size_t rlen = n2;
      TS_ASSERT(s.size() == old_size - xlen + rlen);
    }
    catch (...)
    {
      TS_ASSERT(pos > old_size);
      TS_ASSERT(s == s0);
    }
#endif
  }


  void test_string_replace_size_size_pointer_size0()
  {
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "12345", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "12345", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "12345", 2, gl_string("12"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "12345", 4, gl_string("1234"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "12345", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "1234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "1234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "1234567890", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "1234567890", 9, gl_string("123456789"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "1234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "12345678901234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "12345678901234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "12345678901234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "12345678901234567890", 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 0, "12345678901234567890", 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "12345", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "12345", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "12345", 2, gl_string("12"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "12345", 4, gl_string("1234"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "12345", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "1234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "1234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "1234567890", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "1234567890", 9, gl_string("123456789"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "1234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "12345678901234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "12345678901234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "12345678901234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "12345678901234567890", 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 0, 1, "12345678901234567890", 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "12345", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "12345", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "12345", 2, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "12345", 4, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "12345", 5, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "1234567890", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "1234567890", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "1234567890", 5, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "1234567890", 9, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "1234567890", 10, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "12345678901234567890", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "12345678901234567890", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "12345678901234567890", 10, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "12345678901234567890", 19, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string(""), 1, 0, "12345678901234567890", 20, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "12345", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "12345", 1, gl_string("1abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "12345", 2, gl_string("12abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "12345", 4, gl_string("1234abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "12345", 5, gl_string("12345abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "1234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "1234567890", 1, gl_string("1abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "1234567890", 5, gl_string("12345abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "1234567890", 9, gl_string("123456789abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "1234567890", 10, gl_string("1234567890abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "12345678901234567890", 1, gl_string("1abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "12345678901234567890", 10, gl_string("1234567890abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "12345678901234567890", 19, gl_string("1234567890123456789abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 0, "12345678901234567890", 20, gl_string("12345678901234567890abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "", 0, gl_string("bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "12345", 0, gl_string("bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "12345", 1, gl_string("1bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "12345", 2, gl_string("12bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "12345", 4, gl_string("1234bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "12345", 5, gl_string("12345bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "1234567890", 0, gl_string("bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "1234567890", 1, gl_string("1bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "1234567890", 5, gl_string("12345bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "1234567890", 9, gl_string("123456789bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "1234567890", 10, gl_string("1234567890bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "12345678901234567890", 0, gl_string("bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "12345678901234567890", 1, gl_string("1bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "12345678901234567890", 10, gl_string("1234567890bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "12345678901234567890", 19, gl_string("1234567890123456789bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 1, "12345678901234567890", 20, gl_string("12345678901234567890bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "", 0, gl_string("cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "12345", 0, gl_string("cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "12345", 1, gl_string("1cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "12345", 2, gl_string("12cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "12345", 4, gl_string("1234cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "12345", 5, gl_string("12345cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "1234567890", 0, gl_string("cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "1234567890", 1, gl_string("1cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "1234567890", 5, gl_string("12345cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "1234567890", 9, gl_string("123456789cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "1234567890", 10, gl_string("1234567890cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "12345678901234567890", 0, gl_string("cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "12345678901234567890", 1, gl_string("1cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "12345678901234567890", 10, gl_string("1234567890cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "12345678901234567890", 19, gl_string("1234567890123456789cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 2, "12345678901234567890", 20, gl_string("12345678901234567890cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "", 0, gl_string("e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "12345", 0, gl_string("e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "12345", 1, gl_string("1e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "12345", 2, gl_string("12e"));
  }


  void test_string_replace_size_size_pointer_size1()
  {
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "12345", 4, gl_string("1234e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "12345", 5, gl_string("12345e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "1234567890", 0, gl_string("e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "1234567890", 1, gl_string("1e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "1234567890", 5, gl_string("12345e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "1234567890", 9, gl_string("123456789e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "1234567890", 10, gl_string("1234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "12345678901234567890", 0, gl_string("e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "12345678901234567890", 1, gl_string("1e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "12345678901234567890", 10, gl_string("1234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "12345678901234567890", 19, gl_string("1234567890123456789e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 4, "12345678901234567890", 20, gl_string("12345678901234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "12345", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "12345", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "12345", 2, gl_string("12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "12345", 4, gl_string("1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "12345", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "1234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "1234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "1234567890", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "1234567890", 9, gl_string("123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "1234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "12345678901234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "12345678901234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "12345678901234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "12345678901234567890", 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 5, "12345678901234567890", 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "12345", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "12345", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "12345", 2, gl_string("12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "12345", 4, gl_string("1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "12345", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "1234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "1234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "1234567890", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "1234567890", 9, gl_string("123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "1234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "12345678901234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "12345678901234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "12345678901234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "12345678901234567890", 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 0, 6, "12345678901234567890", 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "12345", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "12345", 1, gl_string("a1bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "12345", 2, gl_string("a12bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "12345", 4, gl_string("a1234bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "12345", 5, gl_string("a12345bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "1234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "1234567890", 1, gl_string("a1bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "1234567890", 5, gl_string("a12345bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "1234567890", 9, gl_string("a123456789bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "1234567890", 10, gl_string("a1234567890bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "12345678901234567890", 1, gl_string("a1bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "12345678901234567890", 10, gl_string("a1234567890bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "12345678901234567890", 19, gl_string("a1234567890123456789bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 0, "12345678901234567890", 20, gl_string("a12345678901234567890bcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "", 0, gl_string("acde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "12345", 0, gl_string("acde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "12345", 1, gl_string("a1cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "12345", 2, gl_string("a12cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "12345", 4, gl_string("a1234cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "12345", 5, gl_string("a12345cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "1234567890", 0, gl_string("acde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "1234567890", 1, gl_string("a1cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "1234567890", 5, gl_string("a12345cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "1234567890", 9, gl_string("a123456789cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "1234567890", 10, gl_string("a1234567890cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "12345678901234567890", 0, gl_string("acde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "12345678901234567890", 1, gl_string("a1cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "12345678901234567890", 10, gl_string("a1234567890cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "12345678901234567890", 19, gl_string("a1234567890123456789cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 1, "12345678901234567890", 20, gl_string("a12345678901234567890cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "", 0, gl_string("ade"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "12345", 0, gl_string("ade"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "12345", 1, gl_string("a1de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "12345", 2, gl_string("a12de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "12345", 4, gl_string("a1234de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "12345", 5, gl_string("a12345de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "1234567890", 0, gl_string("ade"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "1234567890", 1, gl_string("a1de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "1234567890", 5, gl_string("a12345de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "1234567890", 9, gl_string("a123456789de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "1234567890", 10, gl_string("a1234567890de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "12345678901234567890", 0, gl_string("ade"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "12345678901234567890", 1, gl_string("a1de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "12345678901234567890", 10, gl_string("a1234567890de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "12345678901234567890", 19, gl_string("a1234567890123456789de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 2, "12345678901234567890", 20, gl_string("a12345678901234567890de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "", 0, gl_string("ae"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "12345", 0, gl_string("ae"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "12345", 1, gl_string("a1e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "12345", 2, gl_string("a12e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "12345", 4, gl_string("a1234e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "12345", 5, gl_string("a12345e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "1234567890", 0, gl_string("ae"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "1234567890", 1, gl_string("a1e"));
  }


  void test_string_replace_size_size_pointer_size2()
  {
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "1234567890", 5, gl_string("a12345e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "1234567890", 9, gl_string("a123456789e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "1234567890", 10, gl_string("a1234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "12345678901234567890", 0, gl_string("ae"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "12345678901234567890", 1, gl_string("a1e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "12345678901234567890", 10, gl_string("a1234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "12345678901234567890", 19, gl_string("a1234567890123456789e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 3, "12345678901234567890", 20, gl_string("a12345678901234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "12345", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "12345", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "12345", 2, gl_string("a12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "12345", 4, gl_string("a1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "12345", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "1234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "1234567890", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "1234567890", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "1234567890", 9, gl_string("a123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "1234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "12345678901234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "12345678901234567890", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "12345678901234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "12345678901234567890", 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 4, "12345678901234567890", 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "12345", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "12345", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "12345", 2, gl_string("a12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "12345", 4, gl_string("a1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "12345", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "1234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "1234567890", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "1234567890", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "1234567890", 9, gl_string("a123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "1234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "12345678901234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "12345678901234567890", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "12345678901234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "12345678901234567890", 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 1, 5, "12345678901234567890", 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "12345", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "12345", 1, gl_string("ab1cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "12345", 2, gl_string("ab12cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "12345", 4, gl_string("ab1234cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "12345", 5, gl_string("ab12345cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "1234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "1234567890", 1, gl_string("ab1cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "1234567890", 5, gl_string("ab12345cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "1234567890", 9, gl_string("ab123456789cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "1234567890", 10, gl_string("ab1234567890cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "12345678901234567890", 1, gl_string("ab1cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "12345678901234567890", 10, gl_string("ab1234567890cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "12345678901234567890", 19, gl_string("ab1234567890123456789cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 0, "12345678901234567890", 20, gl_string("ab12345678901234567890cde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "", 0, gl_string("abde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "12345", 0, gl_string("abde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "12345", 1, gl_string("ab1de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "12345", 2, gl_string("ab12de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "12345", 4, gl_string("ab1234de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "12345", 5, gl_string("ab12345de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "1234567890", 0, gl_string("abde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "1234567890", 1, gl_string("ab1de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "1234567890", 5, gl_string("ab12345de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "1234567890", 9, gl_string("ab123456789de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "1234567890", 10, gl_string("ab1234567890de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "12345678901234567890", 0, gl_string("abde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "12345678901234567890", 1, gl_string("ab1de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "12345678901234567890", 10, gl_string("ab1234567890de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "12345678901234567890", 19, gl_string("ab1234567890123456789de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 1, "12345678901234567890", 20, gl_string("ab12345678901234567890de"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "", 0, gl_string("abe"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "12345", 0, gl_string("abe"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "12345", 1, gl_string("ab1e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "12345", 2, gl_string("ab12e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "12345", 4, gl_string("ab1234e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "12345", 5, gl_string("ab12345e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "1234567890", 0, gl_string("abe"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "1234567890", 1, gl_string("ab1e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "1234567890", 5, gl_string("ab12345e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "1234567890", 9, gl_string("ab123456789e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "1234567890", 10, gl_string("ab1234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "12345678901234567890", 0, gl_string("abe"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "12345678901234567890", 1, gl_string("ab1e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "12345678901234567890", 10, gl_string("ab1234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "12345678901234567890", 19, gl_string("ab1234567890123456789e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 2, "12345678901234567890", 20, gl_string("ab12345678901234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "", 0, gl_string("ab"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "12345", 0, gl_string("ab"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "12345", 1, gl_string("ab1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "12345", 2, gl_string("ab12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "12345", 4, gl_string("ab1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "12345", 5, gl_string("ab12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "1234567890", 0, gl_string("ab"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "1234567890", 1, gl_string("ab1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "1234567890", 5, gl_string("ab12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "1234567890", 9, gl_string("ab123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "1234567890", 10, gl_string("ab1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "12345678901234567890", 0, gl_string("ab"));
  }


  void test_string_replace_size_size_pointer_size3()
  {
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "12345678901234567890", 1, gl_string("ab1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "12345678901234567890", 10, gl_string("ab1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "12345678901234567890", 19, gl_string("ab1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 3, "12345678901234567890", 20, gl_string("ab12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "", 0, gl_string("ab"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "12345", 0, gl_string("ab"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "12345", 1, gl_string("ab1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "12345", 2, gl_string("ab12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "12345", 4, gl_string("ab1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "12345", 5, gl_string("ab12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "1234567890", 0, gl_string("ab"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "1234567890", 1, gl_string("ab1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "1234567890", 5, gl_string("ab12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "1234567890", 9, gl_string("ab123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "1234567890", 10, gl_string("ab1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "12345678901234567890", 0, gl_string("ab"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "12345678901234567890", 1, gl_string("ab1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "12345678901234567890", 10, gl_string("ab1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "12345678901234567890", 19, gl_string("ab1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 2, 4, "12345678901234567890", 20, gl_string("ab12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "12345", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "12345", 1, gl_string("abcd1e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "12345", 2, gl_string("abcd12e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "12345", 4, gl_string("abcd1234e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "12345", 5, gl_string("abcd12345e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "1234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "1234567890", 1, gl_string("abcd1e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "1234567890", 5, gl_string("abcd12345e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "1234567890", 9, gl_string("abcd123456789e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "1234567890", 10, gl_string("abcd1234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "12345678901234567890", 1, gl_string("abcd1e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "12345678901234567890", 10, gl_string("abcd1234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "12345678901234567890", 19, gl_string("abcd1234567890123456789e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 0, "12345678901234567890", 20, gl_string("abcd12345678901234567890e"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "", 0, gl_string("abcd"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "12345", 0, gl_string("abcd"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "12345", 1, gl_string("abcd1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "12345", 2, gl_string("abcd12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "12345", 4, gl_string("abcd1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "12345", 5, gl_string("abcd12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "1234567890", 0, gl_string("abcd"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "1234567890", 1, gl_string("abcd1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "1234567890", 5, gl_string("abcd12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "1234567890", 9, gl_string("abcd123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "1234567890", 10, gl_string("abcd1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "12345678901234567890", 0, gl_string("abcd"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "12345678901234567890", 1, gl_string("abcd1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "12345678901234567890", 10, gl_string("abcd1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "12345678901234567890", 19, gl_string("abcd1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 1, "12345678901234567890", 20, gl_string("abcd12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "", 0, gl_string("abcd"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "12345", 0, gl_string("abcd"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "12345", 1, gl_string("abcd1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "12345", 2, gl_string("abcd12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "12345", 4, gl_string("abcd1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "12345", 5, gl_string("abcd12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "1234567890", 0, gl_string("abcd"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "1234567890", 1, gl_string("abcd1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "1234567890", 5, gl_string("abcd12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "1234567890", 9, gl_string("abcd123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "1234567890", 10, gl_string("abcd1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "12345678901234567890", 0, gl_string("abcd"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "12345678901234567890", 1, gl_string("abcd1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "12345678901234567890", 10, gl_string("abcd1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "12345678901234567890", 19, gl_string("abcd1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 4, 2, "12345678901234567890", 20, gl_string("abcd12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "12345", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "12345", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "12345", 2, gl_string("abcde12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "12345", 4, gl_string("abcde1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "12345", 5, gl_string("abcde12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "1234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "1234567890", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "1234567890", 5, gl_string("abcde12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "1234567890", 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "1234567890", 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "12345678901234567890", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "12345678901234567890", 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "12345678901234567890", 19, gl_string("abcde1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 0, "12345678901234567890", 20, gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "12345", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "12345", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "12345", 2, gl_string("abcde12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "12345", 4, gl_string("abcde1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "12345", 5, gl_string("abcde12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "1234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "1234567890", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "1234567890", 5, gl_string("abcde12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "1234567890", 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "1234567890", 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "12345678901234567890", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "12345678901234567890", 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "12345678901234567890", 19, gl_string("abcde1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 5, 1, "12345678901234567890", 20, gl_string("abcde12345678901234567890"));
  }


  void test_string_replace_size_size_pointer_size4()
  {
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "12345", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "12345", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "12345", 2, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "12345", 4, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "12345", 5, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "1234567890", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "1234567890", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "1234567890", 5, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "1234567890", 9, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "1234567890", 10, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "12345678901234567890", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "12345678901234567890", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "12345678901234567890", 10, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "12345678901234567890", 19, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcde"), 6, 0, "12345678901234567890", 20, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "12345", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "12345", 1, gl_string("1abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "12345", 2, gl_string("12abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "12345", 4, gl_string("1234abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "12345", 5, gl_string("12345abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "1234567890", 1, gl_string("1abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "1234567890", 5, gl_string("12345abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "1234567890", 9, gl_string("123456789abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "1234567890", 10, gl_string("1234567890abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "12345678901234567890", 1, gl_string("1abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "12345678901234567890", 10, gl_string("1234567890abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "12345678901234567890", 19, gl_string("1234567890123456789abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 0, "12345678901234567890", 20, gl_string("12345678901234567890abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "", 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "12345", 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "12345", 1, gl_string("1bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "12345", 2, gl_string("12bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "12345", 4, gl_string("1234bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "12345", 5, gl_string("12345bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "1234567890", 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "1234567890", 1, gl_string("1bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "1234567890", 5, gl_string("12345bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "1234567890", 9, gl_string("123456789bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "1234567890", 10, gl_string("1234567890bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "12345678901234567890", 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "12345678901234567890", 1, gl_string("1bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "12345678901234567890", 10, gl_string("1234567890bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "12345678901234567890", 19, gl_string("1234567890123456789bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 1, "12345678901234567890", 20, gl_string("12345678901234567890bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "", 0, gl_string("fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "12345", 0, gl_string("fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "12345", 1, gl_string("1fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "12345", 2, gl_string("12fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "12345", 4, gl_string("1234fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "12345", 5, gl_string("12345fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "1234567890", 0, gl_string("fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "1234567890", 1, gl_string("1fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "1234567890", 5, gl_string("12345fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "1234567890", 9, gl_string("123456789fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "1234567890", 10, gl_string("1234567890fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "12345678901234567890", 0, gl_string("fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "12345678901234567890", 1, gl_string("1fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "12345678901234567890", 10, gl_string("1234567890fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "12345678901234567890", 19, gl_string("1234567890123456789fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 5, "12345678901234567890", 20, gl_string("12345678901234567890fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "", 0, gl_string("j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "12345", 0, gl_string("j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "12345", 1, gl_string("1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "12345", 2, gl_string("12j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "12345", 4, gl_string("1234j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "12345", 5, gl_string("12345j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "1234567890", 0, gl_string("j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "1234567890", 1, gl_string("1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "1234567890", 5, gl_string("12345j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "1234567890", 9, gl_string("123456789j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "1234567890", 10, gl_string("1234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "12345678901234567890", 0, gl_string("j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "12345678901234567890", 1, gl_string("1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "12345678901234567890", 10, gl_string("1234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "12345678901234567890", 19, gl_string("1234567890123456789j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 9, "12345678901234567890", 20, gl_string("12345678901234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "12345", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "12345", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "12345", 2, gl_string("12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "12345", 4, gl_string("1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "12345", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "1234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "1234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "1234567890", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "1234567890", 9, gl_string("123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "1234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "12345678901234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "12345678901234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "12345678901234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "12345678901234567890", 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 10, "12345678901234567890", 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "12345", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "12345", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "12345", 2, gl_string("12"));
  }


  void test_string_replace_size_size_pointer_size5()
  {
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "12345", 4, gl_string("1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "12345", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "1234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "1234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "1234567890", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "1234567890", 9, gl_string("123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "1234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "12345678901234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "12345678901234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "12345678901234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "12345678901234567890", 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 0, 11, "12345678901234567890", 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "12345", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "12345", 1, gl_string("a1bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "12345", 2, gl_string("a12bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "12345", 4, gl_string("a1234bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "12345", 5, gl_string("a12345bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "1234567890", 1, gl_string("a1bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "1234567890", 5, gl_string("a12345bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "1234567890", 9, gl_string("a123456789bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "1234567890", 10, gl_string("a1234567890bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "12345678901234567890", 1, gl_string("a1bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "12345678901234567890", 10, gl_string("a1234567890bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "12345678901234567890", 19, gl_string("a1234567890123456789bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 0, "12345678901234567890", 20, gl_string("a12345678901234567890bcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "", 0, gl_string("acdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "12345", 0, gl_string("acdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "12345", 1, gl_string("a1cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "12345", 2, gl_string("a12cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "12345", 4, gl_string("a1234cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "12345", 5, gl_string("a12345cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "1234567890", 0, gl_string("acdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "1234567890", 1, gl_string("a1cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "1234567890", 5, gl_string("a12345cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "1234567890", 9, gl_string("a123456789cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "1234567890", 10, gl_string("a1234567890cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "12345678901234567890", 0, gl_string("acdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "12345678901234567890", 1, gl_string("a1cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "12345678901234567890", 10, gl_string("a1234567890cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "12345678901234567890", 19, gl_string("a1234567890123456789cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 1, "12345678901234567890", 20, gl_string("a12345678901234567890cdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "", 0, gl_string("afghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "12345", 0, gl_string("afghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "12345", 1, gl_string("a1fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "12345", 2, gl_string("a12fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "12345", 4, gl_string("a1234fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "12345", 5, gl_string("a12345fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "1234567890", 0, gl_string("afghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "1234567890", 1, gl_string("a1fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "1234567890", 5, gl_string("a12345fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "1234567890", 9, gl_string("a123456789fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "1234567890", 10, gl_string("a1234567890fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "12345678901234567890", 0, gl_string("afghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "12345678901234567890", 1, gl_string("a1fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "12345678901234567890", 10, gl_string("a1234567890fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "12345678901234567890", 19, gl_string("a1234567890123456789fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 4, "12345678901234567890", 20, gl_string("a12345678901234567890fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "", 0, gl_string("aj"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "12345", 0, gl_string("aj"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "12345", 1, gl_string("a1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "12345", 2, gl_string("a12j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "12345", 4, gl_string("a1234j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "12345", 5, gl_string("a12345j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "1234567890", 0, gl_string("aj"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "1234567890", 1, gl_string("a1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "1234567890", 5, gl_string("a12345j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "1234567890", 9, gl_string("a123456789j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "1234567890", 10, gl_string("a1234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "12345678901234567890", 0, gl_string("aj"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "12345678901234567890", 1, gl_string("a1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "12345678901234567890", 10, gl_string("a1234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "12345678901234567890", 19, gl_string("a1234567890123456789j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 8, "12345678901234567890", 20, gl_string("a12345678901234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "12345", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "12345", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "12345", 2, gl_string("a12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "12345", 4, gl_string("a1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "12345", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "1234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "1234567890", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "1234567890", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "1234567890", 9, gl_string("a123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "1234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "12345678901234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "12345678901234567890", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "12345678901234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "12345678901234567890", 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 9, "12345678901234567890", 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "12345", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "12345", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "12345", 2, gl_string("a12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "12345", 4, gl_string("a1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "12345", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "1234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "1234567890", 1, gl_string("a1"));
  }


  void test_string_replace_size_size_pointer_size6()
  {
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "1234567890", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "1234567890", 9, gl_string("a123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "1234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "12345678901234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "12345678901234567890", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "12345678901234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "12345678901234567890", 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 1, 10, "12345678901234567890", 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "12345", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "12345", 1, gl_string("abcde1fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "12345", 2, gl_string("abcde12fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "12345", 4, gl_string("abcde1234fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "12345", 5, gl_string("abcde12345fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "1234567890", 1, gl_string("abcde1fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "1234567890", 5, gl_string("abcde12345fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "1234567890", 9, gl_string("abcde123456789fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "1234567890", 10, gl_string("abcde1234567890fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "12345678901234567890", 1, gl_string("abcde1fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "12345678901234567890", 10, gl_string("abcde1234567890fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "12345678901234567890", 19, gl_string("abcde1234567890123456789fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 0, "12345678901234567890", 20, gl_string("abcde12345678901234567890fghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "", 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "12345", 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "12345", 1, gl_string("abcde1ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "12345", 2, gl_string("abcde12ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "12345", 4, gl_string("abcde1234ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "12345", 5, gl_string("abcde12345ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "1234567890", 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "1234567890", 1, gl_string("abcde1ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "1234567890", 5, gl_string("abcde12345ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "1234567890", 9, gl_string("abcde123456789ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "1234567890", 10, gl_string("abcde1234567890ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "12345678901234567890", 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "12345678901234567890", 1, gl_string("abcde1ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "12345678901234567890", 10, gl_string("abcde1234567890ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "12345678901234567890", 19, gl_string("abcde1234567890123456789ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 1, "12345678901234567890", 20, gl_string("abcde12345678901234567890ghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "", 0, gl_string("abcdehij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "12345", 0, gl_string("abcdehij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "12345", 1, gl_string("abcde1hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "12345", 2, gl_string("abcde12hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "12345", 4, gl_string("abcde1234hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "12345", 5, gl_string("abcde12345hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "1234567890", 0, gl_string("abcdehij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "1234567890", 1, gl_string("abcde1hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "1234567890", 5, gl_string("abcde12345hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "1234567890", 9, gl_string("abcde123456789hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "1234567890", 10, gl_string("abcde1234567890hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "12345678901234567890", 0, gl_string("abcdehij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "12345678901234567890", 1, gl_string("abcde1hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "12345678901234567890", 10, gl_string("abcde1234567890hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "12345678901234567890", 19, gl_string("abcde1234567890123456789hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 2, "12345678901234567890", 20, gl_string("abcde12345678901234567890hij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "", 0, gl_string("abcdej"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "12345", 0, gl_string("abcdej"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "12345", 1, gl_string("abcde1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "12345", 2, gl_string("abcde12j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "12345", 4, gl_string("abcde1234j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "12345", 5, gl_string("abcde12345j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "1234567890", 0, gl_string("abcdej"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "1234567890", 1, gl_string("abcde1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "1234567890", 5, gl_string("abcde12345j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "1234567890", 9, gl_string("abcde123456789j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "1234567890", 10, gl_string("abcde1234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "12345678901234567890", 0, gl_string("abcdej"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "12345678901234567890", 1, gl_string("abcde1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "12345678901234567890", 10, gl_string("abcde1234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "12345678901234567890", 19, gl_string("abcde1234567890123456789j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 4, "12345678901234567890", 20, gl_string("abcde12345678901234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "12345", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "12345", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "12345", 2, gl_string("abcde12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "12345", 4, gl_string("abcde1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "12345", 5, gl_string("abcde12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "1234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "1234567890", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "1234567890", 5, gl_string("abcde12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "1234567890", 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "1234567890", 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "12345678901234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "12345678901234567890", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "12345678901234567890", 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "12345678901234567890", 19, gl_string("abcde1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 5, "12345678901234567890", 20, gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "12345", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "12345", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "12345", 2, gl_string("abcde12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "12345", 4, gl_string("abcde1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "12345", 5, gl_string("abcde12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "1234567890", 0, gl_string("abcde"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "1234567890", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "1234567890", 5, gl_string("abcde12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "1234567890", 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "1234567890", 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "12345678901234567890", 0, gl_string("abcde"));
  }


  void test_string_replace_size_size_pointer_size7()
  {
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "12345678901234567890", 1, gl_string("abcde1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "12345678901234567890", 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "12345678901234567890", 19, gl_string("abcde1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 5, 6, "12345678901234567890", 20, gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "12345", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "12345", 1, gl_string("abcdefghi1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "12345", 2, gl_string("abcdefghi12j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "12345", 4, gl_string("abcdefghi1234j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "12345", 5, gl_string("abcdefghi12345j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "1234567890", 1, gl_string("abcdefghi1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "1234567890", 5, gl_string("abcdefghi12345j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "1234567890", 9, gl_string("abcdefghi123456789j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "1234567890", 10, gl_string("abcdefghi1234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "12345678901234567890", 1, gl_string("abcdefghi1j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "12345678901234567890", 10, gl_string("abcdefghi1234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "12345678901234567890", 19, gl_string("abcdefghi1234567890123456789j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 0, "12345678901234567890", 20, gl_string("abcdefghi12345678901234567890j"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "", 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "12345", 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "12345", 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "12345", 2, gl_string("abcdefghi12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "12345", 4, gl_string("abcdefghi1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "12345", 5, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "1234567890", 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "1234567890", 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "1234567890", 5, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "1234567890", 9, gl_string("abcdefghi123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "1234567890", 10, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "12345678901234567890", 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "12345678901234567890", 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "12345678901234567890", 10, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "12345678901234567890", 19, gl_string("abcdefghi1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 1, "12345678901234567890", 20, gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "", 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "12345", 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "12345", 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "12345", 2, gl_string("abcdefghi12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "12345", 4, gl_string("abcdefghi1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "12345", 5, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "1234567890", 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "1234567890", 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "1234567890", 5, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "1234567890", 9, gl_string("abcdefghi123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "1234567890", 10, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "12345678901234567890", 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "12345678901234567890", 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "12345678901234567890", 10, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "12345678901234567890", 19, gl_string("abcdefghi1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 9, 2, "12345678901234567890", 20, gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "12345", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "12345", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "12345", 2, gl_string("abcdefghij12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "12345", 4, gl_string("abcdefghij1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "12345", 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "1234567890", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "1234567890", 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "1234567890", 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "1234567890", 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "12345678901234567890", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "12345678901234567890", 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "12345678901234567890", 19, gl_string("abcdefghij1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 0, "12345678901234567890", 20, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "12345", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "12345", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "12345", 2, gl_string("abcdefghij12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "12345", 4, gl_string("abcdefghij1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "12345", 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "1234567890", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "1234567890", 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "1234567890", 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "1234567890", 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "12345678901234567890", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "12345678901234567890", 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "12345678901234567890", 19, gl_string("abcdefghij1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 10, 1, "12345678901234567890", 20, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "12345", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "12345", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "12345", 2, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "12345", 4, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "12345", 5, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "1234567890", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "1234567890", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "1234567890", 5, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "1234567890", 9, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "1234567890", 10, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "12345678901234567890", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "12345678901234567890", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "12345678901234567890", 10, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "12345678901234567890", 19, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghij"), 11, 0, "12345678901234567890", 20, gl_string("can't happen"));
  }


  void test_string_replace_size_size_pointer_size8()
  {
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345", 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345", 2, gl_string("12abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345", 4, gl_string("1234abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345", 5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "1234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "1234567890", 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "1234567890", 5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "1234567890", 9, gl_string("123456789abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "1234567890", 10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345678901234567890", 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345678901234567890", 10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345678901234567890", 19, gl_string("1234567890123456789abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 0, "12345678901234567890", 20, gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "", 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345", 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345", 1, gl_string("1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345", 2, gl_string("12bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345", 4, gl_string("1234bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345", 5, gl_string("12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "1234567890", 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "1234567890", 1, gl_string("1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "1234567890", 5, gl_string("12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "1234567890", 9, gl_string("123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "1234567890", 10, gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345678901234567890", 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345678901234567890", 1, gl_string("1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345678901234567890", 10, gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345678901234567890", 19, gl_string("1234567890123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 1, "12345678901234567890", 20, gl_string("12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "", 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345", 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345", 1, gl_string("1klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345", 2, gl_string("12klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345", 4, gl_string("1234klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345", 5, gl_string("12345klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "1234567890", 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "1234567890", 1, gl_string("1klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "1234567890", 5, gl_string("12345klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "1234567890", 9, gl_string("123456789klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "1234567890", 10, gl_string("1234567890klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345678901234567890", 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345678901234567890", 1, gl_string("1klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345678901234567890", 10, gl_string("1234567890klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345678901234567890", 19, gl_string("1234567890123456789klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 10, "12345678901234567890", 20, gl_string("12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "", 0, gl_string("t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345", 0, gl_string("t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345", 1, gl_string("1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345", 2, gl_string("12t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345", 4, gl_string("1234t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345", 5, gl_string("12345t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "1234567890", 0, gl_string("t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "1234567890", 1, gl_string("1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "1234567890", 5, gl_string("12345t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "1234567890", 9, gl_string("123456789t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "1234567890", 10, gl_string("1234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345678901234567890", 0, gl_string("t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345678901234567890", 1, gl_string("1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345678901234567890", 10, gl_string("1234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345678901234567890", 19, gl_string("1234567890123456789t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 19, "12345678901234567890", 20, gl_string("12345678901234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345", 2, gl_string("12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345", 4, gl_string("1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "1234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "1234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "1234567890", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "1234567890", 9, gl_string("123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "1234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345678901234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345678901234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345678901234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345678901234567890", 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 20, "12345678901234567890", 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345", 2, gl_string("12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345", 4, gl_string("1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "1234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "1234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "1234567890", 5, gl_string("12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "1234567890", 9, gl_string("123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "1234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345678901234567890", 0, gl_string(""));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345678901234567890", 1, gl_string("1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345678901234567890", 10, gl_string("1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345678901234567890", 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 0, 21, "12345678901234567890", 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345", 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345", 2, gl_string("a12bcdefghijklmnopqrst"));
  }


  void test_string_replace_size_size_pointer_size9()
  {
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345", 4, gl_string("a1234bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345", 5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "1234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "1234567890", 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "1234567890", 5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "1234567890", 9, gl_string("a123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "1234567890", 10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345678901234567890", 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345678901234567890", 10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345678901234567890", 19, gl_string("a1234567890123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 0, "12345678901234567890", 20, gl_string("a12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "", 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345", 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345", 1, gl_string("a1cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345", 2, gl_string("a12cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345", 4, gl_string("a1234cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345", 5, gl_string("a12345cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "1234567890", 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "1234567890", 1, gl_string("a1cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "1234567890", 5, gl_string("a12345cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "1234567890", 9, gl_string("a123456789cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "1234567890", 10, gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345678901234567890", 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345678901234567890", 1, gl_string("a1cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345678901234567890", 10, gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345678901234567890", 19, gl_string("a1234567890123456789cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 1, "12345678901234567890", 20, gl_string("a12345678901234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "", 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345", 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345", 1, gl_string("a1klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345", 2, gl_string("a12klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345", 4, gl_string("a1234klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345", 5, gl_string("a12345klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "1234567890", 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "1234567890", 1, gl_string("a1klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "1234567890", 5, gl_string("a12345klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "1234567890", 9, gl_string("a123456789klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "1234567890", 10, gl_string("a1234567890klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345678901234567890", 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345678901234567890", 1, gl_string("a1klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345678901234567890", 10, gl_string("a1234567890klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345678901234567890", 19, gl_string("a1234567890123456789klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 9, "12345678901234567890", 20, gl_string("a12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "", 0, gl_string("at"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345", 0, gl_string("at"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345", 1, gl_string("a1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345", 2, gl_string("a12t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345", 4, gl_string("a1234t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345", 5, gl_string("a12345t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "1234567890", 0, gl_string("at"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "1234567890", 1, gl_string("a1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "1234567890", 5, gl_string("a12345t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "1234567890", 9, gl_string("a123456789t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "1234567890", 10, gl_string("a1234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345678901234567890", 0, gl_string("at"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345678901234567890", 1, gl_string("a1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345678901234567890", 10, gl_string("a1234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345678901234567890", 19, gl_string("a1234567890123456789t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 18, "12345678901234567890", 20, gl_string("a12345678901234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345", 2, gl_string("a12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345", 4, gl_string("a1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "1234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "1234567890", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "1234567890", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "1234567890", 9, gl_string("a123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "1234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345678901234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345678901234567890", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345678901234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345678901234567890", 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 19, "12345678901234567890", 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345", 2, gl_string("a12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345", 4, gl_string("a1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "1234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "1234567890", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "1234567890", 5, gl_string("a12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "1234567890", 9, gl_string("a123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "1234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345678901234567890", 0, gl_string("a"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345678901234567890", 1, gl_string("a1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345678901234567890", 10, gl_string("a1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345678901234567890", 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 1, 20, "12345678901234567890", 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345", 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345", 2, gl_string("abcdefghij12klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345", 4, gl_string("abcdefghij1234klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345", 5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "1234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "1234567890", 1, gl_string("abcdefghij1klmnopqrst"));
  }


  void test_string_replace_size_size_pointer_size10()
  {
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "1234567890", 5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "1234567890", 9, gl_string("abcdefghij123456789klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "1234567890", 10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345678901234567890", 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345678901234567890", 10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345678901234567890", 19, gl_string("abcdefghij1234567890123456789klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 0, "12345678901234567890", 20, gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "", 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345", 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345", 1, gl_string("abcdefghij1lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345", 2, gl_string("abcdefghij12lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345", 4, gl_string("abcdefghij1234lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345", 5, gl_string("abcdefghij12345lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "1234567890", 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "1234567890", 1, gl_string("abcdefghij1lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "1234567890", 5, gl_string("abcdefghij12345lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "1234567890", 9, gl_string("abcdefghij123456789lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "1234567890", 10, gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345678901234567890", 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345678901234567890", 1, gl_string("abcdefghij1lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345678901234567890", 10, gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345678901234567890", 19, gl_string("abcdefghij1234567890123456789lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 1, "12345678901234567890", 20, gl_string("abcdefghij12345678901234567890lmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "", 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345", 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345", 1, gl_string("abcdefghij1pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345", 2, gl_string("abcdefghij12pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345", 4, gl_string("abcdefghij1234pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345", 5, gl_string("abcdefghij12345pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "1234567890", 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "1234567890", 1, gl_string("abcdefghij1pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "1234567890", 5, gl_string("abcdefghij12345pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "1234567890", 9, gl_string("abcdefghij123456789pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "1234567890", 10, gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345678901234567890", 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345678901234567890", 1, gl_string("abcdefghij1pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345678901234567890", 10, gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345678901234567890", 19, gl_string("abcdefghij1234567890123456789pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 5, "12345678901234567890", 20, gl_string("abcdefghij12345678901234567890pqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "", 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345", 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345", 1, gl_string("abcdefghij1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345", 2, gl_string("abcdefghij12t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345", 4, gl_string("abcdefghij1234t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345", 5, gl_string("abcdefghij12345t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "1234567890", 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "1234567890", 1, gl_string("abcdefghij1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "1234567890", 5, gl_string("abcdefghij12345t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "1234567890", 9, gl_string("abcdefghij123456789t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "1234567890", 10, gl_string("abcdefghij1234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345678901234567890", 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345678901234567890", 1, gl_string("abcdefghij1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345678901234567890", 10, gl_string("abcdefghij1234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345678901234567890", 19, gl_string("abcdefghij1234567890123456789t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 9, "12345678901234567890", 20, gl_string("abcdefghij12345678901234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345", 2, gl_string("abcdefghij12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345", 4, gl_string("abcdefghij1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345", 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "1234567890", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "1234567890", 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "1234567890", 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "1234567890", 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345678901234567890", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345678901234567890", 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345678901234567890", 19, gl_string("abcdefghij1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 10, "12345678901234567890", 20, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345", 2, gl_string("abcdefghij12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345", 4, gl_string("abcdefghij1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345", 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "1234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "1234567890", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "1234567890", 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "1234567890", 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "1234567890", 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345678901234567890", 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345678901234567890", 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345678901234567890", 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345678901234567890", 19, gl_string("abcdefghij1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 10, 11, "12345678901234567890", 20, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345", 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345", 2, gl_string("abcdefghijklmnopqrs12t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345", 4, gl_string("abcdefghijklmnopqrs1234t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345", 5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "1234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "1234567890", 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "1234567890", 5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "1234567890", 9, gl_string("abcdefghijklmnopqrs123456789t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "1234567890", 10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrst"));
  }


  void test_string_replace_size_size_pointer_size11()
  {
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345678901234567890", 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345678901234567890", 10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345678901234567890", 19, gl_string("abcdefghijklmnopqrs1234567890123456789t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 0, "12345678901234567890", 20, gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "", 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345", 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345", 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345", 2, gl_string("abcdefghijklmnopqrs12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345", 4, gl_string("abcdefghijklmnopqrs1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345", 5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "1234567890", 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "1234567890", 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "1234567890", 5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "1234567890", 9, gl_string("abcdefghijklmnopqrs123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "1234567890", 10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345678901234567890", 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345678901234567890", 10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345678901234567890", 19, gl_string("abcdefghijklmnopqrs1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 1, "12345678901234567890", 20, gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "", 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345", 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345", 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345", 2, gl_string("abcdefghijklmnopqrs12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345", 4, gl_string("abcdefghijklmnopqrs1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345", 5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "1234567890", 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "1234567890", 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "1234567890", 5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "1234567890", 9, gl_string("abcdefghijklmnopqrs123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "1234567890", 10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345678901234567890", 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345678901234567890", 10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345678901234567890", 19, gl_string("abcdefghijklmnopqrs1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 19, 2, "12345678901234567890", 20, gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345", 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345", 2, gl_string("abcdefghijklmnopqrst12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345", 4, gl_string("abcdefghijklmnopqrst1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345", 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "1234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "1234567890", 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "1234567890", 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "1234567890", 9, gl_string("abcdefghijklmnopqrst123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "1234567890", 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345678901234567890", 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345678901234567890", 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345678901234567890", 19, gl_string("abcdefghijklmnopqrst1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 0, "12345678901234567890", 20, gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345", 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345", 2, gl_string("abcdefghijklmnopqrst12"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345", 4, gl_string("abcdefghijklmnopqrst1234"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345", 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "1234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "1234567890", 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "1234567890", 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "1234567890", 9, gl_string("abcdefghijklmnopqrst123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "1234567890", 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345678901234567890", 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345678901234567890", 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345678901234567890", 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345678901234567890", 19, gl_string("abcdefghijklmnopqrst1234567890123456789"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 20, 1, "12345678901234567890", 20, gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345", 2, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345", 4, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345", 5, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "1234567890", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "1234567890", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "1234567890", 5, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "1234567890", 9, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "1234567890", 10, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345678901234567890", 0, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345678901234567890", 1, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345678901234567890", 10, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345678901234567890", 19, gl_string("can't happen"));
    _test_string_replace_size_size_pointer_size(gl_string("abcdefghijklmnopqrst"), 21, 0, "12345678901234567890", 20, gl_string("can't happen"));
  }

  void
  _test_string_replace_size_size_size_char(gl_string s, size_t pos, size_t n1,
                                           size_t n2, char c,
                                           gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.replace(pos, n1, n2, c);
        
      TS_ASSERT(pos <= old_size);
      TS_ASSERT(s == expected);
      size_t xlen = std::min(n1, old_size - pos);
      size_t rlen = n2;
      TS_ASSERT(s.size() == old_size - xlen + rlen);
    }
    catch (...)
    {
      TS_ASSERT(pos > old_size);
      TS_ASSERT(s == s0);
    }
#endif
  }


  void test_string_replace_size_size_size_char0()
  {
    _test_string_replace_size_size_size_char(gl_string(""), 0, 0, 0, '2', gl_string(""));
    _test_string_replace_size_size_size_char(gl_string(""), 0, 0, 5, '2', gl_string("22222"));
    _test_string_replace_size_size_size_char(gl_string(""), 0, 0, 10, '2', gl_string("2222222222"));
    _test_string_replace_size_size_size_char(gl_string(""), 0, 0, 20, '2', gl_string("22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string(""), 0, 1, 0, '2', gl_string(""));
    _test_string_replace_size_size_size_char(gl_string(""), 0, 1, 5, '2', gl_string("22222"));
    _test_string_replace_size_size_size_char(gl_string(""), 0, 1, 10, '2', gl_string("2222222222"));
    _test_string_replace_size_size_size_char(gl_string(""), 0, 1, 20, '2', gl_string("22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string(""), 1, 0, 0, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string(""), 1, 0, 5, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string(""), 1, 0, 10, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string(""), 1, 0, 20, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 0, 0, '2', gl_string("abcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 0, 5, '2', gl_string("22222abcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 0, 10, '2', gl_string("2222222222abcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 0, 20, '2', gl_string("22222222222222222222abcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 1, 0, '2', gl_string("bcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 1, 5, '2', gl_string("22222bcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 1, 10, '2', gl_string("2222222222bcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 1, 20, '2', gl_string("22222222222222222222bcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 2, 0, '2', gl_string("cde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 2, 5, '2', gl_string("22222cde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 2, 10, '2', gl_string("2222222222cde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 2, 20, '2', gl_string("22222222222222222222cde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 4, 0, '2', gl_string("e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 4, 5, '2', gl_string("22222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 4, 10, '2', gl_string("2222222222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 4, 20, '2', gl_string("22222222222222222222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 5, 0, '2', gl_string(""));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 5, 5, '2', gl_string("22222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 5, 10, '2', gl_string("2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 5, 20, '2', gl_string("22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 6, 0, '2', gl_string(""));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 6, 5, '2', gl_string("22222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 6, 10, '2', gl_string("2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 0, 6, 20, '2', gl_string("22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 0, 0, '2', gl_string("abcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 0, 5, '2', gl_string("a22222bcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 0, 10, '2', gl_string("a2222222222bcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 0, 20, '2', gl_string("a22222222222222222222bcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 1, 0, '2', gl_string("acde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 1, 5, '2', gl_string("a22222cde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 1, 10, '2', gl_string("a2222222222cde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 1, 20, '2', gl_string("a22222222222222222222cde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 2, 0, '2', gl_string("ade"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 2, 5, '2', gl_string("a22222de"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 2, 10, '2', gl_string("a2222222222de"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 2, 20, '2', gl_string("a22222222222222222222de"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 3, 0, '2', gl_string("ae"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 3, 5, '2', gl_string("a22222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 3, 10, '2', gl_string("a2222222222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 3, 20, '2', gl_string("a22222222222222222222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 4, 0, '2', gl_string("a"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 4, 5, '2', gl_string("a22222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 4, 10, '2', gl_string("a2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 4, 20, '2', gl_string("a22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 5, 0, '2', gl_string("a"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 5, 5, '2', gl_string("a22222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 5, 10, '2', gl_string("a2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 1, 5, 20, '2', gl_string("a22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 0, 0, '2', gl_string("abcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 0, 5, '2', gl_string("ab22222cde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 0, 10, '2', gl_string("ab2222222222cde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 0, 20, '2', gl_string("ab22222222222222222222cde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 1, 0, '2', gl_string("abde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 1, 5, '2', gl_string("ab22222de"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 1, 10, '2', gl_string("ab2222222222de"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 1, 20, '2', gl_string("ab22222222222222222222de"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 2, 0, '2', gl_string("abe"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 2, 5, '2', gl_string("ab22222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 2, 10, '2', gl_string("ab2222222222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 2, 20, '2', gl_string("ab22222222222222222222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 3, 0, '2', gl_string("ab"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 3, 5, '2', gl_string("ab22222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 3, 10, '2', gl_string("ab2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 3, 20, '2', gl_string("ab22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 4, 0, '2', gl_string("ab"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 4, 5, '2', gl_string("ab22222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 4, 10, '2', gl_string("ab2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 2, 4, 20, '2', gl_string("ab22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 0, 0, '2', gl_string("abcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 0, 5, '2', gl_string("abcd22222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 0, 10, '2', gl_string("abcd2222222222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 0, 20, '2', gl_string("abcd22222222222222222222e"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 1, 0, '2', gl_string("abcd"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 1, 5, '2', gl_string("abcd22222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 1, 10, '2', gl_string("abcd2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 1, 20, '2', gl_string("abcd22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 2, 0, '2', gl_string("abcd"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 2, 5, '2', gl_string("abcd22222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 2, 10, '2', gl_string("abcd2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 4, 2, 20, '2', gl_string("abcd22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 5, 0, 0, '2', gl_string("abcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 5, 0, 5, '2', gl_string("abcde22222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 5, 0, 10, '2', gl_string("abcde2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 5, 0, 20, '2', gl_string("abcde22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 5, 1, 0, '2', gl_string("abcde"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 5, 1, 5, '2', gl_string("abcde22222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 5, 1, 10, '2', gl_string("abcde2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 5, 1, 20, '2', gl_string("abcde22222222222222222222"));
  }


  void test_string_replace_size_size_size_char1()
  {
    _test_string_replace_size_size_size_char(gl_string("abcde"), 6, 0, 0, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 6, 0, 5, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 6, 0, 10, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string("abcde"), 6, 0, 20, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 0, 0, '2', gl_string("abcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 0, 5, '2', gl_string("22222abcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 0, 10, '2', gl_string("2222222222abcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 0, 20, '2', gl_string("22222222222222222222abcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 1, 0, '2', gl_string("bcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 1, 5, '2', gl_string("22222bcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 1, 10, '2', gl_string("2222222222bcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 1, 20, '2', gl_string("22222222222222222222bcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 5, 0, '2', gl_string("fghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 5, 5, '2', gl_string("22222fghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 5, 10, '2', gl_string("2222222222fghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 5, 20, '2', gl_string("22222222222222222222fghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 9, 0, '2', gl_string("j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 9, 5, '2', gl_string("22222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 9, 10, '2', gl_string("2222222222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 9, 20, '2', gl_string("22222222222222222222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 10, 0, '2', gl_string(""));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 10, 5, '2', gl_string("22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 10, 10, '2', gl_string("2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 10, 20, '2', gl_string("22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 11, 0, '2', gl_string(""));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 11, 5, '2', gl_string("22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 11, 10, '2', gl_string("2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 0, 11, 20, '2', gl_string("22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 0, 0, '2', gl_string("abcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 0, 5, '2', gl_string("a22222bcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 0, 10, '2', gl_string("a2222222222bcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 0, 20, '2', gl_string("a22222222222222222222bcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 1, 0, '2', gl_string("acdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 1, 5, '2', gl_string("a22222cdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 1, 10, '2', gl_string("a2222222222cdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 1, 20, '2', gl_string("a22222222222222222222cdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 4, 0, '2', gl_string("afghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 4, 5, '2', gl_string("a22222fghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 4, 10, '2', gl_string("a2222222222fghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 4, 20, '2', gl_string("a22222222222222222222fghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 8, 0, '2', gl_string("aj"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 8, 5, '2', gl_string("a22222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 8, 10, '2', gl_string("a2222222222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 8, 20, '2', gl_string("a22222222222222222222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 9, 0, '2', gl_string("a"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 9, 5, '2', gl_string("a22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 9, 10, '2', gl_string("a2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 9, 20, '2', gl_string("a22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 10, 0, '2', gl_string("a"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 10, 5, '2', gl_string("a22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 10, 10, '2', gl_string("a2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 1, 10, 20, '2', gl_string("a22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 0, 0, '2', gl_string("abcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 0, 5, '2', gl_string("abcde22222fghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 0, 10, '2', gl_string("abcde2222222222fghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 0, 20, '2', gl_string("abcde22222222222222222222fghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 1, 0, '2', gl_string("abcdeghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 1, 5, '2', gl_string("abcde22222ghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 1, 10, '2', gl_string("abcde2222222222ghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 1, 20, '2', gl_string("abcde22222222222222222222ghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 2, 0, '2', gl_string("abcdehij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 2, 5, '2', gl_string("abcde22222hij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 2, 10, '2', gl_string("abcde2222222222hij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 2, 20, '2', gl_string("abcde22222222222222222222hij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 4, 0, '2', gl_string("abcdej"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 4, 5, '2', gl_string("abcde22222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 4, 10, '2', gl_string("abcde2222222222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 4, 20, '2', gl_string("abcde22222222222222222222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 5, 0, '2', gl_string("abcde"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 5, 5, '2', gl_string("abcde22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 5, 10, '2', gl_string("abcde2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 5, 20, '2', gl_string("abcde22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 6, 0, '2', gl_string("abcde"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 6, 5, '2', gl_string("abcde22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 6, 10, '2', gl_string("abcde2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 5, 6, 20, '2', gl_string("abcde22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 0, 0, '2', gl_string("abcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 0, 5, '2', gl_string("abcdefghi22222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 0, 10, '2', gl_string("abcdefghi2222222222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 0, 20, '2', gl_string("abcdefghi22222222222222222222j"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 1, 0, '2', gl_string("abcdefghi"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 1, 5, '2', gl_string("abcdefghi22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 1, 10, '2', gl_string("abcdefghi2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 1, 20, '2', gl_string("abcdefghi22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 2, 0, '2', gl_string("abcdefghi"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 2, 5, '2', gl_string("abcdefghi22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 2, 10, '2', gl_string("abcdefghi2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 9, 2, 20, '2', gl_string("abcdefghi22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 10, 0, 0, '2', gl_string("abcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 10, 0, 5, '2', gl_string("abcdefghij22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 10, 0, 10, '2', gl_string("abcdefghij2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 10, 0, 20, '2', gl_string("abcdefghij22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 10, 1, 0, '2', gl_string("abcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 10, 1, 5, '2', gl_string("abcdefghij22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 10, 1, 10, '2', gl_string("abcdefghij2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 10, 1, 20, '2', gl_string("abcdefghij22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 11, 0, 0, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 11, 0, 5, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 11, 0, 10, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghij"), 11, 0, 20, '2', gl_string("can't happen"));
  }


  void test_string_replace_size_size_size_char2()
  {
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 0, 0, '2', gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 0, 5, '2', gl_string("22222abcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 0, 10, '2', gl_string("2222222222abcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 0, 20, '2', gl_string("22222222222222222222abcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 1, 0, '2', gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 1, 5, '2', gl_string("22222bcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 1, 10, '2', gl_string("2222222222bcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 1, 20, '2', gl_string("22222222222222222222bcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 10, 0, '2', gl_string("klmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 10, 5, '2', gl_string("22222klmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 10, 10, '2', gl_string("2222222222klmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 10, 20, '2', gl_string("22222222222222222222klmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 19, 0, '2', gl_string("t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 19, 5, '2', gl_string("22222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 19, 10, '2', gl_string("2222222222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 19, 20, '2', gl_string("22222222222222222222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 20, 0, '2', gl_string(""));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 20, 5, '2', gl_string("22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 20, 10, '2', gl_string("2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 20, 20, '2', gl_string("22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 21, 0, '2', gl_string(""));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 21, 5, '2', gl_string("22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 21, 10, '2', gl_string("2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 0, 21, 20, '2', gl_string("22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 0, 0, '2', gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 0, 5, '2', gl_string("a22222bcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 0, 10, '2', gl_string("a2222222222bcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 0, 20, '2', gl_string("a22222222222222222222bcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 1, 0, '2', gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 1, 5, '2', gl_string("a22222cdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 1, 10, '2', gl_string("a2222222222cdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 1, 20, '2', gl_string("a22222222222222222222cdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 9, 0, '2', gl_string("aklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 9, 5, '2', gl_string("a22222klmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 9, 10, '2', gl_string("a2222222222klmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 9, 20, '2', gl_string("a22222222222222222222klmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 18, 0, '2', gl_string("at"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 18, 5, '2', gl_string("a22222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 18, 10, '2', gl_string("a2222222222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 18, 20, '2', gl_string("a22222222222222222222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 19, 0, '2', gl_string("a"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 19, 5, '2', gl_string("a22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 19, 10, '2', gl_string("a2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 19, 20, '2', gl_string("a22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 20, 0, '2', gl_string("a"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 20, 5, '2', gl_string("a22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 20, 10, '2', gl_string("a2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 1, 20, 20, '2', gl_string("a22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 0, 0, '2', gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 0, 5, '2', gl_string("abcdefghij22222klmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 0, 10, '2', gl_string("abcdefghij2222222222klmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 0, 20, '2', gl_string("abcdefghij22222222222222222222klmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 1, 0, '2', gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 1, 5, '2', gl_string("abcdefghij22222lmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 1, 10, '2', gl_string("abcdefghij2222222222lmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 1, 20, '2', gl_string("abcdefghij22222222222222222222lmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 5, 0, '2', gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 5, 5, '2', gl_string("abcdefghij22222pqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 5, 10, '2', gl_string("abcdefghij2222222222pqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 5, 20, '2', gl_string("abcdefghij22222222222222222222pqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 9, 0, '2', gl_string("abcdefghijt"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 9, 5, '2', gl_string("abcdefghij22222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 9, 10, '2', gl_string("abcdefghij2222222222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 9, 20, '2', gl_string("abcdefghij22222222222222222222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 10, 0, '2', gl_string("abcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 10, 5, '2', gl_string("abcdefghij22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 10, 10, '2', gl_string("abcdefghij2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 10, 20, '2', gl_string("abcdefghij22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 11, 0, '2', gl_string("abcdefghij"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 11, 5, '2', gl_string("abcdefghij22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 11, 10, '2', gl_string("abcdefghij2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 10, 11, 20, '2', gl_string("abcdefghij22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 0, 0, '2', gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 0, 5, '2', gl_string("abcdefghijklmnopqrs22222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 0, 10, '2', gl_string("abcdefghijklmnopqrs2222222222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 0, 20, '2', gl_string("abcdefghijklmnopqrs22222222222222222222t"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 1, 0, '2', gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 1, 5, '2', gl_string("abcdefghijklmnopqrs22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 1, 10, '2', gl_string("abcdefghijklmnopqrs2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 1, 20, '2', gl_string("abcdefghijklmnopqrs22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 2, 0, '2', gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 2, 5, '2', gl_string("abcdefghijklmnopqrs22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 2, 10, '2', gl_string("abcdefghijklmnopqrs2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 19, 2, 20, '2', gl_string("abcdefghijklmnopqrs22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 0, 0, '2', gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 0, 5, '2', gl_string("abcdefghijklmnopqrst22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 0, 10, '2', gl_string("abcdefghijklmnopqrst2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 0, 20, '2', gl_string("abcdefghijklmnopqrst22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 1, 0, '2', gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 1, 5, '2', gl_string("abcdefghijklmnopqrst22222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 1, 10, '2', gl_string("abcdefghijklmnopqrst2222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 20, 1, 20, '2', gl_string("abcdefghijklmnopqrst22222222222222222222"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 21, 0, 0, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 21, 0, 5, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 21, 0, 10, '2', gl_string("can't happen"));
    _test_string_replace_size_size_size_char(gl_string("abcdefghijklmnopqrst"), 21, 0, 20, '2', gl_string("can't happen"));
  }

  void
  _test_string_replace_size_size_string(gl_string s, size_t pos1, size_t n1, gl_string str, gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.replace(pos1, n1, str);
        
      TS_ASSERT(pos1 <= old_size);
      TS_ASSERT(s == expected);
      size_t xlen = std::min(n1, old_size - pos1);
      size_t rlen = str.size();
      TS_ASSERT(s.size() == old_size - xlen + rlen);
    }
    catch (...)
    {
      TS_ASSERT(pos1 > old_size);
      TS_ASSERT(s == s0);
    }
#endif
  }


  void test_string_replace_size_size_string0()
  {
    _test_string_replace_size_size_string(gl_string(""), 0, 0, gl_string(""), gl_string(""));
    _test_string_replace_size_size_string(gl_string(""), 0, 0, gl_string("12345"), gl_string("12345"));
    _test_string_replace_size_size_string(gl_string(""), 0, 0, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_size_size_string(gl_string(""), 0, 0, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_size_size_string(gl_string(""), 0, 1, gl_string(""), gl_string(""));
    _test_string_replace_size_size_string(gl_string(""), 0, 1, gl_string("12345"), gl_string("12345"));
    _test_string_replace_size_size_string(gl_string(""), 0, 1, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_size_size_string(gl_string(""), 0, 1, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_size_size_string(gl_string(""), 1, 0, gl_string(""), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string(""), 1, 0, gl_string("12345"), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string(""), 1, 0, gl_string("1234567890"), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string(""), 1, 0, gl_string("12345678901234567890"), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 0, gl_string(""), gl_string("abcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 0, gl_string("12345"), gl_string("12345abcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 0, gl_string("1234567890"), gl_string("1234567890abcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), gl_string("12345678901234567890abcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 1, gl_string(""), gl_string("bcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 1, gl_string("12345"), gl_string("12345bcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 1, gl_string("1234567890"), gl_string("1234567890bcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), gl_string("12345678901234567890bcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 2, gl_string(""), gl_string("cde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 2, gl_string("12345"), gl_string("12345cde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 2, gl_string("1234567890"), gl_string("1234567890cde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), gl_string("12345678901234567890cde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 4, gl_string(""), gl_string("e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 4, gl_string("12345"), gl_string("12345e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 4, gl_string("1234567890"), gl_string("1234567890e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), gl_string("12345678901234567890e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 5, gl_string(""), gl_string(""));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 5, gl_string("12345"), gl_string("12345"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 5, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 6, gl_string(""), gl_string(""));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 6, gl_string("12345"), gl_string("12345"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 6, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 0, gl_string(""), gl_string("abcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 0, gl_string("12345"), gl_string("a12345bcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 0, gl_string("1234567890"), gl_string("a1234567890bcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), gl_string("a12345678901234567890bcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 1, gl_string(""), gl_string("acde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 1, gl_string("12345"), gl_string("a12345cde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 1, gl_string("1234567890"), gl_string("a1234567890cde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), gl_string("a12345678901234567890cde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 2, gl_string(""), gl_string("ade"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 2, gl_string("12345"), gl_string("a12345de"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 2, gl_string("1234567890"), gl_string("a1234567890de"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), gl_string("a12345678901234567890de"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 3, gl_string(""), gl_string("ae"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 3, gl_string("12345"), gl_string("a12345e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 3, gl_string("1234567890"), gl_string("a1234567890e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), gl_string("a12345678901234567890e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 4, gl_string(""), gl_string("a"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 4, gl_string("12345"), gl_string("a12345"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 4, gl_string("1234567890"), gl_string("a1234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 5, gl_string(""), gl_string("a"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 5, gl_string("12345"), gl_string("a12345"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 5, gl_string("1234567890"), gl_string("a1234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 0, gl_string(""), gl_string("abcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 0, gl_string("12345"), gl_string("ab12345cde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 0, gl_string("1234567890"), gl_string("ab1234567890cde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), gl_string("ab12345678901234567890cde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 1, gl_string(""), gl_string("abde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 1, gl_string("12345"), gl_string("ab12345de"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 1, gl_string("1234567890"), gl_string("ab1234567890de"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), gl_string("ab12345678901234567890de"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 2, gl_string(""), gl_string("abe"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 2, gl_string("12345"), gl_string("ab12345e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 2, gl_string("1234567890"), gl_string("ab1234567890e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), gl_string("ab12345678901234567890e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 3, gl_string(""), gl_string("ab"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 3, gl_string("12345"), gl_string("ab12345"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 3, gl_string("1234567890"), gl_string("ab1234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), gl_string("ab12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 4, gl_string(""), gl_string("ab"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 4, gl_string("12345"), gl_string("ab12345"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 4, gl_string("1234567890"), gl_string("ab1234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), gl_string("ab12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 0, gl_string(""), gl_string("abcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 0, gl_string("12345"), gl_string("abcd12345e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 0, gl_string("1234567890"), gl_string("abcd1234567890e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), gl_string("abcd12345678901234567890e"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 1, gl_string(""), gl_string("abcd"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 1, gl_string("12345"), gl_string("abcd12345"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 1, gl_string("1234567890"), gl_string("abcd1234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), gl_string("abcd12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 2, gl_string(""), gl_string("abcd"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 2, gl_string("12345"), gl_string("abcd12345"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 2, gl_string("1234567890"), gl_string("abcd1234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), gl_string("abcd12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 5, 0, gl_string(""), gl_string("abcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 5, 0, gl_string("12345"), gl_string("abcde12345"));
    _test_string_replace_size_size_string(gl_string("abcde"), 5, 0, gl_string("1234567890"), gl_string("abcde1234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 5, 1, gl_string(""), gl_string("abcde"));
    _test_string_replace_size_size_string(gl_string("abcde"), 5, 1, gl_string("12345"), gl_string("abcde12345"));
    _test_string_replace_size_size_string(gl_string("abcde"), 5, 1, gl_string("1234567890"), gl_string("abcde1234567890"));
    _test_string_replace_size_size_string(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890"));
  }


  void test_string_replace_size_size_string1()
  {
    _test_string_replace_size_size_string(gl_string("abcde"), 6, 0, gl_string(""), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string("abcde"), 6, 0, gl_string("12345"), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string("abcde"), 6, 0, gl_string("1234567890"), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 0, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 0, gl_string("12345"), gl_string("12345abcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), gl_string("1234567890abcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), gl_string("12345678901234567890abcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 1, gl_string(""), gl_string("bcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 1, gl_string("12345"), gl_string("12345bcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), gl_string("1234567890bcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), gl_string("12345678901234567890bcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 5, gl_string(""), gl_string("fghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 5, gl_string("12345"), gl_string("12345fghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), gl_string("1234567890fghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), gl_string("12345678901234567890fghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 9, gl_string(""), gl_string("j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 9, gl_string("12345"), gl_string("12345j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), gl_string("1234567890j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), gl_string("12345678901234567890j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 10, gl_string(""), gl_string(""));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 10, gl_string("12345"), gl_string("12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 11, gl_string(""), gl_string(""));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 11, gl_string("12345"), gl_string("12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 0, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 0, gl_string("12345"), gl_string("a12345bcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), gl_string("a1234567890bcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), gl_string("a12345678901234567890bcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 1, gl_string(""), gl_string("acdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 1, gl_string("12345"), gl_string("a12345cdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), gl_string("a1234567890cdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), gl_string("a12345678901234567890cdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 4, gl_string(""), gl_string("afghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 4, gl_string("12345"), gl_string("a12345fghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), gl_string("a1234567890fghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), gl_string("a12345678901234567890fghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 8, gl_string(""), gl_string("aj"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 8, gl_string("12345"), gl_string("a12345j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), gl_string("a1234567890j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), gl_string("a12345678901234567890j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 9, gl_string(""), gl_string("a"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 9, gl_string("12345"), gl_string("a12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), gl_string("a1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 10, gl_string(""), gl_string("a"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 10, gl_string("12345"), gl_string("a12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), gl_string("a1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 0, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 0, gl_string("12345"), gl_string("abcde12345fghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), gl_string("abcde1234567890fghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890fghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 1, gl_string(""), gl_string("abcdeghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 1, gl_string("12345"), gl_string("abcde12345ghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), gl_string("abcde1234567890ghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890ghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 2, gl_string(""), gl_string("abcdehij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 2, gl_string("12345"), gl_string("abcde12345hij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), gl_string("abcde1234567890hij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890hij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 4, gl_string(""), gl_string("abcdej"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 4, gl_string("12345"), gl_string("abcde12345j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), gl_string("abcde1234567890j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 5, gl_string(""), gl_string("abcde"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 5, gl_string("12345"), gl_string("abcde12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), gl_string("abcde1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 6, gl_string(""), gl_string("abcde"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 6, gl_string("12345"), gl_string("abcde12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), gl_string("abcde1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 0, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 0, gl_string("12345"), gl_string("abcdefghi12345j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), gl_string("abcdefghi1234567890j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), gl_string("abcdefghi12345678901234567890j"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 1, gl_string(""), gl_string("abcdefghi"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 1, gl_string("12345"), gl_string("abcdefghi12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 2, gl_string(""), gl_string("abcdefghi"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 2, gl_string("12345"), gl_string("abcdefghi12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 10, 0, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 10, 0, gl_string("12345"), gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 10, 1, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 10, 1, gl_string("12345"), gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 11, 0, gl_string(""), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 11, 0, gl_string("12345"), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), gl_string("can't happen"));
  }


  void test_string_replace_size_size_string2()
  {
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), gl_string("12345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string(""), gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), gl_string("12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), gl_string("12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string(""), gl_string("klmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), gl_string("12345klmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), gl_string("1234567890klmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), gl_string("12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string(""), gl_string("t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), gl_string("12345t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), gl_string("1234567890t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), gl_string("12345678901234567890t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string(""), gl_string(""));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), gl_string("12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string(""), gl_string(""));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), gl_string("12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), gl_string("1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), gl_string("12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), gl_string("a12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string(""), gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), gl_string("a12345cdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), gl_string("a12345678901234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string(""), gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), gl_string("a12345klmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), gl_string("a1234567890klmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), gl_string("a12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string(""), gl_string("at"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), gl_string("a12345t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), gl_string("a1234567890t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), gl_string("a12345678901234567890t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string(""), gl_string("a"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), gl_string("a12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), gl_string("a1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string(""), gl_string("a"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), gl_string("a12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), gl_string("a1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), gl_string("abcdefghij12345klmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string(""), gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), gl_string("abcdefghij12345lmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890lmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string(""), gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), gl_string("abcdefghij12345pqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890pqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string(""), gl_string("abcdefghijt"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), gl_string("abcdefghij12345t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), gl_string("abcdefghij1234567890t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string(""), gl_string("abcdefghij"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string(""), gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string(""), gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string(""), gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string(""), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), gl_string("can't happen"));
    _test_string_replace_size_size_string(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), gl_string("can't happen"));
  }

  void
  _test_string_replace_size_size_string_size_size(gl_string s, size_t pos1, size_t n1,
                                                  gl_string str, size_t pos2, size_t n2,
                                                  gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.replace(pos1, n1, str, pos2, n2);
        
      TS_ASSERT(pos1 <= old_size && pos2 <= str.size());
      TS_ASSERT(s == expected);
      size_t xlen = std::min(n1, old_size - pos1);
      size_t rlen = std::min(n2, str.size() - pos2);
      TS_ASSERT(s.size() == old_size - xlen + rlen);
    }
    catch (...)
    {
      TS_ASSERT(pos1 > old_size || pos2 > str.size());
      TS_ASSERT(s == s0);
    }
#endif
  }


  void
  test_string_replace_size_size_string_size_size_npos(gl_string s, size_t pos1, size_t n1,
                                                      gl_string str, size_t pos2,
                                                      gl_string expected)
  {
#ifndef NDEBUG
    size_t old_size = s.size();
    gl_string s0 = s;
    try
    {
      s.replace(pos1, n1, str, pos2);
        
      TS_ASSERT(pos1 <= old_size && pos2 <= str.size());
      TS_ASSERT(s == expected);
      size_t xlen = std::min(n1, old_size - pos1);
      size_t npos = gl_string::npos;
      size_t rlen = std::min(npos, str.size() - pos2);
      TS_ASSERT(s.size() == old_size - xlen + rlen);
    }
    catch (...)
    {
      TS_ASSERT(pos1 > old_size || pos2 > str.size());
      TS_ASSERT(s == s0);
    }
#endif
  }



  void test_string_replace_size_size_string_size_size0()
  {
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string(""), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string(""), 0, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 0, 2, gl_string("12"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 0, 4, gl_string("1234"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 0, 6, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 1, 2, gl_string("23"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 1, 3, gl_string("234"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 1, 5, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 2, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 2, 1, gl_string("3"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 2, 2, gl_string("34"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 2, 3, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 2, 4, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 4, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 4, 1, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 4, 2, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 5, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 0, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 0, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 1, 8, gl_string("23456789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 1, 10, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 5, 1, gl_string("6"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 5, 2, gl_string("67"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 5, 4, gl_string("6789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 5, 5, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 5, 6, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 9, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 9, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 9, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 10, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 10, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 10, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 10, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 19, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 19, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 19, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 20, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 20, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string(""), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string(""), 0, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 0, 2, gl_string("12"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 0, 4, gl_string("1234"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 0, 6, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 1, 2, gl_string("23"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 1, 3, gl_string("234"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 1, 5, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 2, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 2, 1, gl_string("3"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 2, 2, gl_string("34"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 2, 3, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 2, 4, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 4, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 4, 1, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 4, 2, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 5, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
  }


  void test_string_replace_size_size_string_size_size1()
  {
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 0, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 0, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 1, 8, gl_string("23456789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 1, 10, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 5, 1, gl_string("6"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 5, 2, gl_string("67"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 5, 4, gl_string("6789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 5, 5, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 5, 6, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 9, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 9, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 9, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 10, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 10, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 10, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 10, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 19, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 19, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 19, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 20, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 20, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string(""), 0, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string(""), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string(""), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 0, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 0, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 0, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 0, 6, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 1, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 1, 3, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 1, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 1, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 2, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 2, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 2, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 2, 3, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 2, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 4, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 4, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 4, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 5, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 5, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 0, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 0, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 0, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 0, 11, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 1, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 1, 8, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 1, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 1, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 5, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 5, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 5, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 5, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 5, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 5, 6, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 9, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 9, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 9, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 10, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 10, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 0, 1, gl_string("can't happen"));
  }


  void test_string_replace_size_size_string_size_size2()
  {
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 0, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 0, 19, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 0, 20, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 0, 21, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 1, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 1, 18, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 1, 19, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 1, 20, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 10, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 10, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 10, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 10, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 10, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 10, 11, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 19, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 19, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 19, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 20, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 20, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string(""), 1, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 0, 1, gl_string("1abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 0, 2, gl_string("12abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 0, 4, gl_string("1234abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 0, 5, gl_string("12345abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 0, 6, gl_string("12345abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 1, 1, gl_string("2abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 1, 2, gl_string("23abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 1, 3, gl_string("234abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 1, 4, gl_string("2345abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 1, 5, gl_string("2345abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 2, 1, gl_string("3abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 2, 2, gl_string("34abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 2, 3, gl_string("345abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 2, 4, gl_string("345abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 4, 1, gl_string("5abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 4, 2, gl_string("5abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 0, 1, gl_string("1abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 0, 5, gl_string("12345abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 0, 9, gl_string("123456789abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 0, 10, gl_string("1234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 0, 11, gl_string("1234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 1, 1, gl_string("2abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 1, 4, gl_string("2345abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 1, 8, gl_string("23456789abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 1, 9, gl_string("234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 1, 10, gl_string("234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 5, 1, gl_string("6abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 5, 2, gl_string("67abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 5, 4, gl_string("6789abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 5, 5, gl_string("67890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 5, 6, gl_string("67890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 9, 1, gl_string("0abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 9, 2, gl_string("0abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 0, 1, gl_string("1abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 1, 1, gl_string("2abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 1, 9, gl_string("234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 10, 1, gl_string("1abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 10, 5, gl_string("12345abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 10, 9, gl_string("123456789abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 19, 1, gl_string("0abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 19, 2, gl_string("0abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string(""), 0, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string(""), 0, 1, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 0, 0, gl_string("bcde"));
  }


  void test_string_replace_size_size_string_size_size3()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 0, 1, gl_string("1bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 0, 2, gl_string("12bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 0, 4, gl_string("1234bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 0, 5, gl_string("12345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 0, 6, gl_string("12345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 1, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 1, 1, gl_string("2bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 1, 2, gl_string("23bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 1, 3, gl_string("234bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 1, 4, gl_string("2345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 1, 5, gl_string("2345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 2, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 2, 1, gl_string("3bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 2, 2, gl_string("34bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 2, 3, gl_string("345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 2, 4, gl_string("345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 4, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 4, 1, gl_string("5bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 4, 2, gl_string("5bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 5, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 5, 1, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 0, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 0, 1, gl_string("1bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 0, 5, gl_string("12345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 0, 9, gl_string("123456789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 0, 10, gl_string("1234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 0, 11, gl_string("1234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 1, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 1, 1, gl_string("2bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 1, 4, gl_string("2345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 1, 8, gl_string("23456789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 1, 9, gl_string("234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 1, 10, gl_string("234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 5, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 5, 1, gl_string("6bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 5, 2, gl_string("67bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 5, 4, gl_string("6789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 5, 5, gl_string("67890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 5, 6, gl_string("67890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 9, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 9, 1, gl_string("0bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 9, 2, gl_string("0bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 10, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 10, 1, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 0, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 0, 1, gl_string("1bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 1, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 1, 1, gl_string("2bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 1, 9, gl_string("234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 10, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 10, 1, gl_string("1bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 10, 5, gl_string("12345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 10, 9, gl_string("123456789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 19, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 19, 1, gl_string("0bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 19, 2, gl_string("0bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 20, 0, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 20, 1, gl_string("bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string(""), 0, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string(""), 0, 1, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 0, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 0, 1, gl_string("1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 0, 2, gl_string("12cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 0, 4, gl_string("1234cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 0, 5, gl_string("12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 0, 6, gl_string("12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 1, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 1, 1, gl_string("2cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 1, 2, gl_string("23cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 1, 3, gl_string("234cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 1, 4, gl_string("2345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 1, 5, gl_string("2345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 2, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 2, 1, gl_string("3cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 2, 2, gl_string("34cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 2, 3, gl_string("345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 2, 4, gl_string("345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 4, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 4, 1, gl_string("5cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 4, 2, gl_string("5cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 5, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 5, 1, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 0, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 0, 1, gl_string("1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 0, 5, gl_string("12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 0, 9, gl_string("123456789cde"));
  }


  void test_string_replace_size_size_string_size_size4()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 0, 10, gl_string("1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 0, 11, gl_string("1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 1, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 1, 1, gl_string("2cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 1, 4, gl_string("2345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 1, 8, gl_string("23456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 1, 9, gl_string("234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 1, 10, gl_string("234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 5, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 5, 1, gl_string("6cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 5, 2, gl_string("67cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 5, 4, gl_string("6789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 5, 5, gl_string("67890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 5, 6, gl_string("67890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 9, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 9, 1, gl_string("0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 9, 2, gl_string("0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 10, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 10, 1, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 0, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 0, 1, gl_string("1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 1, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 1, 1, gl_string("2cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 1, 9, gl_string("234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 10, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 10, 1, gl_string("1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 10, 5, gl_string("12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 10, 9, gl_string("123456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 19, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 19, 1, gl_string("0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 19, 2, gl_string("0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 20, 0, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 20, 1, gl_string("cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 2, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string(""), 0, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string(""), 0, 1, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 0, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 0, 1, gl_string("1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 0, 2, gl_string("12e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 0, 4, gl_string("1234e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 0, 5, gl_string("12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 0, 6, gl_string("12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 1, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 1, 1, gl_string("2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 1, 2, gl_string("23e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 1, 3, gl_string("234e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 1, 4, gl_string("2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 1, 5, gl_string("2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 2, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 2, 1, gl_string("3e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 2, 2, gl_string("34e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 2, 3, gl_string("345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 2, 4, gl_string("345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 4, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 4, 1, gl_string("5e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 4, 2, gl_string("5e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 5, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 5, 1, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 0, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 0, 1, gl_string("1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 0, 5, gl_string("12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 0, 9, gl_string("123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 0, 10, gl_string("1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 0, 11, gl_string("1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 1, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 1, 1, gl_string("2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 1, 4, gl_string("2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 1, 8, gl_string("23456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 1, 9, gl_string("234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 1, 10, gl_string("234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 5, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 5, 1, gl_string("6e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 5, 2, gl_string("67e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 5, 4, gl_string("6789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 5, 5, gl_string("67890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 5, 6, gl_string("67890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 9, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 9, 1, gl_string("0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 9, 2, gl_string("0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 10, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 10, 1, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 0, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 0, 1, gl_string("1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890e"));
  }


  void test_string_replace_size_size_string_size_size5()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 1, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 1, 1, gl_string("2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 1, 9, gl_string("234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 10, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 10, 1, gl_string("1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 10, 5, gl_string("12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 10, 9, gl_string("123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 19, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 19, 1, gl_string("0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 19, 2, gl_string("0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 20, 0, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 20, 1, gl_string("e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 4, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string(""), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string(""), 0, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 0, 2, gl_string("12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 0, 4, gl_string("1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 0, 6, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 1, 2, gl_string("23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 1, 3, gl_string("234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 1, 5, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 2, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 2, 1, gl_string("3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 2, 2, gl_string("34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 2, 3, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 2, 4, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 4, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 4, 1, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 4, 2, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 5, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 0, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 0, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 1, 8, gl_string("23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 1, 10, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 5, 1, gl_string("6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 5, 2, gl_string("67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 5, 4, gl_string("6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 5, 5, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 5, 6, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 9, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 9, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 9, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 10, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 10, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 10, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 10, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 19, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 19, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 19, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 20, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 20, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 5, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string(""), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string(""), 0, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 0, 2, gl_string("12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 0, 4, gl_string("1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 0, 5, gl_string("12345"));
  }


  void test_string_replace_size_size_string_size_size6()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 0, 6, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 1, 2, gl_string("23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 1, 3, gl_string("234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 1, 5, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 2, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 2, 1, gl_string("3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 2, 2, gl_string("34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 2, 3, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 2, 4, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 4, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 4, 1, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 4, 2, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 5, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 0, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 0, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 1, 8, gl_string("23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 1, 10, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 5, 1, gl_string("6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 5, 2, gl_string("67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 5, 4, gl_string("6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 5, 5, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 5, 6, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 9, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 9, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 9, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 10, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 10, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 10, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 10, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 19, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 19, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 19, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 20, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 20, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 0, 6, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 0, 1, gl_string("a1bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 0, 2, gl_string("a12bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 0, 4, gl_string("a1234bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 0, 5, gl_string("a12345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 0, 6, gl_string("a12345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 1, 1, gl_string("a2bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 1, 2, gl_string("a23bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 1, 3, gl_string("a234bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 1, 4, gl_string("a2345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 1, 5, gl_string("a2345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 2, 1, gl_string("a3bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 2, 2, gl_string("a34bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 2, 3, gl_string("a345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 2, 4, gl_string("a345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 4, 1, gl_string("a5bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 4, 2, gl_string("a5bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 0, 1, gl_string("a1bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 0, 5, gl_string("a12345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 0, 9, gl_string("a123456789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 0, 10, gl_string("a1234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 0, 11, gl_string("a1234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 1, 1, gl_string("a2bcde"));
  }


  void test_string_replace_size_size_string_size_size7()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 1, 4, gl_string("a2345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 1, 8, gl_string("a23456789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 1, 9, gl_string("a234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 1, 10, gl_string("a234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 5, 1, gl_string("a6bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 5, 2, gl_string("a67bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 5, 4, gl_string("a6789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 5, 5, gl_string("a67890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 5, 6, gl_string("a67890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 9, 1, gl_string("a0bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 9, 2, gl_string("a0bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 0, 1, gl_string("a1bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 1, 1, gl_string("a2bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 10, 1, gl_string("a1bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 10, 5, gl_string("a12345bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 19, 1, gl_string("a0bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 19, 2, gl_string("a0bcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string(""), 0, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string(""), 0, 1, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 0, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 0, 1, gl_string("a1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 0, 2, gl_string("a12cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 0, 4, gl_string("a1234cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 0, 5, gl_string("a12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 0, 6, gl_string("a12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 1, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 1, 1, gl_string("a2cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 1, 2, gl_string("a23cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 1, 3, gl_string("a234cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 1, 4, gl_string("a2345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 1, 5, gl_string("a2345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 2, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 2, 1, gl_string("a3cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 2, 2, gl_string("a34cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 2, 3, gl_string("a345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 2, 4, gl_string("a345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 4, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 4, 1, gl_string("a5cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 4, 2, gl_string("a5cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 5, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 5, 1, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 0, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 0, 1, gl_string("a1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 0, 5, gl_string("a12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 0, 9, gl_string("a123456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 0, 10, gl_string("a1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 0, 11, gl_string("a1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 1, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 1, 1, gl_string("a2cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 1, 4, gl_string("a2345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 1, 8, gl_string("a23456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 1, 9, gl_string("a234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 1, 10, gl_string("a234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 5, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 5, 1, gl_string("a6cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 5, 2, gl_string("a67cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 5, 4, gl_string("a6789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 5, 5, gl_string("a67890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 5, 6, gl_string("a67890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 9, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 9, 1, gl_string("a0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 9, 2, gl_string("a0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 10, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 10, 1, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 0, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 0, 1, gl_string("a1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 1, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 1, 1, gl_string("a2cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789cde"));
  }


  void test_string_replace_size_size_string_size_size8()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 10, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 10, 1, gl_string("a1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 10, 5, gl_string("a12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 19, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 19, 1, gl_string("a0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 19, 2, gl_string("a0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 20, 0, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 20, 1, gl_string("acde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string(""), 0, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string(""), 0, 1, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 0, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 0, 1, gl_string("a1de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 0, 2, gl_string("a12de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 0, 4, gl_string("a1234de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 0, 5, gl_string("a12345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 0, 6, gl_string("a12345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 1, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 1, 1, gl_string("a2de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 1, 2, gl_string("a23de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 1, 3, gl_string("a234de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 1, 4, gl_string("a2345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 1, 5, gl_string("a2345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 2, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 2, 1, gl_string("a3de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 2, 2, gl_string("a34de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 2, 3, gl_string("a345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 2, 4, gl_string("a345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 4, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 4, 1, gl_string("a5de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 4, 2, gl_string("a5de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 5, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 5, 1, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 0, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 0, 1, gl_string("a1de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 0, 5, gl_string("a12345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 0, 9, gl_string("a123456789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 0, 10, gl_string("a1234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 0, 11, gl_string("a1234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 1, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 1, 1, gl_string("a2de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 1, 4, gl_string("a2345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 1, 8, gl_string("a23456789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 1, 9, gl_string("a234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 1, 10, gl_string("a234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 5, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 5, 1, gl_string("a6de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 5, 2, gl_string("a67de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 5, 4, gl_string("a6789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 5, 5, gl_string("a67890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 5, 6, gl_string("a67890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 9, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 9, 1, gl_string("a0de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 9, 2, gl_string("a0de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 10, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 10, 1, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 0, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 0, 1, gl_string("a1de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 1, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 1, 1, gl_string("a2de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 10, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 10, 1, gl_string("a1de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 10, 5, gl_string("a12345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 19, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 19, 1, gl_string("a0de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 19, 2, gl_string("a0de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 20, 0, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 20, 1, gl_string("ade"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 2, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string(""), 0, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string(""), 0, 1, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 0, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 0, 1, gl_string("a1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 0, 2, gl_string("a12e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 0, 4, gl_string("a1234e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 0, 5, gl_string("a12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 0, 6, gl_string("a12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 1, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 1, 1, gl_string("a2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 1, 2, gl_string("a23e"));
  }


  void test_string_replace_size_size_string_size_size9()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 1, 3, gl_string("a234e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 1, 4, gl_string("a2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 1, 5, gl_string("a2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 2, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 2, 1, gl_string("a3e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 2, 2, gl_string("a34e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 2, 3, gl_string("a345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 2, 4, gl_string("a345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 4, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 4, 1, gl_string("a5e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 4, 2, gl_string("a5e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 5, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 5, 1, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 0, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 0, 1, gl_string("a1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 0, 5, gl_string("a12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 0, 9, gl_string("a123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 0, 10, gl_string("a1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 0, 11, gl_string("a1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 1, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 1, 1, gl_string("a2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 1, 4, gl_string("a2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 1, 8, gl_string("a23456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 1, 9, gl_string("a234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 1, 10, gl_string("a234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 5, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 5, 1, gl_string("a6e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 5, 2, gl_string("a67e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 5, 4, gl_string("a6789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 5, 5, gl_string("a67890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 5, 6, gl_string("a67890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 9, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 9, 1, gl_string("a0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 9, 2, gl_string("a0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 10, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 10, 1, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 0, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 0, 1, gl_string("a1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 1, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 1, 1, gl_string("a2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 10, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 10, 1, gl_string("a1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 10, 5, gl_string("a12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 19, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 19, 1, gl_string("a0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 19, 2, gl_string("a0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 20, 0, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 20, 1, gl_string("ae"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 3, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string(""), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string(""), 0, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 0, 2, gl_string("a12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 0, 4, gl_string("a1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 0, 6, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 1, 2, gl_string("a23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 1, 3, gl_string("a234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 1, 5, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 2, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 2, 1, gl_string("a3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 2, 2, gl_string("a34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 2, 3, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 2, 4, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 4, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 4, 1, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 4, 2, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 5, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 0, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 0, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 1, 8, gl_string("a23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 1, 10, gl_string("a234567890"));
  }


  void test_string_replace_size_size_string_size_size10()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 5, 1, gl_string("a6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 5, 2, gl_string("a67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 5, 4, gl_string("a6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 5, 5, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 5, 6, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 9, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 9, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 9, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 10, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 10, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 10, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 19, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 19, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 19, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 20, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 20, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 4, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string(""), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string(""), 0, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 0, 2, gl_string("a12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 0, 4, gl_string("a1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 0, 6, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 1, 2, gl_string("a23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 1, 3, gl_string("a234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 1, 5, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 2, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 2, 1, gl_string("a3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 2, 2, gl_string("a34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 2, 3, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 2, 4, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 4, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 4, 1, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 4, 2, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 5, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 0, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 0, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 1, 8, gl_string("a23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 1, 10, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 5, 1, gl_string("a6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 5, 2, gl_string("a67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 5, 4, gl_string("a6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 5, 5, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 5, 6, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 9, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 9, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 9, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 10, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 10, 1, gl_string("a1"));
  }


  void test_string_replace_size_size_string_size_size11()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 10, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 19, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 19, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 19, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 20, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 20, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 1, 5, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 0, 1, gl_string("ab1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 0, 2, gl_string("ab12cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 0, 4, gl_string("ab1234cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 0, 5, gl_string("ab12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 0, 6, gl_string("ab12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 1, 1, gl_string("ab2cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 1, 2, gl_string("ab23cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 1, 3, gl_string("ab234cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 1, 4, gl_string("ab2345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 1, 5, gl_string("ab2345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 2, 1, gl_string("ab3cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 2, 2, gl_string("ab34cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 2, 3, gl_string("ab345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 2, 4, gl_string("ab345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 4, 1, gl_string("ab5cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 4, 2, gl_string("ab5cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 0, 1, gl_string("ab1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 0, 5, gl_string("ab12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 0, 9, gl_string("ab123456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 0, 10, gl_string("ab1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 0, 11, gl_string("ab1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 1, 1, gl_string("ab2cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 1, 4, gl_string("ab2345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 1, 8, gl_string("ab23456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 1, 9, gl_string("ab234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 1, 10, gl_string("ab234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 5, 1, gl_string("ab6cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 5, 2, gl_string("ab67cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 5, 4, gl_string("ab6789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 5, 5, gl_string("ab67890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 5, 6, gl_string("ab67890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 9, 1, gl_string("ab0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 9, 2, gl_string("ab0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 0, 1, gl_string("ab1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 0, 10, gl_string("ab1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 0, 19, gl_string("ab1234567890123456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 0, 20, gl_string("ab12345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 0, 21, gl_string("ab12345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 1, 1, gl_string("ab2cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 1, 9, gl_string("ab234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 1, 18, gl_string("ab234567890123456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 1, 19, gl_string("ab2345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 1, 20, gl_string("ab2345678901234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 10, 1, gl_string("ab1cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 10, 5, gl_string("ab12345cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 10, 9, gl_string("ab123456789cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 10, 10, gl_string("ab1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 10, 11, gl_string("ab1234567890cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 19, 1, gl_string("ab0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 19, 2, gl_string("ab0cde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string(""), 0, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string(""), 0, 1, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 0, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 0, 1, gl_string("ab1de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 0, 2, gl_string("ab12de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 0, 4, gl_string("ab1234de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 0, 5, gl_string("ab12345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 0, 6, gl_string("ab12345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 1, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 1, 1, gl_string("ab2de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 1, 2, gl_string("ab23de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 1, 3, gl_string("ab234de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 1, 4, gl_string("ab2345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 1, 5, gl_string("ab2345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 2, 0, gl_string("abde"));
  }


  void test_string_replace_size_size_string_size_size12()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 2, 1, gl_string("ab3de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 2, 2, gl_string("ab34de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 2, 3, gl_string("ab345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 2, 4, gl_string("ab345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 4, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 4, 1, gl_string("ab5de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 4, 2, gl_string("ab5de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 5, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 5, 1, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 0, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 0, 1, gl_string("ab1de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 0, 5, gl_string("ab12345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 0, 9, gl_string("ab123456789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 0, 10, gl_string("ab1234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 0, 11, gl_string("ab1234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 1, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 1, 1, gl_string("ab2de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 1, 4, gl_string("ab2345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 1, 8, gl_string("ab23456789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 1, 9, gl_string("ab234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 1, 10, gl_string("ab234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 5, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 5, 1, gl_string("ab6de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 5, 2, gl_string("ab67de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 5, 4, gl_string("ab6789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 5, 5, gl_string("ab67890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 5, 6, gl_string("ab67890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 9, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 9, 1, gl_string("ab0de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 9, 2, gl_string("ab0de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 10, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 10, 1, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 0, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 0, 1, gl_string("ab1de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 0, 10, gl_string("ab1234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 0, 19, gl_string("ab1234567890123456789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 0, 20, gl_string("ab12345678901234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 0, 21, gl_string("ab12345678901234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 1, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 1, 1, gl_string("ab2de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 1, 9, gl_string("ab234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 1, 18, gl_string("ab234567890123456789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 1, 19, gl_string("ab2345678901234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 1, 20, gl_string("ab2345678901234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 10, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 10, 1, gl_string("ab1de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 10, 5, gl_string("ab12345de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 10, 9, gl_string("ab123456789de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 10, 10, gl_string("ab1234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 10, 11, gl_string("ab1234567890de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 19, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 19, 1, gl_string("ab0de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 19, 2, gl_string("ab0de"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 20, 0, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 20, 1, gl_string("abde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string(""), 0, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string(""), 0, 1, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 0, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 0, 1, gl_string("ab1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 0, 2, gl_string("ab12e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 0, 4, gl_string("ab1234e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 0, 5, gl_string("ab12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 0, 6, gl_string("ab12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 1, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 1, 1, gl_string("ab2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 1, 2, gl_string("ab23e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 1, 3, gl_string("ab234e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 1, 4, gl_string("ab2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 1, 5, gl_string("ab2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 2, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 2, 1, gl_string("ab3e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 2, 2, gl_string("ab34e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 2, 3, gl_string("ab345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 2, 4, gl_string("ab345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 4, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 4, 1, gl_string("ab5e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 4, 2, gl_string("ab5e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 5, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 5, 1, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 0, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 0, 1, gl_string("ab1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 0, 5, gl_string("ab12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 0, 9, gl_string("ab123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 0, 10, gl_string("ab1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 0, 11, gl_string("ab1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 1, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 1, 1, gl_string("ab2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 1, 4, gl_string("ab2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 1, 8, gl_string("ab23456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 1, 9, gl_string("ab234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 1, 10, gl_string("ab234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 5, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 5, 1, gl_string("ab6e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 5, 2, gl_string("ab67e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 5, 4, gl_string("ab6789e"));
  }


  void test_string_replace_size_size_string_size_size13()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 5, 5, gl_string("ab67890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 5, 6, gl_string("ab67890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 9, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 9, 1, gl_string("ab0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 9, 2, gl_string("ab0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 10, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 10, 1, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 0, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 0, 1, gl_string("ab1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 0, 10, gl_string("ab1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 0, 19, gl_string("ab1234567890123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 0, 20, gl_string("ab12345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 0, 21, gl_string("ab12345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 1, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 1, 1, gl_string("ab2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 1, 9, gl_string("ab234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 1, 18, gl_string("ab234567890123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 1, 19, gl_string("ab2345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 1, 20, gl_string("ab2345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 10, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 10, 1, gl_string("ab1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 10, 5, gl_string("ab12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 10, 9, gl_string("ab123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 10, 10, gl_string("ab1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 10, 11, gl_string("ab1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 19, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 19, 1, gl_string("ab0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 19, 2, gl_string("ab0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 20, 0, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 20, 1, gl_string("abe"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 2, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string(""), 0, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string(""), 0, 1, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 0, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 0, 1, gl_string("ab1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 0, 2, gl_string("ab12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 0, 4, gl_string("ab1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 0, 5, gl_string("ab12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 0, 6, gl_string("ab12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 1, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 1, 1, gl_string("ab2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 1, 2, gl_string("ab23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 1, 3, gl_string("ab234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 1, 4, gl_string("ab2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 1, 5, gl_string("ab2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 2, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 2, 1, gl_string("ab3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 2, 2, gl_string("ab34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 2, 3, gl_string("ab345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 2, 4, gl_string("ab345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 4, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 4, 1, gl_string("ab5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 4, 2, gl_string("ab5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 5, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 5, 1, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 0, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 0, 1, gl_string("ab1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 0, 5, gl_string("ab12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 0, 9, gl_string("ab123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 0, 10, gl_string("ab1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 0, 11, gl_string("ab1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 1, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 1, 1, gl_string("ab2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 1, 4, gl_string("ab2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 1, 8, gl_string("ab23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 1, 9, gl_string("ab234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 1, 10, gl_string("ab234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 5, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 5, 1, gl_string("ab6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 5, 2, gl_string("ab67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 5, 4, gl_string("ab6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 5, 5, gl_string("ab67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 5, 6, gl_string("ab67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 9, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 9, 1, gl_string("ab0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 9, 2, gl_string("ab0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 10, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 10, 1, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 0, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 0, 1, gl_string("ab1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 0, 10, gl_string("ab1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 0, 19, gl_string("ab1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 0, 20, gl_string("ab12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 0, 21, gl_string("ab12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 1, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 1, 1, gl_string("ab2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 1, 9, gl_string("ab234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 1, 18, gl_string("ab234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 1, 19, gl_string("ab2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 1, 20, gl_string("ab2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 10, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 10, 1, gl_string("ab1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 10, 5, gl_string("ab12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 10, 9, gl_string("ab123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 10, 10, gl_string("ab1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 10, 11, gl_string("ab1234567890"));
  }


  void test_string_replace_size_size_string_size_size14()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 19, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 19, 1, gl_string("ab0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 19, 2, gl_string("ab0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 20, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 20, 1, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 3, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string(""), 0, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string(""), 0, 1, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 0, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 0, 1, gl_string("ab1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 0, 2, gl_string("ab12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 0, 4, gl_string("ab1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 0, 5, gl_string("ab12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 0, 6, gl_string("ab12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 1, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 1, 1, gl_string("ab2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 1, 2, gl_string("ab23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 1, 3, gl_string("ab234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 1, 4, gl_string("ab2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 1, 5, gl_string("ab2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 2, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 2, 1, gl_string("ab3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 2, 2, gl_string("ab34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 2, 3, gl_string("ab345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 2, 4, gl_string("ab345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 4, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 4, 1, gl_string("ab5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 4, 2, gl_string("ab5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 5, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 5, 1, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 0, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 0, 1, gl_string("ab1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 0, 5, gl_string("ab12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 0, 9, gl_string("ab123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 0, 10, gl_string("ab1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 0, 11, gl_string("ab1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 1, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 1, 1, gl_string("ab2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 1, 4, gl_string("ab2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 1, 8, gl_string("ab23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 1, 9, gl_string("ab234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 1, 10, gl_string("ab234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 5, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 5, 1, gl_string("ab6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 5, 2, gl_string("ab67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 5, 4, gl_string("ab6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 5, 5, gl_string("ab67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 5, 6, gl_string("ab67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 9, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 9, 1, gl_string("ab0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 9, 2, gl_string("ab0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 10, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 10, 1, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 0, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 0, 1, gl_string("ab1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 0, 10, gl_string("ab1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 0, 19, gl_string("ab1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 0, 20, gl_string("ab12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 0, 21, gl_string("ab12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 1, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 1, 1, gl_string("ab2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 1, 9, gl_string("ab234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 1, 18, gl_string("ab234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 1, 19, gl_string("ab2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 1, 20, gl_string("ab2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 10, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 10, 1, gl_string("ab1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 10, 5, gl_string("ab12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 10, 9, gl_string("ab123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 10, 10, gl_string("ab1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 10, 11, gl_string("ab1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 19, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 19, 1, gl_string("ab0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 19, 2, gl_string("ab0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 20, 0, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 20, 1, gl_string("ab"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 2, 4, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 0, 1, gl_string("abcd1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 0, 2, gl_string("abcd12e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 0, 4, gl_string("abcd1234e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 0, 5, gl_string("abcd12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 0, 6, gl_string("abcd12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 1, 1, gl_string("abcd2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 1, 2, gl_string("abcd23e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 1, 3, gl_string("abcd234e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 1, 4, gl_string("abcd2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 1, 5, gl_string("abcd2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 2, 1, gl_string("abcd3e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 2, 2, gl_string("abcd34e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 2, 3, gl_string("abcd345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 2, 4, gl_string("abcd345e"));
  }


  void test_string_replace_size_size_string_size_size15()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 4, 1, gl_string("abcd5e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 4, 2, gl_string("abcd5e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 0, 1, gl_string("abcd1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 0, 5, gl_string("abcd12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 0, 9, gl_string("abcd123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 0, 10, gl_string("abcd1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 0, 11, gl_string("abcd1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 1, 1, gl_string("abcd2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 1, 4, gl_string("abcd2345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 1, 8, gl_string("abcd23456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 1, 9, gl_string("abcd234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 1, 10, gl_string("abcd234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 5, 1, gl_string("abcd6e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 5, 2, gl_string("abcd67e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 5, 4, gl_string("abcd6789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 5, 5, gl_string("abcd67890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 5, 6, gl_string("abcd67890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 9, 1, gl_string("abcd0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 9, 2, gl_string("abcd0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 0, 1, gl_string("abcd1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 0, 10, gl_string("abcd1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 0, 19, gl_string("abcd1234567890123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 0, 20, gl_string("abcd12345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 0, 21, gl_string("abcd12345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 1, 1, gl_string("abcd2e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 1, 9, gl_string("abcd234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 1, 18, gl_string("abcd234567890123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 1, 19, gl_string("abcd2345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 1, 20, gl_string("abcd2345678901234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 10, 1, gl_string("abcd1e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 10, 5, gl_string("abcd12345e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 10, 9, gl_string("abcd123456789e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 10, 10, gl_string("abcd1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 10, 11, gl_string("abcd1234567890e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 19, 1, gl_string("abcd0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 19, 2, gl_string("abcd0e"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string(""), 0, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string(""), 0, 1, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 0, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 0, 1, gl_string("abcd1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 0, 2, gl_string("abcd12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 0, 4, gl_string("abcd1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 0, 5, gl_string("abcd12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 0, 6, gl_string("abcd12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 1, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 1, 1, gl_string("abcd2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 1, 2, gl_string("abcd23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 1, 3, gl_string("abcd234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 1, 4, gl_string("abcd2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 1, 5, gl_string("abcd2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 2, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 2, 1, gl_string("abcd3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 2, 2, gl_string("abcd34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 2, 3, gl_string("abcd345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 2, 4, gl_string("abcd345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 4, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 4, 1, gl_string("abcd5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 4, 2, gl_string("abcd5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 5, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 5, 1, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 0, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 0, 1, gl_string("abcd1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 0, 5, gl_string("abcd12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 0, 9, gl_string("abcd123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 0, 10, gl_string("abcd1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 0, 11, gl_string("abcd1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 1, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 1, 1, gl_string("abcd2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 1, 4, gl_string("abcd2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 1, 8, gl_string("abcd23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 1, 9, gl_string("abcd234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 1, 10, gl_string("abcd234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 5, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 5, 1, gl_string("abcd6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 5, 2, gl_string("abcd67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 5, 4, gl_string("abcd6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 5, 5, gl_string("abcd67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 5, 6, gl_string("abcd67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 9, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 9, 1, gl_string("abcd0"));
  }


  void test_string_replace_size_size_string_size_size16()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 9, 2, gl_string("abcd0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 10, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 10, 1, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 0, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 0, 1, gl_string("abcd1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 0, 10, gl_string("abcd1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 0, 19, gl_string("abcd1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 0, 20, gl_string("abcd12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 0, 21, gl_string("abcd12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 1, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 1, 1, gl_string("abcd2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 1, 9, gl_string("abcd234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 1, 18, gl_string("abcd234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 1, 19, gl_string("abcd2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 1, 20, gl_string("abcd2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 10, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 10, 1, gl_string("abcd1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 10, 5, gl_string("abcd12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 10, 9, gl_string("abcd123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 10, 10, gl_string("abcd1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 10, 11, gl_string("abcd1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 19, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 19, 1, gl_string("abcd0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 19, 2, gl_string("abcd0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 20, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 20, 1, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string(""), 0, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string(""), 0, 1, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 0, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 0, 1, gl_string("abcd1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 0, 2, gl_string("abcd12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 0, 4, gl_string("abcd1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 0, 5, gl_string("abcd12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 0, 6, gl_string("abcd12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 1, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 1, 1, gl_string("abcd2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 1, 2, gl_string("abcd23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 1, 3, gl_string("abcd234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 1, 4, gl_string("abcd2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 1, 5, gl_string("abcd2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 2, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 2, 1, gl_string("abcd3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 2, 2, gl_string("abcd34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 2, 3, gl_string("abcd345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 2, 4, gl_string("abcd345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 4, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 4, 1, gl_string("abcd5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 4, 2, gl_string("abcd5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 5, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 5, 1, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 0, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 0, 1, gl_string("abcd1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 0, 5, gl_string("abcd12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 0, 9, gl_string("abcd123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 0, 10, gl_string("abcd1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 0, 11, gl_string("abcd1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 1, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 1, 1, gl_string("abcd2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 1, 4, gl_string("abcd2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 1, 8, gl_string("abcd23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 1, 9, gl_string("abcd234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 1, 10, gl_string("abcd234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 5, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 5, 1, gl_string("abcd6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 5, 2, gl_string("abcd67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 5, 4, gl_string("abcd6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 5, 5, gl_string("abcd67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 5, 6, gl_string("abcd67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 9, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 9, 1, gl_string("abcd0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 9, 2, gl_string("abcd0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 10, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 10, 1, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 0, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 0, 1, gl_string("abcd1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 0, 10, gl_string("abcd1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 0, 19, gl_string("abcd1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 0, 20, gl_string("abcd12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 0, 21, gl_string("abcd12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 1, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 1, 1, gl_string("abcd2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 1, 9, gl_string("abcd234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 1, 18, gl_string("abcd234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 1, 19, gl_string("abcd2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 1, 20, gl_string("abcd2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 10, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 10, 1, gl_string("abcd1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 10, 5, gl_string("abcd12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 10, 9, gl_string("abcd123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 10, 10, gl_string("abcd1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 10, 11, gl_string("abcd1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 19, 0, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 19, 1, gl_string("abcd0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 19, 2, gl_string("abcd0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 20, 0, gl_string("abcd"));
  }


  void test_string_replace_size_size_string_size_size17()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 20, 1, gl_string("abcd"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 4, 2, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 0, 2, gl_string("abcde12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 0, 4, gl_string("abcde1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 0, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 0, 6, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 1, 2, gl_string("abcde23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 1, 3, gl_string("abcde234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 1, 4, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 1, 5, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 2, 1, gl_string("abcde3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 2, 2, gl_string("abcde34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 2, 3, gl_string("abcde345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 2, 4, gl_string("abcde345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 4, 1, gl_string("abcde5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 4, 2, gl_string("abcde5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 0, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 0, 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 0, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 0, 11, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 1, 4, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 1, 8, gl_string("abcde23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 1, 9, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 1, 10, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 5, 1, gl_string("abcde6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 5, 2, gl_string("abcde67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 5, 4, gl_string("abcde6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 5, 5, gl_string("abcde67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 5, 6, gl_string("abcde67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 9, 1, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 9, 2, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 0, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 0, 19, gl_string("abcde1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 0, 20, gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 0, 21, gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 1, 9, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 1, 18, gl_string("abcde234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 1, 19, gl_string("abcde2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 1, 20, gl_string("abcde2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 10, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 10, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 10, 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 10, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 10, 11, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 19, 1, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 19, 2, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 0, 2, gl_string("abcde12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 0, 4, gl_string("abcde1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 0, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 0, 6, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 1, 2, gl_string("abcde23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 1, 3, gl_string("abcde234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 1, 4, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 1, 5, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 2, 1, gl_string("abcde3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 2, 2, gl_string("abcde34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 2, 3, gl_string("abcde345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 2, 4, gl_string("abcde345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 4, 1, gl_string("abcde5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 4, 2, gl_string("abcde5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 5, 0, gl_string("abcde"));
  }


  void test_string_replace_size_size_string_size_size18()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 0, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 0, 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 0, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 0, 11, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 1, 4, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 1, 8, gl_string("abcde23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 1, 9, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 1, 10, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 5, 1, gl_string("abcde6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 5, 2, gl_string("abcde67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 5, 4, gl_string("abcde6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 5, 5, gl_string("abcde67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 5, 6, gl_string("abcde67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 9, 1, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 9, 2, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 0, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 0, 19, gl_string("abcde1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 0, 20, gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 0, 21, gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 1, 9, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 1, 18, gl_string("abcde234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 1, 19, gl_string("abcde2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 1, 20, gl_string("abcde2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 10, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 10, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 10, 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 10, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 10, 11, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 19, 1, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 19, 2, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 5, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string(""), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string(""), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 0, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 0, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 0, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 0, 6, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 1, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 1, 3, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 1, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 1, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 2, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 2, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 2, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 2, 3, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 2, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 4, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 4, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 4, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 5, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 5, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 0, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 0, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 0, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 0, 11, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 1, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 1, 8, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 1, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 1, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 5, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 5, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 5, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 5, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 5, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 5, 6, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 9, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 9, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 9, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 10, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 10, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
  }


  void test_string_replace_size_size_string_size_size19()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 0, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 0, 19, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 0, 20, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 0, 21, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 1, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 1, 18, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 1, 19, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 1, 20, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 10, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 10, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 10, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 10, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 10, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 10, 11, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 19, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 19, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 19, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 20, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 20, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcde"), 6, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 0, 1, gl_string("1abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 0, 2, gl_string("12abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 0, 4, gl_string("1234abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 0, 5, gl_string("12345abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 0, 6, gl_string("12345abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 1, 1, gl_string("2abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 1, 2, gl_string("23abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 1, 3, gl_string("234abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 1, 4, gl_string("2345abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 1, 5, gl_string("2345abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 2, 1, gl_string("3abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 2, 2, gl_string("34abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 2, 3, gl_string("345abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 2, 4, gl_string("345abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 4, 1, gl_string("5abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 4, 2, gl_string("5abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 0, 1, gl_string("1abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 0, 5, gl_string("12345abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 0, 9, gl_string("123456789abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 0, 10, gl_string("1234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 0, 11, gl_string("1234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 1, 1, gl_string("2abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 1, 4, gl_string("2345abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 1, 8, gl_string("23456789abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 1, 9, gl_string("234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 1, 10, gl_string("234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 5, 1, gl_string("6abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 5, 2, gl_string("67abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 5, 4, gl_string("6789abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 5, 5, gl_string("67890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 5, 6, gl_string("67890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 9, 1, gl_string("0abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 9, 2, gl_string("0abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 0, 1, gl_string("1abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 1, 1, gl_string("2abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 1, 9, gl_string("234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 10, 1, gl_string("1abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 10, 5, gl_string("12345abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 10, 9, gl_string("123456789abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 19, 1, gl_string("0abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 19, 2, gl_string("0abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string(""), 0, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string(""), 0, 1, gl_string("bcdefghij"));
  }


  void test_string_replace_size_size_string_size_size20()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 0, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 0, 1, gl_string("1bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 0, 2, gl_string("12bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 0, 4, gl_string("1234bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 0, 5, gl_string("12345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 0, 6, gl_string("12345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 1, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 1, 1, gl_string("2bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 1, 2, gl_string("23bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 1, 3, gl_string("234bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 1, 4, gl_string("2345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 1, 5, gl_string("2345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 2, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 2, 1, gl_string("3bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 2, 2, gl_string("34bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 2, 3, gl_string("345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 2, 4, gl_string("345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 4, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 4, 1, gl_string("5bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 4, 2, gl_string("5bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 5, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 5, 1, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 0, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 0, 1, gl_string("1bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 0, 5, gl_string("12345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 0, 9, gl_string("123456789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 0, 10, gl_string("1234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 0, 11, gl_string("1234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 1, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 1, 1, gl_string("2bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 1, 4, gl_string("2345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 1, 8, gl_string("23456789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 1, 9, gl_string("234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 1, 10, gl_string("234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 5, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 5, 1, gl_string("6bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 5, 2, gl_string("67bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 5, 4, gl_string("6789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 5, 5, gl_string("67890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 5, 6, gl_string("67890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 9, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 9, 1, gl_string("0bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 9, 2, gl_string("0bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 10, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 10, 1, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 0, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 0, 1, gl_string("1bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 1, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 1, 1, gl_string("2bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 1, 9, gl_string("234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 10, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 10, 1, gl_string("1bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 10, 5, gl_string("12345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 10, 9, gl_string("123456789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 19, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 19, 1, gl_string("0bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 19, 2, gl_string("0bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 20, 0, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 20, 1, gl_string("bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string(""), 0, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string(""), 0, 1, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 0, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 0, 1, gl_string("1fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 0, 2, gl_string("12fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 0, 4, gl_string("1234fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 0, 5, gl_string("12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 0, 6, gl_string("12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 1, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 1, 1, gl_string("2fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 1, 2, gl_string("23fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 1, 3, gl_string("234fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 1, 4, gl_string("2345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 1, 5, gl_string("2345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 2, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 2, 1, gl_string("3fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 2, 2, gl_string("34fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 2, 3, gl_string("345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 2, 4, gl_string("345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 4, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 4, 1, gl_string("5fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 4, 2, gl_string("5fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 5, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 5, 1, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 0, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 0, 1, gl_string("1fghij"));
  }


  void test_string_replace_size_size_string_size_size21()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 0, 5, gl_string("12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 0, 9, gl_string("123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 0, 10, gl_string("1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 0, 11, gl_string("1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 1, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 1, 1, gl_string("2fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 1, 4, gl_string("2345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 1, 8, gl_string("23456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 1, 9, gl_string("234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 1, 10, gl_string("234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 5, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 5, 1, gl_string("6fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 5, 2, gl_string("67fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 5, 4, gl_string("6789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 5, 5, gl_string("67890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 5, 6, gl_string("67890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 9, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 9, 1, gl_string("0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 9, 2, gl_string("0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 10, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 10, 1, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 0, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 0, 1, gl_string("1fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 1, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 1, 1, gl_string("2fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 1, 9, gl_string("234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 10, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 10, 1, gl_string("1fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 10, 5, gl_string("12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 10, 9, gl_string("123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 19, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 19, 1, gl_string("0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 19, 2, gl_string("0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 20, 0, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 20, 1, gl_string("fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 5, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string(""), 0, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string(""), 0, 1, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 0, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 0, 1, gl_string("1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 0, 2, gl_string("12j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 0, 4, gl_string("1234j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 0, 5, gl_string("12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 0, 6, gl_string("12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 1, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 1, 1, gl_string("2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 1, 2, gl_string("23j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 1, 3, gl_string("234j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 1, 4, gl_string("2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 1, 5, gl_string("2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 2, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 2, 1, gl_string("3j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 2, 2, gl_string("34j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 2, 3, gl_string("345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 2, 4, gl_string("345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 4, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 4, 1, gl_string("5j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 4, 2, gl_string("5j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 5, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 5, 1, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 0, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 0, 1, gl_string("1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 0, 5, gl_string("12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 0, 9, gl_string("123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 0, 10, gl_string("1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 0, 11, gl_string("1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 1, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 1, 1, gl_string("2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 1, 4, gl_string("2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 1, 8, gl_string("23456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 1, 9, gl_string("234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 1, 10, gl_string("234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 5, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 5, 1, gl_string("6j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 5, 2, gl_string("67j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 5, 4, gl_string("6789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 5, 5, gl_string("67890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 5, 6, gl_string("67890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 9, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 9, 1, gl_string("0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 9, 2, gl_string("0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 10, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 10, 1, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 0, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 0, 1, gl_string("1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789j"));
  }


  void test_string_replace_size_size_string_size_size22()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 1, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 1, 1, gl_string("2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 1, 9, gl_string("234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 10, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 10, 1, gl_string("1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 10, 5, gl_string("12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 10, 9, gl_string("123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 19, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 19, 1, gl_string("0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 19, 2, gl_string("0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 20, 0, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 20, 1, gl_string("j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 9, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string(""), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string(""), 0, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 0, 2, gl_string("12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 0, 4, gl_string("1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 0, 6, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 1, 2, gl_string("23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 1, 3, gl_string("234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 1, 5, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 2, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 2, 1, gl_string("3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 2, 2, gl_string("34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 2, 3, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 2, 4, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 4, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 4, 1, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 4, 2, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 5, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 0, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 0, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 1, 8, gl_string("23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 1, 10, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 5, 1, gl_string("6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 5, 2, gl_string("67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 5, 4, gl_string("6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 5, 5, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 5, 6, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 9, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 9, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 9, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 10, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 10, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 10, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 10, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 19, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 19, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 19, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 20, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 20, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 10, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string(""), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string(""), 0, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 0, 2, gl_string("12"));
  }


  void test_string_replace_size_size_string_size_size23()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 0, 4, gl_string("1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 0, 6, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 1, 2, gl_string("23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 1, 3, gl_string("234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 1, 5, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 2, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 2, 1, gl_string("3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 2, 2, gl_string("34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 2, 3, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 2, 4, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 4, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 4, 1, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 4, 2, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 5, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 0, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 0, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 1, 8, gl_string("23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 1, 10, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 5, 1, gl_string("6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 5, 2, gl_string("67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 5, 4, gl_string("6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 5, 5, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 5, 6, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 9, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 9, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 9, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 10, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 10, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 10, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 10, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 19, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 19, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 19, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 20, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 20, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 0, 11, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 0, 1, gl_string("a1bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 0, 2, gl_string("a12bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 0, 4, gl_string("a1234bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 0, 5, gl_string("a12345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 0, 6, gl_string("a12345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 1, 1, gl_string("a2bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 1, 2, gl_string("a23bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 1, 3, gl_string("a234bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 1, 4, gl_string("a2345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 1, 5, gl_string("a2345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 2, 1, gl_string("a3bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 2, 2, gl_string("a34bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 2, 3, gl_string("a345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 2, 4, gl_string("a345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 4, 1, gl_string("a5bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 4, 2, gl_string("a5bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 0, 1, gl_string("a1bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 0, 5, gl_string("a12345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 0, 9, gl_string("a123456789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 0, 10, gl_string("a1234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 0, 11, gl_string("a1234567890bcdefghij"));
  }


  void test_string_replace_size_size_string_size_size24()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 1, 1, gl_string("a2bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 1, 4, gl_string("a2345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 1, 8, gl_string("a23456789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 1, 9, gl_string("a234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 1, 10, gl_string("a234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 5, 1, gl_string("a6bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 5, 2, gl_string("a67bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 5, 4, gl_string("a6789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 5, 5, gl_string("a67890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 5, 6, gl_string("a67890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 9, 1, gl_string("a0bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 9, 2, gl_string("a0bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 0, 1, gl_string("a1bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 1, 1, gl_string("a2bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 10, 1, gl_string("a1bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 10, 5, gl_string("a12345bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 19, 1, gl_string("a0bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 19, 2, gl_string("a0bcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string(""), 0, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string(""), 0, 1, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 0, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 0, 1, gl_string("a1cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 0, 2, gl_string("a12cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 0, 4, gl_string("a1234cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 0, 5, gl_string("a12345cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 0, 6, gl_string("a12345cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 1, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 1, 1, gl_string("a2cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 1, 2, gl_string("a23cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 1, 3, gl_string("a234cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 1, 4, gl_string("a2345cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 1, 5, gl_string("a2345cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 2, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 2, 1, gl_string("a3cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 2, 2, gl_string("a34cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 2, 3, gl_string("a345cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 2, 4, gl_string("a345cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 4, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 4, 1, gl_string("a5cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 4, 2, gl_string("a5cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 5, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 5, 1, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 0, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 0, 1, gl_string("a1cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 0, 5, gl_string("a12345cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 0, 9, gl_string("a123456789cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 0, 10, gl_string("a1234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 0, 11, gl_string("a1234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 1, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 1, 1, gl_string("a2cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 1, 4, gl_string("a2345cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 1, 8, gl_string("a23456789cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 1, 9, gl_string("a234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 1, 10, gl_string("a234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 5, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 5, 1, gl_string("a6cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 5, 2, gl_string("a67cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 5, 4, gl_string("a6789cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 5, 5, gl_string("a67890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 5, 6, gl_string("a67890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 9, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 9, 1, gl_string("a0cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 9, 2, gl_string("a0cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 10, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 10, 1, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 0, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 0, 1, gl_string("a1cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 1, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 1, 1, gl_string("a2cdefghij"));
  }


  void test_string_replace_size_size_string_size_size25()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 10, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 10, 1, gl_string("a1cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 10, 5, gl_string("a12345cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 19, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 19, 1, gl_string("a0cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 19, 2, gl_string("a0cdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 20, 0, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 20, 1, gl_string("acdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string(""), 0, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string(""), 0, 1, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 0, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 0, 1, gl_string("a1fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 0, 2, gl_string("a12fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 0, 4, gl_string("a1234fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 0, 5, gl_string("a12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 0, 6, gl_string("a12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 1, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 1, 1, gl_string("a2fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 1, 2, gl_string("a23fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 1, 3, gl_string("a234fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 1, 4, gl_string("a2345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 1, 5, gl_string("a2345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 2, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 2, 1, gl_string("a3fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 2, 2, gl_string("a34fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 2, 3, gl_string("a345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 2, 4, gl_string("a345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 4, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 4, 1, gl_string("a5fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 4, 2, gl_string("a5fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 5, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 5, 1, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 0, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 0, 1, gl_string("a1fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 0, 5, gl_string("a12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 0, 9, gl_string("a123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 0, 10, gl_string("a1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 0, 11, gl_string("a1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 1, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 1, 1, gl_string("a2fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 1, 4, gl_string("a2345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 1, 8, gl_string("a23456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 1, 9, gl_string("a234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 1, 10, gl_string("a234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 5, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 5, 1, gl_string("a6fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 5, 2, gl_string("a67fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 5, 4, gl_string("a6789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 5, 5, gl_string("a67890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 5, 6, gl_string("a67890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 9, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 9, 1, gl_string("a0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 9, 2, gl_string("a0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 10, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 10, 1, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 0, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 0, 1, gl_string("a1fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 1, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 1, 1, gl_string("a2fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 10, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 10, 1, gl_string("a1fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 10, 5, gl_string("a12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 19, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 19, 1, gl_string("a0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 19, 2, gl_string("a0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 20, 0, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 20, 1, gl_string("afghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 4, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string(""), 0, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string(""), 0, 1, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 0, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 0, 1, gl_string("a1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 0, 2, gl_string("a12j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 0, 4, gl_string("a1234j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 0, 5, gl_string("a12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 0, 6, gl_string("a12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 1, 0, gl_string("aj"));
  }


  void test_string_replace_size_size_string_size_size26()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 1, 1, gl_string("a2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 1, 2, gl_string("a23j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 1, 3, gl_string("a234j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 1, 4, gl_string("a2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 1, 5, gl_string("a2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 2, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 2, 1, gl_string("a3j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 2, 2, gl_string("a34j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 2, 3, gl_string("a345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 2, 4, gl_string("a345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 4, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 4, 1, gl_string("a5j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 4, 2, gl_string("a5j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 5, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 5, 1, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 0, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 0, 1, gl_string("a1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 0, 5, gl_string("a12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 0, 9, gl_string("a123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 0, 10, gl_string("a1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 0, 11, gl_string("a1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 1, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 1, 1, gl_string("a2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 1, 4, gl_string("a2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 1, 8, gl_string("a23456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 1, 9, gl_string("a234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 1, 10, gl_string("a234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 5, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 5, 1, gl_string("a6j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 5, 2, gl_string("a67j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 5, 4, gl_string("a6789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 5, 5, gl_string("a67890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 5, 6, gl_string("a67890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 9, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 9, 1, gl_string("a0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 9, 2, gl_string("a0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 10, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 10, 1, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 0, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 0, 1, gl_string("a1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 1, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 1, 1, gl_string("a2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 10, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 10, 1, gl_string("a1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 10, 5, gl_string("a12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 19, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 19, 1, gl_string("a0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 19, 2, gl_string("a0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 20, 0, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 20, 1, gl_string("aj"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 8, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string(""), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string(""), 0, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 0, 2, gl_string("a12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 0, 4, gl_string("a1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 0, 6, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 1, 2, gl_string("a23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 1, 3, gl_string("a234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 1, 5, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 2, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 2, 1, gl_string("a3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 2, 2, gl_string("a34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 2, 3, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 2, 4, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 4, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 4, 1, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 4, 2, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 5, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 0, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 0, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 1, 8, gl_string("a23456789"));
  }


  void test_string_replace_size_size_string_size_size27()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 1, 10, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 5, 1, gl_string("a6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 5, 2, gl_string("a67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 5, 4, gl_string("a6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 5, 5, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 5, 6, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 9, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 9, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 9, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 10, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 10, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 10, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 19, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 19, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 19, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 20, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 20, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 9, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string(""), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string(""), 0, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 0, 2, gl_string("a12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 0, 4, gl_string("a1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 0, 6, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 1, 2, gl_string("a23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 1, 3, gl_string("a234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 1, 5, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 2, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 2, 1, gl_string("a3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 2, 2, gl_string("a34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 2, 3, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 2, 4, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 4, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 4, 1, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 4, 2, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 5, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 0, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 0, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 1, 8, gl_string("a23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 1, 10, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 5, 1, gl_string("a6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 5, 2, gl_string("a67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 5, 4, gl_string("a6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 5, 5, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 5, 6, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 9, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 9, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 9, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 10, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890"));
  }


  void test_string_replace_size_size_string_size_size28()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 10, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 10, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 19, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 19, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 19, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 20, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 20, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 1, 10, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 0, 1, gl_string("abcde1fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 0, 2, gl_string("abcde12fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 0, 4, gl_string("abcde1234fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 0, 5, gl_string("abcde12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 0, 6, gl_string("abcde12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 1, 1, gl_string("abcde2fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 1, 2, gl_string("abcde23fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 1, 3, gl_string("abcde234fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 1, 4, gl_string("abcde2345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 1, 5, gl_string("abcde2345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 2, 1, gl_string("abcde3fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 2, 2, gl_string("abcde34fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 2, 3, gl_string("abcde345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 2, 4, gl_string("abcde345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 4, 1, gl_string("abcde5fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 4, 2, gl_string("abcde5fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 0, 1, gl_string("abcde1fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 0, 5, gl_string("abcde12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 0, 9, gl_string("abcde123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 0, 10, gl_string("abcde1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 0, 11, gl_string("abcde1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 1, 1, gl_string("abcde2fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 1, 4, gl_string("abcde2345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 1, 8, gl_string("abcde23456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 1, 9, gl_string("abcde234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 1, 10, gl_string("abcde234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 5, 1, gl_string("abcde6fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 5, 2, gl_string("abcde67fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 5, 4, gl_string("abcde6789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 5, 5, gl_string("abcde67890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 5, 6, gl_string("abcde67890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 9, 1, gl_string("abcde0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 9, 2, gl_string("abcde0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 0, 1, gl_string("abcde1fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 0, 10, gl_string("abcde1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 0, 19, gl_string("abcde1234567890123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 0, 20, gl_string("abcde12345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 0, 21, gl_string("abcde12345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 1, 1, gl_string("abcde2fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 1, 9, gl_string("abcde234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 1, 18, gl_string("abcde234567890123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 1, 19, gl_string("abcde2345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 1, 20, gl_string("abcde2345678901234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 10, 1, gl_string("abcde1fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 10, 5, gl_string("abcde12345fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 10, 9, gl_string("abcde123456789fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 10, 10, gl_string("abcde1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 10, 11, gl_string("abcde1234567890fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 19, 1, gl_string("abcde0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 19, 2, gl_string("abcde0fghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string(""), 0, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string(""), 0, 1, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 0, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 0, 1, gl_string("abcde1ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 0, 2, gl_string("abcde12ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 0, 4, gl_string("abcde1234ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 0, 5, gl_string("abcde12345ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 0, 6, gl_string("abcde12345ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 1, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 1, 1, gl_string("abcde2ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 1, 2, gl_string("abcde23ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 1, 3, gl_string("abcde234ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 1, 4, gl_string("abcde2345ghij"));
  }


  void test_string_replace_size_size_string_size_size29()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 1, 5, gl_string("abcde2345ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 2, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 2, 1, gl_string("abcde3ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 2, 2, gl_string("abcde34ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 2, 3, gl_string("abcde345ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 2, 4, gl_string("abcde345ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 4, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 4, 1, gl_string("abcde5ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 4, 2, gl_string("abcde5ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 5, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 5, 1, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 0, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 0, 1, gl_string("abcde1ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 0, 5, gl_string("abcde12345ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 0, 9, gl_string("abcde123456789ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 0, 10, gl_string("abcde1234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 0, 11, gl_string("abcde1234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 1, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 1, 1, gl_string("abcde2ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 1, 4, gl_string("abcde2345ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 1, 8, gl_string("abcde23456789ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 1, 9, gl_string("abcde234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 1, 10, gl_string("abcde234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 5, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 5, 1, gl_string("abcde6ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 5, 2, gl_string("abcde67ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 5, 4, gl_string("abcde6789ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 5, 5, gl_string("abcde67890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 5, 6, gl_string("abcde67890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 9, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 9, 1, gl_string("abcde0ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 9, 2, gl_string("abcde0ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 10, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 10, 1, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 0, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 0, 1, gl_string("abcde1ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 0, 10, gl_string("abcde1234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 0, 19, gl_string("abcde1234567890123456789ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 0, 20, gl_string("abcde12345678901234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 0, 21, gl_string("abcde12345678901234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 1, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 1, 1, gl_string("abcde2ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 1, 9, gl_string("abcde234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 1, 18, gl_string("abcde234567890123456789ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 1, 19, gl_string("abcde2345678901234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 1, 20, gl_string("abcde2345678901234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 10, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 10, 1, gl_string("abcde1ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 10, 5, gl_string("abcde12345ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 10, 9, gl_string("abcde123456789ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 10, 10, gl_string("abcde1234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 10, 11, gl_string("abcde1234567890ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 19, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 19, 1, gl_string("abcde0ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 19, 2, gl_string("abcde0ghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 20, 0, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 20, 1, gl_string("abcdeghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string(""), 0, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string(""), 0, 1, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 0, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 0, 1, gl_string("abcde1hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 0, 2, gl_string("abcde12hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 0, 4, gl_string("abcde1234hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 0, 5, gl_string("abcde12345hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 0, 6, gl_string("abcde12345hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 1, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 1, 1, gl_string("abcde2hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 1, 2, gl_string("abcde23hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 1, 3, gl_string("abcde234hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 1, 4, gl_string("abcde2345hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 1, 5, gl_string("abcde2345hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 2, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 2, 1, gl_string("abcde3hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 2, 2, gl_string("abcde34hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 2, 3, gl_string("abcde345hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 2, 4, gl_string("abcde345hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 4, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 4, 1, gl_string("abcde5hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 4, 2, gl_string("abcde5hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 5, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 5, 1, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 0, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 0, 1, gl_string("abcde1hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 0, 5, gl_string("abcde12345hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 0, 9, gl_string("abcde123456789hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 0, 10, gl_string("abcde1234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 0, 11, gl_string("abcde1234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 1, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 1, 1, gl_string("abcde2hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 1, 4, gl_string("abcde2345hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 1, 8, gl_string("abcde23456789hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 1, 9, gl_string("abcde234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 1, 10, gl_string("abcde234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 5, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 5, 1, gl_string("abcde6hij"));
  }


  void test_string_replace_size_size_string_size_size30()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 5, 2, gl_string("abcde67hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 5, 4, gl_string("abcde6789hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 5, 5, gl_string("abcde67890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 5, 6, gl_string("abcde67890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 9, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 9, 1, gl_string("abcde0hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 9, 2, gl_string("abcde0hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 10, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 10, 1, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 0, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 0, 1, gl_string("abcde1hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 0, 10, gl_string("abcde1234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 0, 19, gl_string("abcde1234567890123456789hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 0, 20, gl_string("abcde12345678901234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 0, 21, gl_string("abcde12345678901234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 1, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 1, 1, gl_string("abcde2hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 1, 9, gl_string("abcde234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 1, 18, gl_string("abcde234567890123456789hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 1, 19, gl_string("abcde2345678901234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 1, 20, gl_string("abcde2345678901234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 10, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 10, 1, gl_string("abcde1hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 10, 5, gl_string("abcde12345hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 10, 9, gl_string("abcde123456789hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 10, 10, gl_string("abcde1234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 10, 11, gl_string("abcde1234567890hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 19, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 19, 1, gl_string("abcde0hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 19, 2, gl_string("abcde0hij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 20, 0, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 20, 1, gl_string("abcdehij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 2, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string(""), 0, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string(""), 0, 1, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 0, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 0, 1, gl_string("abcde1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 0, 2, gl_string("abcde12j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 0, 4, gl_string("abcde1234j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 0, 5, gl_string("abcde12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 0, 6, gl_string("abcde12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 1, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 1, 1, gl_string("abcde2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 1, 2, gl_string("abcde23j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 1, 3, gl_string("abcde234j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 1, 4, gl_string("abcde2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 1, 5, gl_string("abcde2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 2, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 2, 1, gl_string("abcde3j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 2, 2, gl_string("abcde34j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 2, 3, gl_string("abcde345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 2, 4, gl_string("abcde345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 4, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 4, 1, gl_string("abcde5j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 4, 2, gl_string("abcde5j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 5, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 5, 1, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 0, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 0, 1, gl_string("abcde1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 0, 5, gl_string("abcde12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 0, 9, gl_string("abcde123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 0, 10, gl_string("abcde1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 0, 11, gl_string("abcde1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 1, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 1, 1, gl_string("abcde2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 1, 4, gl_string("abcde2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 1, 8, gl_string("abcde23456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 1, 9, gl_string("abcde234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 1, 10, gl_string("abcde234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 5, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 5, 1, gl_string("abcde6j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 5, 2, gl_string("abcde67j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 5, 4, gl_string("abcde6789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 5, 5, gl_string("abcde67890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 5, 6, gl_string("abcde67890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 9, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 9, 1, gl_string("abcde0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 9, 2, gl_string("abcde0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 10, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 10, 1, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 0, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 0, 1, gl_string("abcde1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 0, 10, gl_string("abcde1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 0, 19, gl_string("abcde1234567890123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 0, 20, gl_string("abcde12345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 0, 21, gl_string("abcde12345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 1, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 1, 1, gl_string("abcde2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 1, 9, gl_string("abcde234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 1, 18, gl_string("abcde234567890123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 1, 19, gl_string("abcde2345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 1, 20, gl_string("abcde2345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 10, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 10, 1, gl_string("abcde1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 10, 5, gl_string("abcde12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 10, 9, gl_string("abcde123456789j"));
  }


  void test_string_replace_size_size_string_size_size31()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 10, 10, gl_string("abcde1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 10, 11, gl_string("abcde1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 19, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 19, 1, gl_string("abcde0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 19, 2, gl_string("abcde0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 20, 0, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 20, 1, gl_string("abcdej"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 4, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 0, 2, gl_string("abcde12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 0, 4, gl_string("abcde1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 0, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 0, 6, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 1, 2, gl_string("abcde23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 1, 3, gl_string("abcde234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 1, 4, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 1, 5, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 2, 1, gl_string("abcde3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 2, 2, gl_string("abcde34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 2, 3, gl_string("abcde345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 2, 4, gl_string("abcde345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 4, 1, gl_string("abcde5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 4, 2, gl_string("abcde5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 0, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 0, 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 0, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 0, 11, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 1, 4, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 1, 8, gl_string("abcde23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 1, 9, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 1, 10, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 5, 1, gl_string("abcde6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 5, 2, gl_string("abcde67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 5, 4, gl_string("abcde6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 5, 5, gl_string("abcde67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 5, 6, gl_string("abcde67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 9, 1, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 9, 2, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 0, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 0, 19, gl_string("abcde1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 0, 20, gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 0, 21, gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 1, 9, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 1, 18, gl_string("abcde234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 1, 19, gl_string("abcde2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 1, 20, gl_string("abcde2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 10, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 10, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 10, 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 10, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 10, 11, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 19, 1, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 19, 2, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 5, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string(""), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string(""), 0, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 0, 2, gl_string("abcde12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 0, 4, gl_string("abcde1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 0, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 0, 6, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 1, 2, gl_string("abcde23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 1, 3, gl_string("abcde234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 1, 4, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 1, 5, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 2, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 2, 1, gl_string("abcde3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 2, 2, gl_string("abcde34"));
  }


  void test_string_replace_size_size_string_size_size32()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 2, 3, gl_string("abcde345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 2, 4, gl_string("abcde345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 4, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 4, 1, gl_string("abcde5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 4, 2, gl_string("abcde5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 5, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 0, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 0, 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 0, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 0, 11, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 1, 4, gl_string("abcde2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 1, 8, gl_string("abcde23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 1, 9, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 1, 10, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 5, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 5, 1, gl_string("abcde6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 5, 2, gl_string("abcde67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 5, 4, gl_string("abcde6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 5, 5, gl_string("abcde67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 5, 6, gl_string("abcde67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 9, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 9, 1, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 9, 2, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 10, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 0, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 0, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 0, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 0, 19, gl_string("abcde1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 0, 20, gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 0, 21, gl_string("abcde12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 1, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 1, 1, gl_string("abcde2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 1, 9, gl_string("abcde234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 1, 18, gl_string("abcde234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 1, 19, gl_string("abcde2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 1, 20, gl_string("abcde2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 10, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 10, 1, gl_string("abcde1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 10, 5, gl_string("abcde12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 10, 9, gl_string("abcde123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 10, 10, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 10, 11, gl_string("abcde1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 19, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 19, 1, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 19, 2, gl_string("abcde0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 20, 0, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 20, 1, gl_string("abcde"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 5, 6, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 0, 1, gl_string("abcdefghi1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 0, 2, gl_string("abcdefghi12j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 0, 4, gl_string("abcdefghi1234j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 0, 5, gl_string("abcdefghi12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 0, 6, gl_string("abcdefghi12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 1, 1, gl_string("abcdefghi2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 1, 2, gl_string("abcdefghi23j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 1, 3, gl_string("abcdefghi234j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 1, 4, gl_string("abcdefghi2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 1, 5, gl_string("abcdefghi2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 2, 1, gl_string("abcdefghi3j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 2, 2, gl_string("abcdefghi34j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 2, 3, gl_string("abcdefghi345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 2, 4, gl_string("abcdefghi345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 4, 1, gl_string("abcdefghi5j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 4, 2, gl_string("abcdefghi5j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 0, 1, gl_string("abcdefghi1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 0, 5, gl_string("abcdefghi12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 0, 9, gl_string("abcdefghi123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 0, 10, gl_string("abcdefghi1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 0, 11, gl_string("abcdefghi1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 1, 1, gl_string("abcdefghi2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 1, 4, gl_string("abcdefghi2345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 1, 8, gl_string("abcdefghi23456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 1, 9, gl_string("abcdefghi234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 1, 10, gl_string("abcdefghi234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 5, 1, gl_string("abcdefghi6j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 5, 2, gl_string("abcdefghi67j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 5, 4, gl_string("abcdefghi6789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 5, 5, gl_string("abcdefghi67890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 5, 6, gl_string("abcdefghi67890j"));
  }


  void test_string_replace_size_size_string_size_size33()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 9, 1, gl_string("abcdefghi0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 9, 2, gl_string("abcdefghi0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghi1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghi1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghi1234567890123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghi12345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghi12345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghi2j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghi234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghi234567890123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghi2345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghi2345678901234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghi1j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghi12345j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghi123456789j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghi1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghi1234567890j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghi0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghi0j"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string(""), 0, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string(""), 0, 1, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 0, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 0, 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 0, 2, gl_string("abcdefghi12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 0, 4, gl_string("abcdefghi1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 0, 5, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 0, 6, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 1, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 1, 1, gl_string("abcdefghi2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 1, 2, gl_string("abcdefghi23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 1, 3, gl_string("abcdefghi234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 1, 4, gl_string("abcdefghi2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 1, 5, gl_string("abcdefghi2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 2, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 2, 1, gl_string("abcdefghi3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 2, 2, gl_string("abcdefghi34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 2, 3, gl_string("abcdefghi345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 2, 4, gl_string("abcdefghi345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 4, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 4, 1, gl_string("abcdefghi5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 4, 2, gl_string("abcdefghi5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 5, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 5, 1, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 0, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 0, 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 0, 5, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 0, 9, gl_string("abcdefghi123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 0, 10, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 0, 11, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 1, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 1, 1, gl_string("abcdefghi2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 1, 4, gl_string("abcdefghi2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 1, 8, gl_string("abcdefghi23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 1, 9, gl_string("abcdefghi234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 1, 10, gl_string("abcdefghi234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 5, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 5, 1, gl_string("abcdefghi6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 5, 2, gl_string("abcdefghi67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 5, 4, gl_string("abcdefghi6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 5, 5, gl_string("abcdefghi67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 5, 6, gl_string("abcdefghi67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 9, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 9, 1, gl_string("abcdefghi0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 9, 2, gl_string("abcdefghi0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 10, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 10, 1, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghi1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghi2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghi234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghi234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghi2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghi2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghi123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghi0"));
  }


  void test_string_replace_size_size_string_size_size34()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghi0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string(""), 0, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string(""), 0, 1, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 0, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 0, 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 0, 2, gl_string("abcdefghi12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 0, 4, gl_string("abcdefghi1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 0, 5, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 0, 6, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 1, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 1, 1, gl_string("abcdefghi2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 1, 2, gl_string("abcdefghi23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 1, 3, gl_string("abcdefghi234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 1, 4, gl_string("abcdefghi2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 1, 5, gl_string("abcdefghi2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 2, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 2, 1, gl_string("abcdefghi3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 2, 2, gl_string("abcdefghi34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 2, 3, gl_string("abcdefghi345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 2, 4, gl_string("abcdefghi345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 4, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 4, 1, gl_string("abcdefghi5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 4, 2, gl_string("abcdefghi5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 5, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 5, 1, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 0, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 0, 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 0, 5, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 0, 9, gl_string("abcdefghi123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 0, 10, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 0, 11, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 1, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 1, 1, gl_string("abcdefghi2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 1, 4, gl_string("abcdefghi2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 1, 8, gl_string("abcdefghi23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 1, 9, gl_string("abcdefghi234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 1, 10, gl_string("abcdefghi234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 5, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 5, 1, gl_string("abcdefghi6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 5, 2, gl_string("abcdefghi67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 5, 4, gl_string("abcdefghi6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 5, 5, gl_string("abcdefghi67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 5, 6, gl_string("abcdefghi67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 9, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 9, 1, gl_string("abcdefghi0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 9, 2, gl_string("abcdefghi0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 10, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 10, 1, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghi1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghi12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghi2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghi234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghi234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghi2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghi2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghi1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghi12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghi123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghi1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghi0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghi0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghi"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 9, 2, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 0, 2, gl_string("abcdefghij12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 0, 4, gl_string("abcdefghij1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 0, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 0, 6, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 1, 2, gl_string("abcdefghij23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 1, 3, gl_string("abcdefghij234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 1, 4, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 1, 5, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 2, 1, gl_string("abcdefghij3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 2, 2, gl_string("abcdefghij34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 2, 3, gl_string("abcdefghij345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 2, 4, gl_string("abcdefghij345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 4, 1, gl_string("abcdefghij5"));
  }


  void test_string_replace_size_size_string_size_size35()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 4, 2, gl_string("abcdefghij5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 0, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 0, 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 0, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 0, 11, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 1, 4, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 1, 8, gl_string("abcdefghij23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 1, 9, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 1, 10, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 5, 1, gl_string("abcdefghij6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 5, 2, gl_string("abcdefghij67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 5, 4, gl_string("abcdefghij6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 5, 5, gl_string("abcdefghij67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 5, 6, gl_string("abcdefghij67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 9, 1, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 9, 2, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghij1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghij234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghij2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghij2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 0, 2, gl_string("abcdefghij12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 0, 4, gl_string("abcdefghij1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 0, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 0, 6, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 1, 2, gl_string("abcdefghij23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 1, 3, gl_string("abcdefghij234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 1, 4, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 1, 5, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 2, 1, gl_string("abcdefghij3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 2, 2, gl_string("abcdefghij34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 2, 3, gl_string("abcdefghij345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 2, 4, gl_string("abcdefghij345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 4, 1, gl_string("abcdefghij5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 4, 2, gl_string("abcdefghij5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 0, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 0, 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 0, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 0, 11, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 1, 4, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 1, 8, gl_string("abcdefghij23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 1, 9, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 1, 10, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 5, 1, gl_string("abcdefghij6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 5, 2, gl_string("abcdefghij67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 5, 4, gl_string("abcdefghij6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 5, 5, gl_string("abcdefghij67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 5, 6, gl_string("abcdefghij67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 9, 1, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 9, 2, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
  }


  void test_string_replace_size_size_string_size_size36()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghij1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghij234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghij2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghij2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 10, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string(""), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string(""), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 0, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 0, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 0, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 0, 6, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 1, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 1, 3, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 1, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 1, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 2, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 2, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 2, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 2, 3, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 2, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 4, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 4, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 4, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 5, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 5, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 0, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 0, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 0, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 0, 11, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 1, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 1, 8, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 1, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 1, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 5, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 5, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 5, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 5, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 5, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 5, 6, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 9, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 9, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 9, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 10, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 10, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 0, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 0, 19, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 0, 20, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 0, 21, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 1, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 1, 18, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 1, 19, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 1, 20, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 10, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 10, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 10, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 10, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 10, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 10, 11, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 19, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 19, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 19, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 20, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 20, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghij"), 11, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
  }


  void test_string_replace_size_size_string_size_size37()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 0, 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 0, 2, gl_string("12abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 0, 4, gl_string("1234abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 0, 5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 0, 6, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 1, 1, gl_string("2abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 1, 2, gl_string("23abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 1, 3, gl_string("234abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 1, 4, gl_string("2345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 1, 5, gl_string("2345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 2, 1, gl_string("3abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 2, 2, gl_string("34abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 2, 3, gl_string("345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 2, 4, gl_string("345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 4, 1, gl_string("5abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 4, 2, gl_string("5abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 0, 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 0, 5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 0, 9, gl_string("123456789abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 0, 10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 0, 11, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 1, 1, gl_string("2abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 1, 4, gl_string("2345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 1, 8, gl_string("23456789abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 1, 9, gl_string("234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 1, 10, gl_string("234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 5, 1, gl_string("6abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 5, 2, gl_string("67abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 5, 4, gl_string("6789abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 5, 5, gl_string("67890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 5, 6, gl_string("67890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 9, 1, gl_string("0abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 9, 2, gl_string("0abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 0, 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 1, 1, gl_string("2abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 1, 9, gl_string("234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 10, 1, gl_string("1abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 10, 5, gl_string("12345abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 10, 9, gl_string("123456789abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 19, 1, gl_string("0abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 19, 2, gl_string("0abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string(""), 0, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string(""), 0, 1, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 0, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 0, 1, gl_string("1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 0, 2, gl_string("12bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 0, 4, gl_string("1234bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 0, 5, gl_string("12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 0, 6, gl_string("12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 1, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 1, 1, gl_string("2bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 1, 2, gl_string("23bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 1, 3, gl_string("234bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 1, 4, gl_string("2345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 1, 5, gl_string("2345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 2, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 2, 1, gl_string("3bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 2, 2, gl_string("34bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 2, 3, gl_string("345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 2, 4, gl_string("345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 4, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 4, 1, gl_string("5bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 4, 2, gl_string("5bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 5, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 5, 1, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
  }


  void test_string_replace_size_size_string_size_size38()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 0, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 0, 1, gl_string("1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 0, 5, gl_string("12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 0, 9, gl_string("123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 0, 10, gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 0, 11, gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 1, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 1, 1, gl_string("2bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 1, 4, gl_string("2345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 1, 8, gl_string("23456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 1, 9, gl_string("234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 1, 10, gl_string("234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 5, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 5, 1, gl_string("6bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 5, 2, gl_string("67bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 5, 4, gl_string("6789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 5, 5, gl_string("67890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 5, 6, gl_string("67890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 9, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 9, 1, gl_string("0bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 9, 2, gl_string("0bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 10, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 10, 1, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 0, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 0, 1, gl_string("1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 1, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 1, 1, gl_string("2bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 1, 9, gl_string("234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 10, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 10, 1, gl_string("1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 10, 5, gl_string("12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 10, 9, gl_string("123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 19, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 19, 1, gl_string("0bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 19, 2, gl_string("0bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 20, 0, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 20, 1, gl_string("bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string(""), 0, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string(""), 0, 1, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 0, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 0, 1, gl_string("1klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 0, 2, gl_string("12klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 0, 4, gl_string("1234klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 0, 5, gl_string("12345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 0, 6, gl_string("12345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 1, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 1, 1, gl_string("2klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 1, 2, gl_string("23klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 1, 3, gl_string("234klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 1, 4, gl_string("2345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 1, 5, gl_string("2345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 2, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 2, 1, gl_string("3klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 2, 2, gl_string("34klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 2, 3, gl_string("345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 2, 4, gl_string("345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 4, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 4, 1, gl_string("5klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 4, 2, gl_string("5klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 5, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 5, 1, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 0, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 0, 1, gl_string("1klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 0, 5, gl_string("12345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 0, 9, gl_string("123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 0, 10, gl_string("1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 0, 11, gl_string("1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 1, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 1, 1, gl_string("2klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 1, 4, gl_string("2345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 1, 8, gl_string("23456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 1, 9, gl_string("234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 1, 10, gl_string("234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 5, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 5, 1, gl_string("6klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 5, 2, gl_string("67klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 5, 4, gl_string("6789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 5, 5, gl_string("67890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 5, 6, gl_string("67890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 9, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 9, 1, gl_string("0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 9, 2, gl_string("0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 10, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 10, 1, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 0, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 0, 1, gl_string("1klmnopqrst"));
  }


  void test_string_replace_size_size_string_size_size39()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 1, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 1, 1, gl_string("2klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 1, 9, gl_string("234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 10, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 10, 1, gl_string("1klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 10, 5, gl_string("12345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 10, 9, gl_string("123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 19, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 19, 1, gl_string("0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 19, 2, gl_string("0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 20, 0, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 20, 1, gl_string("klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 10, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string(""), 0, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string(""), 0, 1, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 0, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 0, 1, gl_string("1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 0, 2, gl_string("12t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 0, 4, gl_string("1234t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 0, 5, gl_string("12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 0, 6, gl_string("12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 1, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 1, 1, gl_string("2t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 1, 2, gl_string("23t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 1, 3, gl_string("234t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 1, 4, gl_string("2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 1, 5, gl_string("2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 2, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 2, 1, gl_string("3t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 2, 2, gl_string("34t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 2, 3, gl_string("345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 2, 4, gl_string("345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 4, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 4, 1, gl_string("5t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 4, 2, gl_string("5t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 5, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 5, 1, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 0, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 0, 1, gl_string("1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 0, 5, gl_string("12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 0, 9, gl_string("123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 0, 10, gl_string("1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 0, 11, gl_string("1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 1, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 1, 1, gl_string("2t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 1, 4, gl_string("2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 1, 8, gl_string("23456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 1, 9, gl_string("234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 1, 10, gl_string("234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 5, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 5, 1, gl_string("6t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 5, 2, gl_string("67t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 5, 4, gl_string("6789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 5, 5, gl_string("67890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 5, 6, gl_string("67890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 9, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 9, 1, gl_string("0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 9, 2, gl_string("0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 10, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 10, 1, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 0, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 0, 1, gl_string("1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 1, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 1, 1, gl_string("2t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 1, 9, gl_string("234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 10, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 10, 1, gl_string("1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 10, 5, gl_string("12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 10, 9, gl_string("123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 19, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 19, 1, gl_string("0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 19, 2, gl_string("0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 20, 0, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 20, 1, gl_string("t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 19, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string(""), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string(""), 0, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 0, 0, gl_string(""));
  }


  void test_string_replace_size_size_string_size_size40()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 0, 2, gl_string("12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 0, 4, gl_string("1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 0, 6, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 1, 2, gl_string("23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 1, 3, gl_string("234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 1, 5, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 2, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 2, 1, gl_string("3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 2, 2, gl_string("34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 2, 3, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 2, 4, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 4, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 4, 1, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 4, 2, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 5, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 0, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 0, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 1, 8, gl_string("23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 1, 10, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 5, 1, gl_string("6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 5, 2, gl_string("67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 5, 4, gl_string("6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 5, 5, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 5, 6, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 9, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 9, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 9, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 10, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 10, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 10, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 10, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 19, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 19, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 19, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 20, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 20, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 20, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string(""), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string(""), 0, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 0, 2, gl_string("12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 0, 4, gl_string("1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 0, 6, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 1, 2, gl_string("23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 1, 3, gl_string("234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 1, 5, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 2, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 2, 1, gl_string("3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 2, 2, gl_string("34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 2, 3, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 2, 4, gl_string("345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 4, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 4, 1, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 4, 2, gl_string("5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 5, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 0, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 0, 9, gl_string("123456789"));
  }


  void test_string_replace_size_size_string_size_size41()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 0, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 1, 4, gl_string("2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 1, 8, gl_string("23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 1, 10, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 5, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 5, 1, gl_string("6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 5, 2, gl_string("67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 5, 4, gl_string("6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 5, 5, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 5, 6, gl_string("67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 9, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 9, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 9, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 10, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 0, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 0, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 0, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 0, 19, gl_string("1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 0, 20, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 0, 21, gl_string("12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 1, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 1, 1, gl_string("2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 1, 9, gl_string("234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 1, 18, gl_string("234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 1, 19, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 1, 20, gl_string("2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 10, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 10, 1, gl_string("1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 10, 5, gl_string("12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 10, 9, gl_string("123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 10, 10, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 10, 11, gl_string("1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 19, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 19, 1, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 19, 2, gl_string("0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 20, 0, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 20, 1, gl_string(""));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 0, 21, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 0, 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 0, 2, gl_string("a12bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 0, 4, gl_string("a1234bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 0, 5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 0, 6, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 1, 1, gl_string("a2bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 1, 2, gl_string("a23bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 1, 3, gl_string("a234bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 1, 4, gl_string("a2345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 1, 5, gl_string("a2345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 2, 1, gl_string("a3bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 2, 2, gl_string("a34bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 2, 3, gl_string("a345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 2, 4, gl_string("a345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 4, 1, gl_string("a5bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 4, 2, gl_string("a5bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 0, 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 0, 5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 0, 9, gl_string("a123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 0, 10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 0, 11, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 1, 1, gl_string("a2bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 1, 4, gl_string("a2345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 1, 8, gl_string("a23456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 1, 9, gl_string("a234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 1, 10, gl_string("a234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 5, 1, gl_string("a6bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 5, 2, gl_string("a67bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 5, 4, gl_string("a6789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 5, 5, gl_string("a67890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 5, 6, gl_string("a67890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 9, 1, gl_string("a0bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 9, 2, gl_string("a0bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 0, 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890bcdefghijklmnopqrst"));
  }


  void test_string_replace_size_size_string_size_size42()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 1, 1, gl_string("a2bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 10, 1, gl_string("a1bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 10, 5, gl_string("a12345bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 19, 1, gl_string("a0bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 19, 2, gl_string("a0bcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string(""), 0, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string(""), 0, 1, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 0, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 0, 1, gl_string("a1cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 0, 2, gl_string("a12cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 0, 4, gl_string("a1234cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 0, 5, gl_string("a12345cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 0, 6, gl_string("a12345cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 1, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 1, 1, gl_string("a2cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 1, 2, gl_string("a23cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 1, 3, gl_string("a234cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 1, 4, gl_string("a2345cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 1, 5, gl_string("a2345cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 2, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 2, 1, gl_string("a3cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 2, 2, gl_string("a34cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 2, 3, gl_string("a345cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 2, 4, gl_string("a345cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 4, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 4, 1, gl_string("a5cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 4, 2, gl_string("a5cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 5, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 5, 1, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 0, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 0, 1, gl_string("a1cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 0, 5, gl_string("a12345cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 0, 9, gl_string("a123456789cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 0, 10, gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 0, 11, gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 1, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 1, 1, gl_string("a2cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 1, 4, gl_string("a2345cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 1, 8, gl_string("a23456789cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 1, 9, gl_string("a234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 1, 10, gl_string("a234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 5, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 5, 1, gl_string("a6cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 5, 2, gl_string("a67cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 5, 4, gl_string("a6789cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 5, 5, gl_string("a67890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 5, 6, gl_string("a67890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 9, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 9, 1, gl_string("a0cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 9, 2, gl_string("a0cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 10, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 10, 1, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 0, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 0, 1, gl_string("a1cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 1, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 1, 1, gl_string("a2cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 10, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 10, 1, gl_string("a1cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 10, 5, gl_string("a12345cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 19, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 19, 1, gl_string("a0cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 19, 2, gl_string("a0cdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 20, 0, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 20, 1, gl_string("acdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string(""), 0, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string(""), 0, 1, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 0, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 0, 1, gl_string("a1klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 0, 2, gl_string("a12klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 0, 4, gl_string("a1234klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 0, 5, gl_string("a12345klmnopqrst"));
  }


  void test_string_replace_size_size_string_size_size43()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 0, 6, gl_string("a12345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 1, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 1, 1, gl_string("a2klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 1, 2, gl_string("a23klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 1, 3, gl_string("a234klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 1, 4, gl_string("a2345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 1, 5, gl_string("a2345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 2, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 2, 1, gl_string("a3klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 2, 2, gl_string("a34klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 2, 3, gl_string("a345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 2, 4, gl_string("a345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 4, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 4, 1, gl_string("a5klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 4, 2, gl_string("a5klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 5, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 5, 1, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 0, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 0, 1, gl_string("a1klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 0, 5, gl_string("a12345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 0, 9, gl_string("a123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 0, 10, gl_string("a1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 0, 11, gl_string("a1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 1, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 1, 1, gl_string("a2klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 1, 4, gl_string("a2345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 1, 8, gl_string("a23456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 1, 9, gl_string("a234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 1, 10, gl_string("a234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 5, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 5, 1, gl_string("a6klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 5, 2, gl_string("a67klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 5, 4, gl_string("a6789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 5, 5, gl_string("a67890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 5, 6, gl_string("a67890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 9, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 9, 1, gl_string("a0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 9, 2, gl_string("a0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 10, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 10, 1, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 0, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 0, 1, gl_string("a1klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 1, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 1, 1, gl_string("a2klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 10, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 10, 1, gl_string("a1klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 10, 5, gl_string("a12345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 19, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 19, 1, gl_string("a0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 19, 2, gl_string("a0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 20, 0, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 20, 1, gl_string("aklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 9, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string(""), 0, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string(""), 0, 1, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 0, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 0, 1, gl_string("a1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 0, 2, gl_string("a12t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 0, 4, gl_string("a1234t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 0, 5, gl_string("a12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 0, 6, gl_string("a12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 1, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 1, 1, gl_string("a2t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 1, 2, gl_string("a23t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 1, 3, gl_string("a234t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 1, 4, gl_string("a2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 1, 5, gl_string("a2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 2, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 2, 1, gl_string("a3t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 2, 2, gl_string("a34t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 2, 3, gl_string("a345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 2, 4, gl_string("a345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 4, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 4, 1, gl_string("a5t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 4, 2, gl_string("a5t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 5, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 5, 1, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 0, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 0, 1, gl_string("a1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 0, 5, gl_string("a12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 0, 9, gl_string("a123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 0, 10, gl_string("a1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 0, 11, gl_string("a1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 1, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 1, 1, gl_string("a2t"));
  }


  void test_string_replace_size_size_string_size_size44()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 1, 4, gl_string("a2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 1, 8, gl_string("a23456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 1, 9, gl_string("a234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 1, 10, gl_string("a234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 5, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 5, 1, gl_string("a6t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 5, 2, gl_string("a67t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 5, 4, gl_string("a6789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 5, 5, gl_string("a67890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 5, 6, gl_string("a67890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 9, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 9, 1, gl_string("a0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 9, 2, gl_string("a0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 10, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 10, 1, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 0, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 0, 1, gl_string("a1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 1, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 1, 1, gl_string("a2t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 10, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 10, 1, gl_string("a1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 10, 5, gl_string("a12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 19, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 19, 1, gl_string("a0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 19, 2, gl_string("a0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 20, 0, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 20, 1, gl_string("at"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 18, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string(""), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string(""), 0, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 0, 2, gl_string("a12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 0, 4, gl_string("a1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 0, 6, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 1, 2, gl_string("a23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 1, 3, gl_string("a234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 1, 5, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 2, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 2, 1, gl_string("a3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 2, 2, gl_string("a34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 2, 3, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 2, 4, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 4, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 4, 1, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 4, 2, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 5, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 0, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 0, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 1, 8, gl_string("a23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 1, 10, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 5, 1, gl_string("a6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 5, 2, gl_string("a67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 5, 4, gl_string("a6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 5, 5, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 5, 6, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 9, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 9, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 9, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 10, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789"));
  }


  void test_string_replace_size_size_string_size_size45()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 10, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 10, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 19, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 19, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 19, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 20, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 20, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 19, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string(""), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string(""), 0, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 0, 2, gl_string("a12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 0, 4, gl_string("a1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 0, 6, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 1, 2, gl_string("a23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 1, 3, gl_string("a234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 1, 5, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 2, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 2, 1, gl_string("a3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 2, 2, gl_string("a34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 2, 3, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 2, 4, gl_string("a345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 4, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 4, 1, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 4, 2, gl_string("a5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 5, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 0, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 0, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 0, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 1, 4, gl_string("a2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 1, 8, gl_string("a23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 1, 10, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 5, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 5, 1, gl_string("a6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 5, 2, gl_string("a67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 5, 4, gl_string("a6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 5, 5, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 5, 6, gl_string("a67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 9, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 9, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 9, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 10, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 0, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 0, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 0, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 0, 19, gl_string("a1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 0, 20, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 0, 21, gl_string("a12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 1, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 1, 1, gl_string("a2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 1, 9, gl_string("a234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 1, 18, gl_string("a234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 1, 19, gl_string("a2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 1, 20, gl_string("a2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 10, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 10, 1, gl_string("a1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 10, 5, gl_string("a12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 10, 9, gl_string("a123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 10, 10, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 10, 11, gl_string("a1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 19, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 19, 1, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 19, 2, gl_string("a0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 20, 0, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 20, 1, gl_string("a"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 1, 20, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 0, 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 0, 2, gl_string("abcdefghij12klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 0, 4, gl_string("abcdefghij1234klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 0, 5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 0, 6, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 1, 1, gl_string("abcdefghij2klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 1, 2, gl_string("abcdefghij23klmnopqrst"));
  }


  void test_string_replace_size_size_string_size_size46()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 1, 3, gl_string("abcdefghij234klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 1, 4, gl_string("abcdefghij2345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 1, 5, gl_string("abcdefghij2345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 2, 1, gl_string("abcdefghij3klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 2, 2, gl_string("abcdefghij34klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 2, 3, gl_string("abcdefghij345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 2, 4, gl_string("abcdefghij345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 4, 1, gl_string("abcdefghij5klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 4, 2, gl_string("abcdefghij5klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 0, 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 0, 5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 0, 9, gl_string("abcdefghij123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 0, 10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 0, 11, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 1, 1, gl_string("abcdefghij2klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 1, 4, gl_string("abcdefghij2345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 1, 8, gl_string("abcdefghij23456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 1, 9, gl_string("abcdefghij234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 1, 10, gl_string("abcdefghij234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 5, 1, gl_string("abcdefghij6klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 5, 2, gl_string("abcdefghij67klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 5, 4, gl_string("abcdefghij6789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 5, 5, gl_string("abcdefghij67890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 5, 6, gl_string("abcdefghij67890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 9, 1, gl_string("abcdefghij0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 9, 2, gl_string("abcdefghij0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghij1234567890123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghij12345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghij2klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghij234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghij234567890123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghij2345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghij2345678901234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghij1klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghij12345klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghij123456789klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghij1234567890klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghij0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghij0klmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string(""), 0, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string(""), 0, 1, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 0, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 0, 1, gl_string("abcdefghij1lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 0, 2, gl_string("abcdefghij12lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 0, 4, gl_string("abcdefghij1234lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 0, 5, gl_string("abcdefghij12345lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 0, 6, gl_string("abcdefghij12345lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 1, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 1, 1, gl_string("abcdefghij2lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 1, 2, gl_string("abcdefghij23lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 1, 3, gl_string("abcdefghij234lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 1, 4, gl_string("abcdefghij2345lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 1, 5, gl_string("abcdefghij2345lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 2, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 2, 1, gl_string("abcdefghij3lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 2, 2, gl_string("abcdefghij34lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 2, 3, gl_string("abcdefghij345lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 2, 4, gl_string("abcdefghij345lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 4, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 4, 1, gl_string("abcdefghij5lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 4, 2, gl_string("abcdefghij5lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 5, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 5, 1, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 0, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 0, 1, gl_string("abcdefghij1lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 0, 5, gl_string("abcdefghij12345lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 0, 9, gl_string("abcdefghij123456789lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 0, 10, gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 0, 11, gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 1, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 1, 1, gl_string("abcdefghij2lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 1, 4, gl_string("abcdefghij2345lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 1, 8, gl_string("abcdefghij23456789lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 1, 9, gl_string("abcdefghij234567890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 1, 10, gl_string("abcdefghij234567890lmnopqrst"));
  }


  void test_string_replace_size_size_string_size_size47()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 5, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 5, 1, gl_string("abcdefghij6lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 5, 2, gl_string("abcdefghij67lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 5, 4, gl_string("abcdefghij6789lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 5, 5, gl_string("abcdefghij67890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 5, 6, gl_string("abcdefghij67890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 9, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 9, 1, gl_string("abcdefghij0lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 9, 2, gl_string("abcdefghij0lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 10, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 10, 1, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghij1lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghij1234567890123456789lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghij12345678901234567890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghij12345678901234567890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghij2lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghij234567890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghij234567890123456789lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghij2345678901234567890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghij2345678901234567890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghij1lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghij12345lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghij123456789lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghij1234567890lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghij0lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghij0lmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijlmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string(""), 0, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string(""), 0, 1, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 0, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 0, 1, gl_string("abcdefghij1pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 0, 2, gl_string("abcdefghij12pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 0, 4, gl_string("abcdefghij1234pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 0, 5, gl_string("abcdefghij12345pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 0, 6, gl_string("abcdefghij12345pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 1, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 1, 1, gl_string("abcdefghij2pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 1, 2, gl_string("abcdefghij23pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 1, 3, gl_string("abcdefghij234pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 1, 4, gl_string("abcdefghij2345pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 1, 5, gl_string("abcdefghij2345pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 2, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 2, 1, gl_string("abcdefghij3pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 2, 2, gl_string("abcdefghij34pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 2, 3, gl_string("abcdefghij345pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 2, 4, gl_string("abcdefghij345pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 4, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 4, 1, gl_string("abcdefghij5pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 4, 2, gl_string("abcdefghij5pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 5, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 5, 1, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 0, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 0, 1, gl_string("abcdefghij1pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 0, 5, gl_string("abcdefghij12345pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 0, 9, gl_string("abcdefghij123456789pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 0, 10, gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 0, 11, gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 1, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 1, 1, gl_string("abcdefghij2pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 1, 4, gl_string("abcdefghij2345pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 1, 8, gl_string("abcdefghij23456789pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 1, 9, gl_string("abcdefghij234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 1, 10, gl_string("abcdefghij234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 5, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 5, 1, gl_string("abcdefghij6pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 5, 2, gl_string("abcdefghij67pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 5, 4, gl_string("abcdefghij6789pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 5, 5, gl_string("abcdefghij67890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 5, 6, gl_string("abcdefghij67890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 9, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 9, 1, gl_string("abcdefghij0pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 9, 2, gl_string("abcdefghij0pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 10, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 10, 1, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghij1pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghij1234567890123456789pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghij12345678901234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghij12345678901234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghij2pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghij234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghij234567890123456789pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghij2345678901234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghij2345678901234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghij1pqrst"));
  }


  void test_string_replace_size_size_string_size_size48()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghij12345pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghij123456789pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghij1234567890pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghij0pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghij0pqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijpqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 5, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string(""), 0, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string(""), 0, 1, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 0, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 0, 1, gl_string("abcdefghij1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 0, 2, gl_string("abcdefghij12t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 0, 4, gl_string("abcdefghij1234t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 0, 5, gl_string("abcdefghij12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 0, 6, gl_string("abcdefghij12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 1, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 1, 1, gl_string("abcdefghij2t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 1, 2, gl_string("abcdefghij23t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 1, 3, gl_string("abcdefghij234t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 1, 4, gl_string("abcdefghij2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 1, 5, gl_string("abcdefghij2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 2, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 2, 1, gl_string("abcdefghij3t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 2, 2, gl_string("abcdefghij34t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 2, 3, gl_string("abcdefghij345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 2, 4, gl_string("abcdefghij345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 4, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 4, 1, gl_string("abcdefghij5t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 4, 2, gl_string("abcdefghij5t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 5, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 5, 1, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 0, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 0, 1, gl_string("abcdefghij1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 0, 5, gl_string("abcdefghij12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 0, 9, gl_string("abcdefghij123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 0, 10, gl_string("abcdefghij1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 0, 11, gl_string("abcdefghij1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 1, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 1, 1, gl_string("abcdefghij2t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 1, 4, gl_string("abcdefghij2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 1, 8, gl_string("abcdefghij23456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 1, 9, gl_string("abcdefghij234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 1, 10, gl_string("abcdefghij234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 5, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 5, 1, gl_string("abcdefghij6t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 5, 2, gl_string("abcdefghij67t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 5, 4, gl_string("abcdefghij6789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 5, 5, gl_string("abcdefghij67890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 5, 6, gl_string("abcdefghij67890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 9, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 9, 1, gl_string("abcdefghij0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 9, 2, gl_string("abcdefghij0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 10, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 10, 1, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghij1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghij1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghij1234567890123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghij12345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghij12345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghij2t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghij234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghij234567890123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghij2345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghij2345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghij1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghij12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghij123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghij1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghij1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghij0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghij0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijt"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 9, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 0, 2, gl_string("abcdefghij12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 0, 4, gl_string("abcdefghij1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 0, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 0, 6, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 1, 2, gl_string("abcdefghij23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 1, 3, gl_string("abcdefghij234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 1, 4, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 1, 5, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
  }


  void test_string_replace_size_size_string_size_size49()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 2, 1, gl_string("abcdefghij3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 2, 2, gl_string("abcdefghij34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 2, 3, gl_string("abcdefghij345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 2, 4, gl_string("abcdefghij345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 4, 1, gl_string("abcdefghij5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 4, 2, gl_string("abcdefghij5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 0, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 0, 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 0, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 0, 11, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 1, 4, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 1, 8, gl_string("abcdefghij23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 1, 9, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 1, 10, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 5, 1, gl_string("abcdefghij6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 5, 2, gl_string("abcdefghij67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 5, 4, gl_string("abcdefghij6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 5, 5, gl_string("abcdefghij67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 5, 6, gl_string("abcdefghij67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 9, 1, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 9, 2, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghij1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghij234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghij2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghij2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 10, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string(""), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string(""), 0, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 0, 2, gl_string("abcdefghij12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 0, 4, gl_string("abcdefghij1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 0, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 0, 6, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 1, 2, gl_string("abcdefghij23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 1, 3, gl_string("abcdefghij234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 1, 4, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 1, 5, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 2, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 2, 1, gl_string("abcdefghij3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 2, 2, gl_string("abcdefghij34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 2, 3, gl_string("abcdefghij345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 2, 4, gl_string("abcdefghij345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 4, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 4, 1, gl_string("abcdefghij5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 4, 2, gl_string("abcdefghij5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 5, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 0, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 0, 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 0, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 0, 11, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 1, 4, gl_string("abcdefghij2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 1, 8, gl_string("abcdefghij23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 1, 9, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 1, 10, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 5, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 5, 1, gl_string("abcdefghij6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 5, 2, gl_string("abcdefghij67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 5, 4, gl_string("abcdefghij6789"));
  }


  void test_string_replace_size_size_string_size_size50()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 5, 5, gl_string("abcdefghij67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 5, 6, gl_string("abcdefghij67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 9, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 9, 1, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 9, 2, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 10, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghij1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghij12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghij2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghij234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghij234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghij2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghij2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghij1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghij12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghij123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghij1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghij0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghij"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 10, 11, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 0, 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 0, 2, gl_string("abcdefghijklmnopqrs12t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 0, 4, gl_string("abcdefghijklmnopqrs1234t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 0, 5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 0, 6, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 1, 1, gl_string("abcdefghijklmnopqrs2t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 1, 2, gl_string("abcdefghijklmnopqrs23t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 1, 3, gl_string("abcdefghijklmnopqrs234t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 1, 4, gl_string("abcdefghijklmnopqrs2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 1, 5, gl_string("abcdefghijklmnopqrs2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 2, 1, gl_string("abcdefghijklmnopqrs3t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 2, 2, gl_string("abcdefghijklmnopqrs34t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 2, 3, gl_string("abcdefghijklmnopqrs345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 2, 4, gl_string("abcdefghijklmnopqrs345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 4, 1, gl_string("abcdefghijklmnopqrs5t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 4, 2, gl_string("abcdefghijklmnopqrs5t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 0, 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 0, 5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 0, 9, gl_string("abcdefghijklmnopqrs123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 0, 10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 0, 11, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 1, 1, gl_string("abcdefghijklmnopqrs2t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 1, 4, gl_string("abcdefghijklmnopqrs2345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 1, 8, gl_string("abcdefghijklmnopqrs23456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 1, 9, gl_string("abcdefghijklmnopqrs234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 1, 10, gl_string("abcdefghijklmnopqrs234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 5, 1, gl_string("abcdefghijklmnopqrs6t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 5, 2, gl_string("abcdefghijklmnopqrs67t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 5, 4, gl_string("abcdefghijklmnopqrs6789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 5, 5, gl_string("abcdefghijklmnopqrs67890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 5, 6, gl_string("abcdefghijklmnopqrs67890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 9, 1, gl_string("abcdefghijklmnopqrs0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 9, 2, gl_string("abcdefghijklmnopqrs0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghijklmnopqrs1234567890123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghijklmnopqrs12345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghijklmnopqrs2t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghijklmnopqrs234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghijklmnopqrs234567890123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghijklmnopqrs2345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghijklmnopqrs2345678901234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghijklmnopqrs1t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghijklmnopqrs12345t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghijklmnopqrs123456789t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghijklmnopqrs1234567890t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghijklmnopqrs1234567890t"));
  }


  void test_string_replace_size_size_string_size_size51()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghijklmnopqrs0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghijklmnopqrs0t"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 0, 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 0, 2, gl_string("abcdefghijklmnopqrs12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 0, 4, gl_string("abcdefghijklmnopqrs1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 0, 5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 0, 6, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 1, 1, gl_string("abcdefghijklmnopqrs2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 1, 2, gl_string("abcdefghijklmnopqrs23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 1, 3, gl_string("abcdefghijklmnopqrs234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 1, 4, gl_string("abcdefghijklmnopqrs2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 1, 5, gl_string("abcdefghijklmnopqrs2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 2, 1, gl_string("abcdefghijklmnopqrs3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 2, 2, gl_string("abcdefghijklmnopqrs34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 2, 3, gl_string("abcdefghijklmnopqrs345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 2, 4, gl_string("abcdefghijklmnopqrs345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 4, 1, gl_string("abcdefghijklmnopqrs5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 4, 2, gl_string("abcdefghijklmnopqrs5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 0, 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 0, 5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 0, 9, gl_string("abcdefghijklmnopqrs123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 0, 10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 0, 11, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 1, 1, gl_string("abcdefghijklmnopqrs2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 1, 4, gl_string("abcdefghijklmnopqrs2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 1, 8, gl_string("abcdefghijklmnopqrs23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 1, 9, gl_string("abcdefghijklmnopqrs234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 1, 10, gl_string("abcdefghijklmnopqrs234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 5, 1, gl_string("abcdefghijklmnopqrs6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 5, 2, gl_string("abcdefghijklmnopqrs67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 5, 4, gl_string("abcdefghijklmnopqrs6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 5, 5, gl_string("abcdefghijklmnopqrs67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 5, 6, gl_string("abcdefghijklmnopqrs67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 9, 1, gl_string("abcdefghijklmnopqrs0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 9, 2, gl_string("abcdefghijklmnopqrs0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghijklmnopqrs1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghijklmnopqrs2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghijklmnopqrs234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghijklmnopqrs234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghijklmnopqrs2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghijklmnopqrs2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghijklmnopqrs123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghijklmnopqrs0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghijklmnopqrs0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 0, 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 0, 2, gl_string("abcdefghijklmnopqrs12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 0, 4, gl_string("abcdefghijklmnopqrs1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 0, 5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 0, 6, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 1, 1, gl_string("abcdefghijklmnopqrs2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 1, 2, gl_string("abcdefghijklmnopqrs23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 1, 3, gl_string("abcdefghijklmnopqrs234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 1, 4, gl_string("abcdefghijklmnopqrs2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 1, 5, gl_string("abcdefghijklmnopqrs2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 2, 1, gl_string("abcdefghijklmnopqrs3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 2, 2, gl_string("abcdefghijklmnopqrs34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 2, 3, gl_string("abcdefghijklmnopqrs345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 2, 4, gl_string("abcdefghijklmnopqrs345"));
  }


  void test_string_replace_size_size_string_size_size52()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 4, 1, gl_string("abcdefghijklmnopqrs5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 4, 2, gl_string("abcdefghijklmnopqrs5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 0, 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 0, 5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 0, 9, gl_string("abcdefghijklmnopqrs123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 0, 10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 0, 11, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 1, 1, gl_string("abcdefghijklmnopqrs2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 1, 4, gl_string("abcdefghijklmnopqrs2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 1, 8, gl_string("abcdefghijklmnopqrs23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 1, 9, gl_string("abcdefghijklmnopqrs234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 1, 10, gl_string("abcdefghijklmnopqrs234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 5, 1, gl_string("abcdefghijklmnopqrs6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 5, 2, gl_string("abcdefghijklmnopqrs67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 5, 4, gl_string("abcdefghijklmnopqrs6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 5, 5, gl_string("abcdefghijklmnopqrs67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 5, 6, gl_string("abcdefghijklmnopqrs67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 9, 1, gl_string("abcdefghijklmnopqrs0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 9, 2, gl_string("abcdefghijklmnopqrs0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghijklmnopqrs1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghijklmnopqrs12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghijklmnopqrs2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghijklmnopqrs234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghijklmnopqrs234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghijklmnopqrs2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghijklmnopqrs2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghijklmnopqrs1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghijklmnopqrs12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghijklmnopqrs123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghijklmnopqrs1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghijklmnopqrs0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghijklmnopqrs0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrs"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 19, 2, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 0, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 0, 2, gl_string("abcdefghijklmnopqrst12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 0, 4, gl_string("abcdefghijklmnopqrst1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 0, 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 0, 6, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 1, 1, gl_string("abcdefghijklmnopqrst2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 1, 2, gl_string("abcdefghijklmnopqrst23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 1, 3, gl_string("abcdefghijklmnopqrst234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 1, 4, gl_string("abcdefghijklmnopqrst2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 1, 5, gl_string("abcdefghijklmnopqrst2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 2, 1, gl_string("abcdefghijklmnopqrst3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 2, 2, gl_string("abcdefghijklmnopqrst34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 2, 3, gl_string("abcdefghijklmnopqrst345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 2, 4, gl_string("abcdefghijklmnopqrst345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 4, 1, gl_string("abcdefghijklmnopqrst5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 4, 2, gl_string("abcdefghijklmnopqrst5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 0, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 0, 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 0, 9, gl_string("abcdefghijklmnopqrst123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 0, 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 0, 11, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 1, 1, gl_string("abcdefghijklmnopqrst2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 1, 4, gl_string("abcdefghijklmnopqrst2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 1, 8, gl_string("abcdefghijklmnopqrst23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 1, 9, gl_string("abcdefghijklmnopqrst234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 1, 10, gl_string("abcdefghijklmnopqrst234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 5, 1, gl_string("abcdefghijklmnopqrst6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 5, 2, gl_string("abcdefghijklmnopqrst67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 5, 4, gl_string("abcdefghijklmnopqrst6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 5, 5, gl_string("abcdefghijklmnopqrst67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 5, 6, gl_string("abcdefghijklmnopqrst67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 9, 1, gl_string("abcdefghijklmnopqrst0"));
  }


  void test_string_replace_size_size_string_size_size53()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 9, 2, gl_string("abcdefghijklmnopqrst0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghijklmnopqrst1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghijklmnopqrst2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghijklmnopqrst234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghijklmnopqrst234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghijklmnopqrst2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghijklmnopqrst2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghijklmnopqrst123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghijklmnopqrst0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghijklmnopqrst0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string(""), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string(""), 0, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 0, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 0, 2, gl_string("abcdefghijklmnopqrst12"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 0, 4, gl_string("abcdefghijklmnopqrst1234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 0, 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 0, 6, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 1, 1, gl_string("abcdefghijklmnopqrst2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 1, 2, gl_string("abcdefghijklmnopqrst23"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 1, 3, gl_string("abcdefghijklmnopqrst234"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 1, 4, gl_string("abcdefghijklmnopqrst2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 1, 5, gl_string("abcdefghijklmnopqrst2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 2, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 2, 1, gl_string("abcdefghijklmnopqrst3"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 2, 2, gl_string("abcdefghijklmnopqrst34"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 2, 3, gl_string("abcdefghijklmnopqrst345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 2, 4, gl_string("abcdefghijklmnopqrst345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 4, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 4, 1, gl_string("abcdefghijklmnopqrst5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 4, 2, gl_string("abcdefghijklmnopqrst5"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 5, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 0, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 0, 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 0, 9, gl_string("abcdefghijklmnopqrst123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 0, 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 0, 11, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 1, 1, gl_string("abcdefghijklmnopqrst2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 1, 4, gl_string("abcdefghijklmnopqrst2345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 1, 8, gl_string("abcdefghijklmnopqrst23456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 1, 9, gl_string("abcdefghijklmnopqrst234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 1, 10, gl_string("abcdefghijklmnopqrst234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 5, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 5, 1, gl_string("abcdefghijklmnopqrst6"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 5, 2, gl_string("abcdefghijklmnopqrst67"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 5, 4, gl_string("abcdefghijklmnopqrst6789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 5, 5, gl_string("abcdefghijklmnopqrst67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 5, 6, gl_string("abcdefghijklmnopqrst67890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 9, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 9, 1, gl_string("abcdefghijklmnopqrst0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 9, 2, gl_string("abcdefghijklmnopqrst0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 10, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 0, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 0, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 0, 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 0, 19, gl_string("abcdefghijklmnopqrst1234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 0, 20, gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 0, 21, gl_string("abcdefghijklmnopqrst12345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 1, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 1, 1, gl_string("abcdefghijklmnopqrst2"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 1, 9, gl_string("abcdefghijklmnopqrst234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 1, 18, gl_string("abcdefghijklmnopqrst234567890123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 1, 19, gl_string("abcdefghijklmnopqrst2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 1, 20, gl_string("abcdefghijklmnopqrst2345678901234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 10, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 10, 1, gl_string("abcdefghijklmnopqrst1"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 10, 5, gl_string("abcdefghijklmnopqrst12345"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 10, 9, gl_string("abcdefghijklmnopqrst123456789"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 10, 10, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 10, 11, gl_string("abcdefghijklmnopqrst1234567890"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 19, 0, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 19, 1, gl_string("abcdefghijklmnopqrst0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 19, 2, gl_string("abcdefghijklmnopqrst0"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 20, 0, gl_string("abcdefghijklmnopqrst"));
  }


  void test_string_replace_size_size_string_size_size54()
  {
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 20, 1, gl_string("abcdefghijklmnopqrst"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 20, 1, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string(""), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string(""), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string(""), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 0, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 0, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 0, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 0, 6, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 1, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 1, 3, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 1, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 1, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 2, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 2, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 2, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 2, 3, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 2, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 4, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 4, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 4, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 5, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 5, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345"), 6, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 0, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 0, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 0, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 0, 11, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 1, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 1, 8, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 1, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 1, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 5, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 5, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 5, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 5, 4, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 5, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 5, 6, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 9, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 9, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 9, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 10, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 10, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("1234567890"), 11, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 0, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 0, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 0, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 0, 19, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 0, 20, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 0, 21, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 1, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 1, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 1, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 1, 18, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 1, 19, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 1, 20, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 10, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 10, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 10, 5, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 10, 9, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 10, 10, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 10, 11, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 19, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 19, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 19, 2, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 20, 0, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 20, 1, gl_string("can't happen"));
    _test_string_replace_size_size_string_size_size(gl_string("abcdefghijklmnopqrst"), 21, 0, gl_string("12345678901234567890"), 21, 0, gl_string("can't happen"));
  }


  void test_string_replace_size_size_string_size_size55()
  {
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 10, gl_string("abcdefghi1234567890"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 19, gl_string("abcdefghi0"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 20, gl_string("abcdefghi"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 20, gl_string("abcdefghi"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 1, gl_string("12345678901234567890"), 21, gl_string("can't happen"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 2, gl_string(""), 0, gl_string("abcdefghi"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 2, gl_string(""), 1, gl_string("can't happen"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 0, gl_string("abcdefghi12345"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 1, gl_string("abcdefghi2345"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 2, gl_string("abcdefghi345"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 4, gl_string("abcdefghi5"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 5, gl_string("abcdefghi"));
    test_string_replace_size_size_string_size_size_npos(gl_string("abcdefghij"), 9, 2, gl_string("12345"), 6, gl_string("can't happen"));
  }
  
  void
  _test_string_swap_swap(gl_string s1, gl_string s2)
  {
    gl_string s1_ = s1;
    gl_string s2_ = s2;
    s1.swap(s2);
    TS_ASSERT(s1 == s2_);
    TS_ASSERT(s2 == s1_);
  }

  void test_string_swap_swap()
  {
    _test_string_swap_swap(gl_string(""), gl_string(""));
    _test_string_swap_swap(gl_string(""), gl_string("12345"));
    _test_string_swap_swap(gl_string(""), gl_string("1234567890"));
    _test_string_swap_swap(gl_string(""), gl_string("12345678901234567890"));
    _test_string_swap_swap(gl_string("abcde"), gl_string(""));
    _test_string_swap_swap(gl_string("abcde"), gl_string("12345"));
    _test_string_swap_swap(gl_string("abcde"), gl_string("1234567890"));
    _test_string_swap_swap(gl_string("abcde"), gl_string("12345678901234567890"));
    _test_string_swap_swap(gl_string("abcdefghij"), gl_string(""));
    _test_string_swap_swap(gl_string("abcdefghij"), gl_string("12345"));
    _test_string_swap_swap(gl_string("abcdefghij"), gl_string("1234567890"));
    _test_string_swap_swap(gl_string("abcdefghij"), gl_string("12345678901234567890"));
    _test_string_swap_swap(gl_string("abcdefghijklmnopqrst"), gl_string(""));
    _test_string_swap_swap(gl_string("abcdefghijklmnopqrst"), gl_string("12345"));
    _test_string_swap_swap(gl_string("abcdefghijklmnopqrst"), gl_string("1234567890"));
    _test_string_swap_swap(gl_string("abcdefghijklmnopqrst"), gl_string("12345678901234567890"));
  }
};

BOOST_FIXTURE_TEST_SUITE(_test_string_modifiers, test_string_modifiers)
BOOST_AUTO_TEST_CASE(test_basic_append) {
  test_string_modifiers::test_basic_append();
}
BOOST_AUTO_TEST_CASE(test_string_append_iterator) {
  test_string_modifiers::test_string_append_iterator();
}
BOOST_AUTO_TEST_CASE(test_string_append_pointer) {
  test_string_modifiers::test_string_append_pointer();
}
BOOST_AUTO_TEST_CASE(test_string_append_pointer_size) {
  test_string_modifiers::test_string_append_pointer_size();
}
BOOST_AUTO_TEST_CASE(test_string_append_push_back) {
  test_string_modifiers::test_string_append_push_back();
}
BOOST_AUTO_TEST_CASE(test_string_append_size_char) {
  test_string_modifiers::test_string_append_size_char();
}
BOOST_AUTO_TEST_CASE(test_string_append_string) {
  test_string_modifiers::test_string_append_string();
}
BOOST_AUTO_TEST_CASE(test_string_append_string_size_size) {
  test_string_modifiers::test_string_append_string_size_size();
}
BOOST_AUTO_TEST_CASE(test_string_assign_initializer_list) {
  test_string_modifiers::test_string_assign_initializer_list();
}
BOOST_AUTO_TEST_CASE(test_string_assign_iterator) {
  test_string_modifiers::test_string_assign_iterator();
}
BOOST_AUTO_TEST_CASE(test_string_assign_pointer) {
  test_string_modifiers::test_string_assign_pointer();
}
BOOST_AUTO_TEST_CASE(test_string_assign_pointer_size) {
  test_string_modifiers::test_string_assign_pointer_size();
}
BOOST_AUTO_TEST_CASE(test_string_assign_rv_string) {
  test_string_modifiers::test_string_assign_rv_string();
}
BOOST_AUTO_TEST_CASE(test_string_assign_size_char) {
  test_string_modifiers::test_string_assign_size_char();
}
BOOST_AUTO_TEST_CASE(test_string_assign_string) {
  test_string_modifiers::test_string_assign_string();
}
BOOST_AUTO_TEST_CASE(test_string_assign_string_size_size) {
  test_string_modifiers::test_string_assign_string_size_size();
}
BOOST_AUTO_TEST_CASE(test_string_copy_copy) {
  test_string_modifiers::test_string_copy_copy();
}
BOOST_AUTO_TEST_CASE(test_string_erase_erase_iter_db1) {
  test_string_modifiers::test_string_erase_erase_iter_db1();
}
BOOST_AUTO_TEST_CASE(test_string_erase_erase_iter_db2) {
  test_string_modifiers::test_string_erase_erase_iter_db2();
}
BOOST_AUTO_TEST_CASE(test_string_erase_erase_iter_iter_db1) {
  test_string_modifiers::test_string_erase_erase_iter_iter_db1();
}
BOOST_AUTO_TEST_CASE(test_string_erase_erase_iter_iter_db2) {
  test_string_modifiers::test_string_erase_erase_iter_iter_db2();
}
BOOST_AUTO_TEST_CASE(test_string_erase_erase_iter_iter_db3) {
  test_string_modifiers::test_string_erase_erase_iter_iter_db3();
}
BOOST_AUTO_TEST_CASE(test_string_erase_erase_iter_iter_db4) {
  test_string_modifiers::test_string_erase_erase_iter_iter_db4();
}
BOOST_AUTO_TEST_CASE(test_string_erase_iter_iter) {
  test_string_modifiers::test_string_erase_iter_iter();
}
BOOST_AUTO_TEST_CASE(test_string_erase_iter) {
  test_string_modifiers::test_string_erase_iter();
}
BOOST_AUTO_TEST_CASE(test_string_erase_pop_back) {
  test_string_modifiers::test_string_erase_pop_back();
}
BOOST_AUTO_TEST_CASE(test_string_erase_size_size) {
  test_string_modifiers::test_string_erase_size_size();
}
BOOST_AUTO_TEST_CASE(test_string_insert_iter_char) {
  test_string_modifiers::test_string_insert_iter_char();
}
BOOST_AUTO_TEST_CASE(test_string_insert_iter_initializer_list) {
  test_string_modifiers::test_string_insert_iter_initializer_list();
}
BOOST_AUTO_TEST_CASE(test_string_insert_iter_iter_iter) {
  test_string_modifiers::test_string_insert_iter_iter_iter();
}
BOOST_AUTO_TEST_CASE(test_string_insert_iter_size_char) {
  test_string_modifiers::test_string_insert_iter_size_char();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_pointer) {
  test_string_modifiers::test_string_insert_size_pointer();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_pointer_size) {
  test_string_modifiers::test_string_insert_size_pointer_size();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_size_char) {
  test_string_modifiers::test_string_insert_size_size_char();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string) {
  test_string_modifiers::test_string_insert_size_string();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size0) {
  test_string_modifiers::test_string_insert_size_string_size_size0();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size1) {
  test_string_modifiers::test_string_insert_size_string_size_size1();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size2) {
  test_string_modifiers::test_string_insert_size_string_size_size2();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size3) {
  test_string_modifiers::test_string_insert_size_string_size_size3();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size4) {
  test_string_modifiers::test_string_insert_size_string_size_size4();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size5) {
  test_string_modifiers::test_string_insert_size_string_size_size5();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size6) {
  test_string_modifiers::test_string_insert_size_string_size_size6();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size7) {
  test_string_modifiers::test_string_insert_size_string_size_size7();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size8) {
  test_string_modifiers::test_string_insert_size_string_size_size8();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size9) {
  test_string_modifiers::test_string_insert_size_string_size_size9();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size10) {
  test_string_modifiers::test_string_insert_size_string_size_size10();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size11) {
  test_string_modifiers::test_string_insert_size_string_size_size11();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size12) {
  test_string_modifiers::test_string_insert_size_string_size_size12();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size13) {
  test_string_modifiers::test_string_insert_size_string_size_size13();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size14) {
  test_string_modifiers::test_string_insert_size_string_size_size14();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size15) {
  test_string_modifiers::test_string_insert_size_string_size_size15();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size16) {
  test_string_modifiers::test_string_insert_size_string_size_size16();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size17) {
  test_string_modifiers::test_string_insert_size_string_size_size17();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size18) {
  test_string_modifiers::test_string_insert_size_string_size_size18();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size19) {
  test_string_modifiers::test_string_insert_size_string_size_size19();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size20) {
  test_string_modifiers::test_string_insert_size_string_size_size20();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size21) {
  test_string_modifiers::test_string_insert_size_string_size_size21();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size22) {
  test_string_modifiers::test_string_insert_size_string_size_size22();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size23) {
  test_string_modifiers::test_string_insert_size_string_size_size23();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size24) {
  test_string_modifiers::test_string_insert_size_string_size_size24();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size25) {
  test_string_modifiers::test_string_insert_size_string_size_size25();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size26) {
  test_string_modifiers::test_string_insert_size_string_size_size26();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size27) {
  test_string_modifiers::test_string_insert_size_string_size_size27();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size28) {
  test_string_modifiers::test_string_insert_size_string_size_size28();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size29) {
  test_string_modifiers::test_string_insert_size_string_size_size29();
}
BOOST_AUTO_TEST_CASE(test_string_insert_size_string_size_size30) {
  test_string_modifiers::test_string_insert_size_string_size_size30();
}
BOOST_AUTO_TEST_CASE(test_string_op_plus_equal_char) {
  test_string_modifiers::test_string_op_plus_equal_char();
}
BOOST_AUTO_TEST_CASE(test_string_op_plus_equal_initializer_list) {
  test_string_modifiers::test_string_op_plus_equal_initializer_list();
}
BOOST_AUTO_TEST_CASE(test_string_op_plus_equal_pointer) {
  test_string_modifiers::test_string_op_plus_equal_pointer();
}
BOOST_AUTO_TEST_CASE(test_string_op_plus_equal_string) {
  test_string_modifiers::test_string_op_plus_equal_string();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_initializer_list) {
  test_string_modifiers::test_string_replace_iter_iter_initializer_list();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_iter_iter0) {
  test_string_modifiers::test_string_replace_iter_iter_iter_iter0();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_iter_iter1) {
  test_string_modifiers::test_string_replace_iter_iter_iter_iter1();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_iter_iter2) {
  test_string_modifiers::test_string_replace_iter_iter_iter_iter2();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_iter_iter3) {
  test_string_modifiers::test_string_replace_iter_iter_iter_iter3();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_iter_iter4) {
  test_string_modifiers::test_string_replace_iter_iter_iter_iter4();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_iter_iter5) {
  test_string_modifiers::test_string_replace_iter_iter_iter_iter5();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_iter_iter6) {
  test_string_modifiers::test_string_replace_iter_iter_iter_iter6();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_iter_iter7) {
  test_string_modifiers::test_string_replace_iter_iter_iter_iter7();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_iter_iter8) {
  test_string_modifiers::test_string_replace_iter_iter_iter_iter8();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_size_char0) {
  test_string_modifiers::test_string_replace_iter_iter_size_char0();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_size_char1) {
  test_string_modifiers::test_string_replace_iter_iter_size_char1();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_size_char2) {
  test_string_modifiers::test_string_replace_iter_iter_size_char2();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_string0) {
  test_string_modifiers::test_string_replace_iter_iter_string0();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_string1) {
  test_string_modifiers::test_string_replace_iter_iter_string1();
}
BOOST_AUTO_TEST_CASE(test_string_replace_iter_iter_string2) {
  test_string_modifiers::test_string_replace_iter_iter_string2();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer0) {
  test_string_modifiers::test_string_replace_size_size_pointer0();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer1) {
  test_string_modifiers::test_string_replace_size_size_pointer1();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer2) {
  test_string_modifiers::test_string_replace_size_size_pointer2();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size0) {
  test_string_modifiers::test_string_replace_size_size_pointer_size0();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size1) {
  test_string_modifiers::test_string_replace_size_size_pointer_size1();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size2) {
  test_string_modifiers::test_string_replace_size_size_pointer_size2();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size3) {
  test_string_modifiers::test_string_replace_size_size_pointer_size3();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size4) {
  test_string_modifiers::test_string_replace_size_size_pointer_size4();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size5) {
  test_string_modifiers::test_string_replace_size_size_pointer_size5();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size6) {
  test_string_modifiers::test_string_replace_size_size_pointer_size6();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size7) {
  test_string_modifiers::test_string_replace_size_size_pointer_size7();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size8) {
  test_string_modifiers::test_string_replace_size_size_pointer_size8();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size9) {
  test_string_modifiers::test_string_replace_size_size_pointer_size9();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size10) {
  test_string_modifiers::test_string_replace_size_size_pointer_size10();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_pointer_size11) {
  test_string_modifiers::test_string_replace_size_size_pointer_size11();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_size_char0) {
  test_string_modifiers::test_string_replace_size_size_size_char0();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_size_char1) {
  test_string_modifiers::test_string_replace_size_size_size_char1();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_size_char2) {
  test_string_modifiers::test_string_replace_size_size_size_char2();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string0) {
  test_string_modifiers::test_string_replace_size_size_string0();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string1) {
  test_string_modifiers::test_string_replace_size_size_string1();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string2) {
  test_string_modifiers::test_string_replace_size_size_string2();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size0) {
  test_string_modifiers::test_string_replace_size_size_string_size_size0();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size1) {
  test_string_modifiers::test_string_replace_size_size_string_size_size1();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size2) {
  test_string_modifiers::test_string_replace_size_size_string_size_size2();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size3) {
  test_string_modifiers::test_string_replace_size_size_string_size_size3();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size4) {
  test_string_modifiers::test_string_replace_size_size_string_size_size4();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size5) {
  test_string_modifiers::test_string_replace_size_size_string_size_size5();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size6) {
  test_string_modifiers::test_string_replace_size_size_string_size_size6();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size7) {
  test_string_modifiers::test_string_replace_size_size_string_size_size7();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size8) {
  test_string_modifiers::test_string_replace_size_size_string_size_size8();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size9) {
  test_string_modifiers::test_string_replace_size_size_string_size_size9();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size10) {
  test_string_modifiers::test_string_replace_size_size_string_size_size10();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size11) {
  test_string_modifiers::test_string_replace_size_size_string_size_size11();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size12) {
  test_string_modifiers::test_string_replace_size_size_string_size_size12();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size13) {
  test_string_modifiers::test_string_replace_size_size_string_size_size13();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size14) {
  test_string_modifiers::test_string_replace_size_size_string_size_size14();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size15) {
  test_string_modifiers::test_string_replace_size_size_string_size_size15();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size16) {
  test_string_modifiers::test_string_replace_size_size_string_size_size16();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size17) {
  test_string_modifiers::test_string_replace_size_size_string_size_size17();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size18) {
  test_string_modifiers::test_string_replace_size_size_string_size_size18();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size19) {
  test_string_modifiers::test_string_replace_size_size_string_size_size19();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size20) {
  test_string_modifiers::test_string_replace_size_size_string_size_size20();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size21) {
  test_string_modifiers::test_string_replace_size_size_string_size_size21();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size22) {
  test_string_modifiers::test_string_replace_size_size_string_size_size22();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size23) {
  test_string_modifiers::test_string_replace_size_size_string_size_size23();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size24) {
  test_string_modifiers::test_string_replace_size_size_string_size_size24();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size25) {
  test_string_modifiers::test_string_replace_size_size_string_size_size25();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size26) {
  test_string_modifiers::test_string_replace_size_size_string_size_size26();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size27) {
  test_string_modifiers::test_string_replace_size_size_string_size_size27();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size28) {
  test_string_modifiers::test_string_replace_size_size_string_size_size28();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size29) {
  test_string_modifiers::test_string_replace_size_size_string_size_size29();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size30) {
  test_string_modifiers::test_string_replace_size_size_string_size_size30();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size31) {
  test_string_modifiers::test_string_replace_size_size_string_size_size31();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size32) {
  test_string_modifiers::test_string_replace_size_size_string_size_size32();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size33) {
  test_string_modifiers::test_string_replace_size_size_string_size_size33();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size34) {
  test_string_modifiers::test_string_replace_size_size_string_size_size34();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size35) {
  test_string_modifiers::test_string_replace_size_size_string_size_size35();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size36) {
  test_string_modifiers::test_string_replace_size_size_string_size_size36();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size37) {
  test_string_modifiers::test_string_replace_size_size_string_size_size37();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size38) {
  test_string_modifiers::test_string_replace_size_size_string_size_size38();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size39) {
  test_string_modifiers::test_string_replace_size_size_string_size_size39();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size40) {
  test_string_modifiers::test_string_replace_size_size_string_size_size40();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size41) {
  test_string_modifiers::test_string_replace_size_size_string_size_size41();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size42) {
  test_string_modifiers::test_string_replace_size_size_string_size_size42();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size43) {
  test_string_modifiers::test_string_replace_size_size_string_size_size43();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size44) {
  test_string_modifiers::test_string_replace_size_size_string_size_size44();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size45) {
  test_string_modifiers::test_string_replace_size_size_string_size_size45();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size46) {
  test_string_modifiers::test_string_replace_size_size_string_size_size46();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size47) {
  test_string_modifiers::test_string_replace_size_size_string_size_size47();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size48) {
  test_string_modifiers::test_string_replace_size_size_string_size_size48();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size49) {
  test_string_modifiers::test_string_replace_size_size_string_size_size49();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size50) {
  test_string_modifiers::test_string_replace_size_size_string_size_size50();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size51) {
  test_string_modifiers::test_string_replace_size_size_string_size_size51();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size52) {
  test_string_modifiers::test_string_replace_size_size_string_size_size52();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size53) {
  test_string_modifiers::test_string_replace_size_size_string_size_size53();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size54) {
  test_string_modifiers::test_string_replace_size_size_string_size_size54();
}
BOOST_AUTO_TEST_CASE(test_string_replace_size_size_string_size_size55) {
  test_string_modifiers::test_string_replace_size_size_string_size_size55();
}
BOOST_AUTO_TEST_CASE(test_string_swap_swap) {
  test_string_modifiers::test_string_swap_swap();
}
BOOST_AUTO_TEST_SUITE_END()
